@@ -14,9 +14,11 @@
 // KIND, either express or implied.  See the License for the
 // specific language governing permissions and limitations
 // under the License.
+// This file is copied from
+// https://github.com/apache/impala/blob/branch-2.9.0/be/src/exec/exec-node.h
+// and modified by Doris
 
-#ifndef DORIS_BE_SRC_QUERY_EXEC_EXEC_NODE_H
-#define DORIS_BE_SRC_QUERY_EXEC_EXEC_NODE_H
+#pragma once
 
 #include <mutex>
 #include <sstream>
@@ -26,13 +28,11 @@
 #include "gen_cpp/PlanNodes_types.h"
 #include "runtime/bufferpool/buffer_pool.h"
 #include "runtime/descriptors.h"
-#include "runtime/mem_pool.h"
 #include "runtime/query_statistics.h"
 #include "service/backend_options.h"
 #include "util/blocking_queue.hpp"
 #include "util/runtime_profile.h"
-#include "util/uid_util.h" // for print_id
-
+#include "util/telemetry/telemetry.h"
 #include "vec/exprs/vexpr_context.h"
 
 namespace doris {
@@ -73,7 +73,7 @@ public:
     /// Initializes this object from the thrift tnode desc. The subclass should
     /// do any initialization that can fail in Init() rather than the ctor.
     /// If overridden in subclass, must first call superclass's Init().
-    virtual Status init(const TPlanNode& tnode, RuntimeState* state = nullptr);
+    virtual Status init(const TPlanNode& tnode, RuntimeState* state);
 
     // Sets up internal structures, etc., without doing any actual work.
     // Must be called prior to open(). Will only be called once in this
@@ -105,6 +105,9 @@ public:
     virtual Status get_next(RuntimeState* state, RowBatch* row_batch, bool* eos);
     virtual Status get_next(RuntimeState* state, vectorized::Block* block, bool* eos);
 
+    // new interface to compatible new optimizers in FE
+    Status get_next_after_projects(RuntimeState* state, vectorized::Block* block, bool* eos);
+
     // Resets the stream of row batches to be retrieved by subsequent GetNext() calls.
     // Clears all internal state, returning this node to the state it was in after calling
     // Prepare() and before calling Open(). This function must not clear memory
@@ -143,10 +146,6 @@ public:
     static Status create_tree(RuntimeState* state, ObjectPool* pool, const TPlan& plan,
                               const DescriptorTbl& descs, ExecNode** root);
 
-    // Set debug action for node with given id in 'tree'
-    static void set_debug_options(int node_id, TExecNodePhase::type phase,
-                                  TDebugAction::type action, ExecNode* tree);
-
     // Collect all nodes of given 'node_type' that are part of this subtree, and return in
     // 'nodes'.
     void collect_nodes(TPlanNodeType::type node_type, std::vector<ExecNode*>* nodes);
@@ -183,7 +182,10 @@ public:
 
     int id() const { return _id; }
     TPlanNodeType::type type() const { return _type; }
-    virtual const RowDescriptor& row_desc() const { return _row_descriptor; }
+    virtual const RowDescriptor& row_desc() const {
+        return _output_row_descriptor ? *_output_row_descriptor : _row_descriptor;
+    }
+    virtual const RowDescriptor& intermediate_row_desc() const { return _row_descriptor; }
     int64_t rows_returned() const { return _num_rows_returned; }
     int64_t limit() const { return _limit; }
     bool reached_limit() const { return _limit != -1 && _num_rows_returned >= _limit; }
@@ -192,11 +194,11 @@ public:
     RuntimeProfile* runtime_profile() const { return _runtime_profile.get(); }
     RuntimeProfile::Counter* memory_used_counter() const { return _memory_used_counter; }
 
-    std::shared_ptr<MemTracker> mem_tracker() const { return _mem_tracker; }
+    MemTracker* mem_tracker() const { return _mem_tracker.get(); }
 
-    std::shared_ptr<MemTracker> expr_mem_tracker() const { return _expr_mem_tracker; }
+    OpentelemetrySpan get_next_span() { return _get_next_span; }
 
-    MemPool* expr_mem_pool() const { return _expr_mem_pool.get(); }
+    virtual std::string get_name();
 
     // Extract node id from p->name().
     static int get_node_id_from_profile(RuntimeProfile* p);
@@ -225,10 +227,8 @@ protected:
     // and add block rows for profile
     void reached_limit(vectorized::Block* block, bool* eos);
 
-    /// Enable the increase reservation denial probability on 'buffer_pool_client_' based on
-    /// the 'debug_action_' set on this node. Returns an error if 'debug_action_param_' is
-    /// invalid.
-    //Status enable_deny_reservation_debug_action();
+    /// Only use in vectorized exec engine try to do projections to trans _row_desc -> _output_row_desc
+    Status do_projections(vectorized::Block* origin_block, vectorized::Block* output_block);
 
     /// Extends blocking queue for row batches. Row batches have a property that
     /// they must be processed in the order they were produced, even in cancellation
@@ -285,34 +285,36 @@ protected:
 
     std::vector<ExecNode*> _children;
     RowDescriptor _row_descriptor;
+    vectorized::Block _origin_block;
+
+    std::unique_ptr<RowDescriptor> _output_row_descriptor;
+    std::vector<doris::vectorized::VExprContext*> _projections;
 
     /// Resource information sent from the frontend.
     const TBackendResourceProfile _resource_profile;
 
-    // debug-only: if _debug_action is not INVALID, node will perform action in
-    // _debug_phase
-    TExecNodePhase::type _debug_phase;
-    TDebugAction::type _debug_action;
-
     int64_t _limit; // -1: no limit
     int64_t _num_rows_returned;
 
     std::unique_ptr<RuntimeProfile> _runtime_profile;
 
-    /// Account for peak memory used by this node
-    std::shared_ptr<MemTracker> _mem_tracker;
-
-    /// MemTracker used by 'expr_mem_pool_'.
-    std::shared_ptr<MemTracker> _expr_mem_tracker;
-
-    /// MemPool for allocating data structures used by expression evaluators in this node.
-    /// Created in Prepare().
-    std::unique_ptr<MemPool> _expr_mem_pool;
+    // Record this node memory size. it is expected that artificial guarantees are accurate,
+    // which will providea reference for operator memory.
+    std::unique_ptr<MemTracker> _mem_tracker;
 
     RuntimeProfile::Counter* _rows_returned_counter;
     RuntimeProfile::Counter* _rows_returned_rate;
     // Account for peak memory used by this node
     RuntimeProfile::Counter* _memory_used_counter;
+    RuntimeProfile::Counter* _projection_timer;
+
+    /// Since get_next is a frequent operation, it is not necessary to generate a span for each call
+    /// to the get_next method. Therefore, the call of the get_next method in the ExecNode is
+    /// merged into this _get_next_span. The _get_next_span is initialized by
+    /// INIT_AND_SCOPE_GET_NEXT_SPAN when the get_next method is called for the first time
+    /// (recording the start timestamp), and is ended by RETURN_IF_ERROR_AND_CHECK_SPAN after the
+    /// last call to the get_next method (the record is terminated timestamp).
+    OpentelemetrySpan _get_next_span;
 
     // Execution options that are determined at runtime.  This is added to the
     // runtime profile at close().  Examples for options logged here would be
@@ -326,6 +328,9 @@ protected:
     /// reservations pool in Close().
     BufferPool::ClientHandle _buffer_pool_client;
 
+    // Set to true if this is a vectorized exec node.
+    bool _is_vec = false;
+
     ExecNode* child(int i) { return _children[i]; }
 
     bool is_closed() const { return _is_closed; }
@@ -352,10 +357,6 @@ protected:
 
     void init_runtime_profile(const std::string& name);
 
-    // Executes _debug_action if phase matches _debug_phase.
-    // 'phase' must not be INVALID.
-    Status exec_debug_action(TExecNodePhase::type phase);
-
     // Appends option to '_runtime_exec_options'
     void add_runtime_exec_option(const std::string& option);
 
@@ -372,25 +373,4 @@ private:
     bool _is_closed;
 };
 
-#define LIMIT_EXCEEDED(tracker, state, msg)                                                   \
-    do {                                                                                      \
-        stringstream str;                                                                     \
-        str << "Memory exceed limit. " << msg << " ";                                         \
-        str << "Backend: " << BackendOptions::get_localhost() << ", ";                        \
-        str << "fragment: " << print_id(state->fragment_instance_id()) << " ";                \
-        str << "Used: " << tracker->consumption() << ", Limit: " << tracker->limit() << ". "; \
-        str << "You can change the limit by session variable exec_mem_limit.";                \
-        return Status::MemoryLimitExceeded(str.str());                                        \
-    } while (false)
-
-#define RETURN_IF_LIMIT_EXCEEDED(state, msg)                                                \
-    do {                                                                                    \
-        /* if (UNLIKELY(MemTracker::limit_exceeded(*(state)->mem_trackers()))) { */         \
-        MemTracker* tracker = state->instance_mem_tracker()->find_limit_exceeded_tracker(); \
-        if (tracker != nullptr) {                                                           \
-            LIMIT_EXCEEDED(tracker, state, msg);                                            \
-        }                                                                                   \
-    } while (false)
 } // namespace doris
-
-#endif
