@@ -33,12 +33,12 @@
 #include "exec/tablet_info.h"
 #include "gen_cpp/Types_types.h"
 #include "gen_cpp/internal_service.pb.h"
+#include "runtime/thread_context.h"
 #include "util/bitmap.h"
 #include "util/countdown_latch.h"
 #include "util/ref_count_closure.h"
 #include "util/spinlock.h"
 #include "util/thread.h"
-#include "util/thrift_util.h"
 
 namespace doris {
 
@@ -53,6 +53,10 @@ class TupleDescriptor;
 class ExprContext;
 class TExpr;
 
+namespace vectorized {
+class Block;
+class MutableBlock;
+} // namespace vectorized
 namespace stream_load {
 
 class OlapTableSink;
@@ -67,15 +71,12 @@ struct AddBatchCounter {
     int64_t add_batch_num = 0;
     // time passed between marked close and finish close
     int64_t close_wait_time_ms = 0;
-    // time consumed by rpc
-    int64_t add_batch_rpc_time_us = 0;
 
     AddBatchCounter& operator+=(const AddBatchCounter& rhs) {
         add_batch_execution_time_us += rhs.add_batch_execution_time_us;
         add_batch_wait_execution_time_us += rhs.add_batch_wait_execution_time_us;
         add_batch_num += rhs.add_batch_num;
         close_wait_time_ms += rhs.close_wait_time_ms;
-        add_batch_rpc_time_us += rhs.add_batch_rpc_time_us;
         return *this;
     }
     friend AddBatchCounter operator+(const AddBatchCounter& lhs, const AddBatchCounter& rhs) {
@@ -89,20 +90,20 @@ struct AddBatchCounter {
 // So using create() to get the closure pointer is recommended. We can delete the closure ptr before the capture vars destruction.
 // Delete this point is safe, don't worry about RPC callback will run after ReusableClosure deleted.
 template <typename T>
-class ReusableClosure : public google::protobuf::Closure {
+class ReusableClosure final : public google::protobuf::Closure {
 public:
-    ReusableClosure() : cid(INVALID_BTHREAD_ID) { watch.start(); }
-    ~ReusableClosure() {
+    ReusableClosure() : cid(INVALID_BTHREAD_ID) {}
+    ~ReusableClosure() override {
         // shouldn't delete when Run() is calling or going to be called, wait for current Run() done.
         join();
-        SCOPED_ATTACH_TASK(ExecEnv::GetInstance()->orphan_mem_tracker());
+        SCOPED_SWITCH_THREAD_MEM_TRACKER_LIMITER(ExecEnv::GetInstance()->orphan_mem_tracker());
         cntl.Reset();
     }
 
     static ReusableClosure<T>* create() { return new ReusableClosure<T>(); }
 
-    void addFailedHandler(std::function<void(bool)> fn) { failed_handler = fn; }
-    void addSuccessHandler(std::function<void(const T&, bool)> fn) { success_handler = fn; }
+    void addFailedHandler(const std::function<void(bool)>& fn) { failed_handler = fn; }
+    void addSuccessHandler(const std::function<void(const T&, bool)>& fn) { success_handler = fn; }
 
     void join() {
         // We rely on in_flight to assure one rpc is running,
@@ -116,17 +117,16 @@ public:
                 brpc::Join(cid);
             }
             if (_packet_in_flight) {
-                SleepFor(MonoDelta::FromMilliseconds(10));
+                std::this_thread::sleep_for(std::chrono::milliseconds(10));
             }
         }
     }
 
     // plz follow this order: reset() -> set_in_flight() -> send brpc batch
     void reset() {
-        SCOPED_ATTACH_TASK(ExecEnv::GetInstance()->orphan_mem_tracker());
+        SCOPED_SWITCH_THREAD_MEM_TRACKER_LIMITER(ExecEnv::GetInstance()->orphan_mem_tracker());
         cntl.Reset();
         cid = cntl.call_id();
-        watch.reset();
     }
 
     bool try_set_in_flight() {
@@ -157,7 +157,6 @@ public:
 
     brpc::Controller cntl;
     T result;
-    MonotonicStopWatch watch;
 
 private:
     brpc::CallId cid;
@@ -171,25 +170,36 @@ class IndexChannel;
 class NodeChannel {
 public:
     NodeChannel(OlapTableSink* parent, IndexChannel* index_channel, int64_t node_id);
-    ~NodeChannel() noexcept;
+    virtual ~NodeChannel() noexcept;
 
     // called before open, used to add tablet located in this backend
     void add_tablet(const TTabletWithPartition& tablet) { _all_tablets.emplace_back(tablet); }
 
-    Status init(RuntimeState* state);
+    virtual Status init(RuntimeState* state);
+
+    void add_slave_tablet_nodes(int64_t tablet_id, const std::vector<int64_t>& slave_nodes) {
+        _slave_tablet_nodes[tablet_id] = slave_nodes;
+    }
 
     // we use open/open_wait to parallel
     void open();
-    Status open_wait();
+    virtual Status open_wait();
 
     Status add_row(Tuple* tuple, int64_t tablet_id);
 
-    Status add_row(BlockRow& block_row, int64_t tablet_id);
+    Status add_row(const BlockRow& block_row, int64_t tablet_id);
+
+    virtual Status add_block(vectorized::Block* block,
+                             const std::pair<std::unique_ptr<vectorized::IColumn::Selector>,
+                                             std::vector<int64_t>>& payload) {
+        LOG(FATAL) << "add block to NodeChannel not supported";
+        return Status::OK();
+    }
 
     // two ways to stop channel:
     // 1. mark_close()->close_wait() PS. close_wait() will block waiting for the last AddBatch rpc response.
     // 2. just cancel()
-    void mark_close();
+    virtual void mark_close();
     Status close_wait(RuntimeState* state);
 
     void cancel(const std::string& cancel_msg);
@@ -199,8 +209,8 @@ public:
     // 1: running, haven't reach eos.
     // only allow 1 rpc in flight
     // plz make sure, this func should be called after open_wait().
-    int try_send_and_fetch_status(RuntimeState* state,
-                                  std::unique_ptr<ThreadPoolToken>& thread_pool_token);
+    virtual int try_send_and_fetch_status(RuntimeState* state,
+                                          std::unique_ptr<ThreadPoolToken>& thread_pool_token);
 
     void try_send_batch(RuntimeState* state);
 
@@ -226,9 +236,10 @@ public:
 
     Status none_of(std::initializer_list<bool> vars);
 
-    // TODO(HW): remove after mem tracker shared
     void clear_all_batches();
 
+    virtual void clear_all_blocks() {}
+
     std::string channel_info() const {
         return fmt::format("{}, {}, node={}:{}", _name, _load_info, _node_info.host,
                            _node_info.brpc_port);
@@ -236,17 +247,21 @@ public:
 
     size_t get_pending_bytes() { return _pending_batches_bytes; }
 
-private:
+protected:
     void _cancel_with_msg(const std::string& msg);
-    void _sleep_if_memory_exceed();
 
-private:
+    virtual void _close_check();
+
+protected:
+    bool _is_vectorized = false;
     OlapTableSink* _parent = nullptr;
     IndexChannel* _index_channel = nullptr;
     int64_t _node_id = -1;
     std::string _load_info;
     std::string _name;
 
+    std::shared_ptr<MemTracker> _node_channel_tracker;
+
     TupleDescriptor* _tuple_desc = nullptr;
     NodeInfo _node_info;
 
@@ -255,6 +270,9 @@ private:
     int64_t _next_packet_seq = 0;
     MonotonicStopWatch _timeout_watch;
 
+    // the timestamp when this node channel be marked closed and finished closed
+    uint64_t _close_time_ms = 0;
+
     // user cancel or get some errors
     std::atomic<bool> _cancelled {false};
     SpinLock _cancel_msg_lock;
@@ -264,28 +282,25 @@ private:
     std::atomic<bool> _send_finished {false};
 
     // add batches finished means the last rpc has be response, used to check whether this channel can be closed
-    std::atomic<bool> _add_batches_finished {false};
+    std::atomic<bool> _add_batches_finished {false}; // reuse for vectorized
 
     bool _eos_is_produced {false}; // only for restricting producer behaviors
 
     std::unique_ptr<RowDescriptor> _row_desc;
     int _batch_size = 0;
-    std::unique_ptr<RowBatch> _cur_batch;
-    PTabletWriterAddBatchRequest _cur_add_batch_request;
 
-    std::mutex _pending_batches_lock;
-    using AddBatchReq = std::pair<std::unique_ptr<RowBatch>, PTabletWriterAddBatchRequest>;
-    std::queue<AddBatchReq> _pending_batches;
-    std::atomic<int> _pending_batches_num {0};
     // limit _pending_batches size
     std::atomic<size_t> _pending_batches_bytes {0};
     size_t _max_pending_batches_bytes {(size_t)config::nodechannel_pending_queue_max_bytes};
+    std::mutex _pending_batches_lock;          // reuse for vectorized
+    std::atomic<int> _pending_batches_num {0}; // reuse for vectorized
 
     std::shared_ptr<PBackendService_Stub> _stub = nullptr;
     RefCountClosure<PTabletWriterOpenResult>* _open_closure = nullptr;
-    ReusableClosure<PTabletWriterAddBatchResult>* _add_batch_closure = nullptr;
 
     std::vector<TTabletWithPartition> _all_tablets;
+    // map from tablet_id to node_id where slave replicas locate in
+    std::unordered_map<int64_t, std::vector<int64_t>> _slave_tablet_nodes;
     std::vector<TTabletCommitInfo> _tablet_commit_infos;
 
     AddBatchCounter _add_batch_counter;
@@ -294,9 +309,6 @@ private:
     std::atomic<int64_t> _queue_push_lock_ns {0};
     std::atomic<int64_t> _actual_consume_ns {0};
 
-    // the timestamp when this node channel be marked closed and finished closed
-    uint64_t _close_time_ms = 0;
-
     // lock to protect _is_closed.
     // The methods in the IndexChannel are called back in the RpcClosure in the NodeChannel.
     // However, this rpc callback may occur after the whole task is finished (e.g. due to network latency),
@@ -307,19 +319,32 @@ private:
     // The IndexChannel is definitely accessible until the NodeChannel is closed.
     std::mutex _closed_lock;
     bool _is_closed = false;
+
     RuntimeState* _state;
+    // rows number received per tablet, tablet_id -> rows_num
+    std::vector<std::pair<int64_t, int64_t>> _tablets_received_rows;
+
+private:
+    std::unique_ptr<RowBatch> _cur_batch;
+    PTabletWriterAddBatchRequest _cur_add_batch_request;
+    using AddBatchReq = std::pair<std::unique_ptr<RowBatch>, PTabletWriterAddBatchRequest>;
+    std::queue<AddBatchReq> _pending_batches;
+    ReusableClosure<PTabletWriterAddBatchResult>* _add_batch_closure = nullptr;
 };
 
 class IndexChannel {
 public:
-    IndexChannel(OlapTableSink* parent, int64_t index_id) : _parent(parent), _index_id(index_id) {}
-    ~IndexChannel();
+    IndexChannel(OlapTableSink* parent, int64_t index_id, bool is_vec)
+            : _parent(parent), _index_id(index_id), _is_vectorized(is_vec) {
+        _index_channel_tracker =
+                std::make_unique<MemTracker>("IndexChannel:indexID=" + std::to_string(_index_id));
+    }
+    ~IndexChannel() = default;
 
     Status init(RuntimeState* state, const std::vector<TTabletWithPartition>& tablets);
 
-    void add_row(Tuple* tuple, int64_t tablet_id);
-
-    void add_row(BlockRow& block_row, int64_t tablet_id);
+    template <typename Row>
+    void add_row(const Row& tuple, int64_t tablet_id);
 
     void for_each_node_channel(
             const std::function<void(const std::shared_ptr<NodeChannel>&)>& func) {
@@ -345,11 +370,20 @@ public:
         return mem_consumption;
     }
 
+    void set_tablets_received_rows(
+            const std::vector<std::pair<int64_t, int64_t>>& tablets_received_rows, int64_t node_id);
+
+    // check whether the rows num written by different replicas is consistent
+    Status check_tablet_received_rows_consistency();
+
 private:
     friend class NodeChannel;
+    friend class VNodeChannel;
     friend class VOlapTableSink;
+
     OlapTableSink* _parent;
     int64_t _index_id;
+    bool _is_vectorized = false;
 
     // from backend channel to tablet_id
     // ATTN: must be placed before `_node_channels` and `_channels_by_tablet`.
@@ -370,8 +404,28 @@ private:
     // key is tablet_id, value is error message
     std::unordered_map<int64_t, std::string> _failed_channels_msgs;
     Status _intolerable_failure_status = Status::OK();
+
+    std::unique_ptr<MemTracker> _index_channel_tracker;
+    // rows num received by DeltaWriter per tablet, tablet_id -> <node_Id, rows_num>
+    // used to verify whether the rows num received by different replicas is consistent
+    std::map<int64_t, std::vector<std::pair<int64_t, int64_t>>> _tablets_received_rows;
 };
 
+template <typename Row>
+void IndexChannel::add_row(const Row& tuple, int64_t tablet_id) {
+    SCOPED_CONSUME_MEM_TRACKER(_index_channel_tracker.get());
+    auto it = _channels_by_tablet.find(tablet_id);
+    DCHECK(it != _channels_by_tablet.end()) << "unknown tablet, tablet_id=" << tablet_id;
+    for (const auto& channel : it->second) {
+        // if this node channel is already failed, this add_row will be skipped
+        auto st = channel->add_row(tuple, tablet_id);
+        if (!st.ok()) {
+            mark_as_failed(channel->node_id(), channel->host(), st.to_string(), tablet_id);
+            // continue add row to other node, the error will be checked for every batch outside
+        }
+    }
+}
+
 // Write data to Olap Table.
 // When OlapTableSink::open() called, there will be a consumer thread running in the background.
 // When you call OlapTableSink::send(), you will be the producer who products pending batches.
@@ -408,6 +462,8 @@ private:
     // set stop_processing is we want to stop the whole process now.
     Status _validate_data(RuntimeState* state, RowBatch* batch, Bitmap* filter_bitmap,
                           int* filtered_rows, bool* stop_processing);
+    bool _validate_cell(const TypeDescriptor& type, const std::string& col_name, void* slot,
+                        size_t slot_index, fmt::memory_buffer& error_msg, RowBatch* batch);
 
     // the consumer func of sending pending batches in every NodeChannel.
     // use polling & NodeChannel::try_send_and_fetch_status() to achieve nonblocking sending.
@@ -416,8 +472,11 @@ private:
 
 protected:
     friend class NodeChannel;
+    friend class VNodeChannel;
     friend class IndexChannel;
 
+    bool _is_vectorized = false;
+
     std::shared_ptr<MemTracker> _mem_tracker;
 
     ObjectPool* _pool;
@@ -432,8 +491,6 @@ protected:
     // this is tuple descriptor of destination OLAP table
     TupleDescriptor* _output_tuple_desc = nullptr;
     RowDescriptor* _output_row_desc = nullptr;
-    std::vector<ExprContext*> _output_expr_ctxs;
-    std::unique_ptr<RowBatch> _output_batch;
 
     bool _need_validate_data = false;
 
@@ -446,8 +503,9 @@ protected:
 
     // TODO(zc): think about cache this data
     std::shared_ptr<OlapTableSchemaParam> _schema;
-    OlapTablePartitionParam* _partition = nullptr;
     OlapTableLocationParam* _location = nullptr;
+    bool _write_single_replica = false;
+    OlapTableLocationParam* _slave_location = nullptr;
     DorisNodesInfo* _nodes_info = nullptr;
 
     RuntimeProfile* _profile = nullptr;
@@ -472,7 +530,6 @@ protected:
     int64_t _convert_batch_ns = 0;
     int64_t _validate_data_ns = 0;
     int64_t _send_data_ns = 0;
-    int64_t _serialize_batch_ns = 0;
     int64_t _number_input_rows = 0;
     int64_t _number_output_rows = 0;
     int64_t _number_filtered_rows = 0;
@@ -515,6 +572,11 @@ protected:
     // only compute tablet index in the corresponding partition once for the whole time in olap table sink
     enum FindTabletMode { FIND_TABLET_EVERY_ROW, FIND_TABLET_EVERY_BATCH, FIND_TABLET_EVERY_SINK };
     FindTabletMode findTabletMode = FindTabletMode::FIND_TABLET_EVERY_ROW;
+
+private:
+    OlapTablePartitionParam* _partition = nullptr;
+    std::vector<ExprContext*> _output_expr_ctxs;
+    std::unique_ptr<RowBatch> _output_batch;
 };
 
 } // namespace stream_load
