@@ -17,11 +17,6 @@
 
 #include "exprs/es_functions.h"
 
-#include "exprs/anyval_util.h"
-#include "exprs/expr.h"
-#include "runtime/tuple_row.h"
-#include "util/debug_util.h"
-
 namespace doris {
 
 void ESFunctions::init() {}
