@@ -22,23 +22,29 @@
 #include <boost/algorithm/string.hpp>
 #include <cmath>
 #include <ctime>
-#include <string>
 #include <random>
+#include <string>
 
 #include "agent/cgroups_mgr.h"
+#include "common/config.h"
 #include "common/status.h"
 #include "gutil/strings/substitute.h"
-#include "olap/convert_rowset.h"
+#include "io/cache/file_cache_manager.h"
 #include "olap/cumulative_compaction.h"
 #include "olap/olap_common.h"
 #include "olap/olap_define.h"
+#include "olap/rowset/beta_rowset_writer.h"
 #include "olap/storage_engine.h"
+#include "util/file_utils.h"
 #include "util/time.h"
 
 using std::string;
 
 namespace doris {
 
+using io::FileCacheManager;
+using io::Path;
+
 // number of running SCHEMA-CHANGE threads
 volatile uint32_t g_schema_change_active_threads = 0;
 
@@ -67,32 +73,6 @@ Status StorageEngine::start_bg_threads() {
         data_dirs.push_back(tmp_store.second);
     }
 
-    int32_t max_thread_num = config::max_base_compaction_threads;
-    ThreadPoolBuilder("BaseCompactionTaskThreadPool")
-            .set_min_threads(max_thread_num)
-            .set_max_threads(max_thread_num)
-            .build(&_base_compaction_thread_pool);
-    max_thread_num = config::max_cumu_compaction_threads;
-    ThreadPoolBuilder("CumuCompactionTaskThreadPool")
-            .set_min_threads(max_thread_num)
-            .set_max_threads(max_thread_num)
-            .build(&_cumu_compaction_thread_pool);
-
-    int32_t convert_rowset_thread_num = config::convert_rowset_thread_num;
-    if (convert_rowset_thread_num > 0) {
-        ThreadPoolBuilder("ConvertRowsetTaskThreadPool")
-                .set_min_threads(convert_rowset_thread_num)
-                .set_max_threads(convert_rowset_thread_num)
-                .build(&_convert_rowset_thread_pool);
-
-        // alpha rowset scan thread
-        RETURN_IF_ERROR(Thread::create(
-                "StorageEngine", "alpha_rowset_scan_thread",
-                [this]() { this->_alpha_rowset_scan_thread_callback(); },
-                &_alpha_rowset_scan_thread));
-        LOG(INFO) << "alpha rowset scan thread started";
-    }
-
     ThreadPoolBuilder("BaseCompactionTaskThreadPool")
             .set_min_threads(config::max_base_compaction_threads)
             .set_max_threads(config::max_base_compaction_threads)
@@ -105,6 +85,12 @@ Status StorageEngine::start_bg_threads() {
             .set_min_threads(config::quick_compaction_max_threads)
             .set_max_threads(config::quick_compaction_max_threads)
             .build(&_quick_compaction_thread_pool);
+    if (config::enable_segcompaction && config::enable_storage_vectorization) {
+        ThreadPoolBuilder("SegCompactionTaskThreadPool")
+                .set_min_threads(config::seg_compaction_max_threads)
+                .set_max_threads(config::seg_compaction_max_threads)
+                .build(&_seg_compaction_thread_pool);
+    }
 
     // compaction tasks producer thread
     RETURN_IF_ERROR(Thread::create(
@@ -112,7 +98,6 @@ Status StorageEngine::start_bg_threads() {
             [this]() { this->_compaction_tasks_producer_callback(); },
             &_compaction_tasks_producer_thread));
     LOG(INFO) << "compaction tasks producer thread started";
-
     int32_t max_checkpoint_thread_num = config::max_meta_checkpoint_threads;
     if (max_checkpoint_thread_num < 0) {
         max_checkpoint_thread_num = data_dirs.size();
@@ -136,9 +121,6 @@ Status StorageEngine::start_bg_threads() {
     // path scan and gc thread
     if (config::path_gc_check) {
         for (auto data_dir : get_stores()) {
-            if (data_dir->is_remote()) {
-                continue;
-            }
             scoped_refptr<Thread> path_scan_thread;
             RETURN_IF_ERROR(Thread::create(
                     "StorageEngine", "path_scan_thread",
@@ -156,6 +138,30 @@ Status StorageEngine::start_bg_threads() {
         LOG(INFO) << "path scan/gc threads started. number:" << get_stores().size();
     }
 
+    ThreadPoolBuilder("CooldownTaskThreadPool")
+            .set_min_threads(config::cooldown_thread_num)
+            .set_max_threads(config::cooldown_thread_num)
+            .build(&_cooldown_thread_pool);
+    LOG(INFO) << "cooldown thread pool started";
+
+    RETURN_IF_ERROR(Thread::create(
+            "StorageEngine", "cooldown_tasks_producer_thread",
+            [this]() { this->_cooldown_tasks_producer_callback(); },
+            &_cooldown_tasks_producer_thread));
+    LOG(INFO) << "cooldown tasks producer thread started";
+
+    RETURN_IF_ERROR(Thread::create(
+            "StorageEngine", "cache_file_cleaner_tasks_producer_thread",
+            [this]() { this->_cache_file_cleaner_tasks_producer_callback(); },
+            &_cache_file_cleaner_tasks_producer_thread));
+    LOG(INFO) << "cache file cleaner tasks producer thread started";
+
+    // add tablet publish version thread pool
+    ThreadPoolBuilder("TabletPublishTxnThreadPool")
+            .set_min_threads(config::tablet_publish_txn_max_thread)
+            .set_max_threads(config::tablet_publish_txn_max_thread)
+            .build(&_tablet_publish_txn_thread_pool);
+
     LOG(INFO) << "all storage engine's background threads are started.";
     return Status::OK();
 }
@@ -165,13 +171,11 @@ void StorageEngine::_fd_cache_clean_callback() {
     ProfilerRegisterThread();
 #endif
     int32_t interval = 600;
-    while (!_stop_background_threads_latch.wait_for(MonoDelta::FromSeconds(interval))) {
+    while (!_stop_background_threads_latch.wait_for(std::chrono::seconds(interval))) {
         interval = config::cache_clean_interval;
         if (interval <= 0) {
-            OLAP_LOG_WARNING(
-                    "config of file descriptor clean interval is illegal: [%d], "
-                    "force set to 3600",
-                    interval);
+            LOG(WARNING) << "config of file descriptor clean interval is illegal: [" << interval
+                         << "], force set to 3600 ";
             interval = 3600;
         }
 
@@ -187,8 +191,8 @@ void StorageEngine::_garbage_sweeper_thread_callback() {
     uint32_t min_interval = config::min_garbage_sweep_interval;
 
     if (!(max_interval >= min_interval && min_interval > 0)) {
-        OLAP_LOG_WARNING("garbage sweep interval config is illegal: [max=%d min=%d].", max_interval,
-                         min_interval);
+        LOG(WARNING) << "garbage sweep interval config is illegal: [max=" << max_interval
+                     << " min=" << min_interval << "].";
         min_interval = 1;
         max_interval = max_interval >= min_interval ? max_interval : min_interval;
         LOG(INFO) << "force reset garbage sweep interval. "
@@ -199,7 +203,7 @@ void StorageEngine::_garbage_sweeper_thread_callback() {
     double usage = 1.0;
     // After the program starts, the first round of cleaning starts after min_interval.
     uint32_t curr_interval = min_interval;
-    while (!_stop_background_threads_latch.wait_for(MonoDelta::FromSeconds(curr_interval))) {
+    while (!_stop_background_threads_latch.wait_for(std::chrono::seconds(curr_interval))) {
         // Function properties:
         // when usage < 0.6,          ratio close to 1.(interval close to max_interval)
         // when usage at [0.6, 0.75], ratio is rapidly decreasing from 0.87 to 0.27.
@@ -213,12 +217,10 @@ void StorageEngine::_garbage_sweeper_thread_callback() {
         curr_interval = std::min(curr_interval, max_interval);
 
         // start clean trash and update usage.
-        OLAPStatus res = start_trash_sweep(&usage);
-        if (res != OLAP_SUCCESS) {
-            OLAP_LOG_WARNING(
-                    "one or more errors occur when sweep trash."
-                    "see previous message for detail. [err code=%d]",
-                    res);
+        Status res = start_trash_sweep(&usage);
+        if (!res.ok()) {
+            LOG(WARNING) << "one or more errors occur when sweep trash."
+                         << "see previous message for detail. err code=" << res;
             // do nothing. continue next loop.
         }
     }
@@ -239,7 +241,7 @@ void StorageEngine::_disk_stat_monitor_thread_callback() {
                          << ", force set to 1";
             interval = 1;
         }
-    } while (!_stop_background_threads_latch.wait_for(MonoDelta::FromSeconds(interval)));
+    } while (!_stop_background_threads_latch.wait_for(std::chrono::seconds(interval)));
 }
 
 void StorageEngine::check_cumulative_compaction_config() {
@@ -276,7 +278,7 @@ void StorageEngine::_unused_rowset_monitor_thread_callback() {
                          << ", force set to 1";
             interval = 1;
         }
-    } while (!_stop_background_threads_latch.wait_for(MonoDelta::FromSeconds(interval)));
+    } while (!_stop_background_threads_latch.wait_for(std::chrono::seconds(interval)));
 }
 
 void StorageEngine::_path_gc_thread_callback(DataDir* data_dir) {
@@ -299,7 +301,7 @@ void StorageEngine::_path_gc_thread_callback(DataDir* data_dir) {
                          << "will be forced set to half hour";
             interval = 1800; // 0.5 hour
         }
-    } while (!_stop_background_threads_latch.wait_for(MonoDelta::FromSeconds(interval)));
+    } while (!_stop_background_threads_latch.wait_for(std::chrono::seconds(interval)));
 }
 
 void StorageEngine::_path_scan_thread_callback(DataDir* data_dir) {
@@ -318,7 +320,7 @@ void StorageEngine::_path_scan_thread_callback(DataDir* data_dir) {
                          << "will be forced set to one day";
             interval = 24 * 3600; // one day
         }
-    } while (!_stop_background_threads_latch.wait_for(MonoDelta::FromSeconds(interval)));
+    } while (!_stop_background_threads_latch.wait_for(std::chrono::seconds(interval)));
 }
 
 void StorageEngine::_tablet_checkpoint_callback(const std::vector<DataDir*>& data_dirs) {
@@ -339,44 +341,7 @@ void StorageEngine::_tablet_checkpoint_callback(const std::vector<DataDir*>& dat
             }
         }
         interval = config::generate_tablet_meta_checkpoint_tasks_interval_secs;
-    } while (!_stop_background_threads_latch.wait_for(MonoDelta::FromSeconds(interval)));
-}
-
-void StorageEngine::_alpha_rowset_scan_thread_callback() {
-    LOG(INFO) << "try to start alpha rowset scan thread!";
-
-    auto scan_interval_sec = config::scan_alpha_rowset_min_interval_sec;
-    auto max_convert_task = config::convert_rowset_thread_num * 2;
-    do {
-        std::vector<TabletSharedPtr> tablet_have_alpha_rowset;
-        _tablet_manager->find_tablet_have_alpha_rowset(tablet_have_alpha_rowset);
-
-        std::random_device rd;
-        std::mt19937 g(rd());
-        std::shuffle(tablet_have_alpha_rowset.begin(), tablet_have_alpha_rowset.end(), g);
-
-        for (int i = 0; i < max_convert_task && i < tablet_have_alpha_rowset.size(); ++i) {
-            auto tablet = tablet_have_alpha_rowset[i];
-            auto st = _convert_rowset_thread_pool->submit_func([=]() {
-                CgroupsMgr::apply_system_cgroup();
-                std::string tracker_label =
-                        "StorageEngine:ConvertRowset:" + std::to_string(tablet->tablet_id());
-                auto convert_rowset = std::make_shared<ConvertRowset>(
-                    tablet, tracker_label, _convert_rowset_mem_tracker);
-                convert_rowset->do_convert();
-            });
-            if (!st.ok()) {
-                LOG(WARNING) << "submit convert tablet tasks failed.";
-            }
-        }
-
-        if (tablet_have_alpha_rowset.size() == 0) {
-            scan_interval_sec = std::min(3600, scan_interval_sec * 2);
-        } else {
-            _convert_rowset_thread_pool->wait();
-            scan_interval_sec = config::scan_alpha_rowset_min_interval_sec;
-        }
-    } while (!_stop_background_threads_latch.wait_for(MonoDelta::FromSeconds(scan_interval_sec)));
+    } while (!_stop_background_threads_latch.wait_for(std::chrono::seconds(interval)));
 }
 
 void StorageEngine::_adjust_compaction_thread_num() {
@@ -502,22 +467,20 @@ void StorageEngine::_compaction_tasks_producer_callback() {
             for (const auto& tablet : tablets_compaction) {
                 Status st = _submit_compaction_task(tablet, compaction_type);
                 if (!st.ok()) {
-                    LOG(WARNING) << "failed to submit compaction task for tablet: " << tablet->tablet_id()
-                        << ", err: " << st.get_error_msg();
+                    LOG(WARNING) << "failed to submit compaction task for tablet: "
+                                 << tablet->tablet_id() << ", err: " << st;
                 }
             }
             interval = config::generate_compaction_tasks_min_interval_ms;
         } else {
             interval = config::check_auto_compaction_interval_seconds * 1000;
         }
-    } while (!_stop_background_threads_latch.wait_for(MonoDelta::FromMilliseconds(interval)));
+    } while (!_stop_background_threads_latch.wait_for(std::chrono::milliseconds(interval)));
 }
 
 std::vector<TabletSharedPtr> StorageEngine::_generate_compaction_tasks(
         CompactionType compaction_type, std::vector<DataDir*>& data_dirs, bool check_score) {
-
     _update_cumulative_compaction_policy();
-
     std::vector<TabletSharedPtr> tablets_compaction;
     uint32_t max_compaction_score = 0;
 
@@ -526,7 +489,7 @@ std::vector<TabletSharedPtr> StorageEngine::_generate_compaction_tasks(
     std::shuffle(data_dirs.begin(), data_dirs.end(), g);
 
     // Copy _tablet_submitted_xxx_compaction map so that we don't need to hold _tablet_submitted_compaction_mutex
-    // when travesing the data dir
+    // when traversing the data dir
     std::map<DataDir*, std::unordered_set<TTabletId>> copied_cumu_map;
     std::map<DataDir*, std::unordered_set<TTabletId>> copied_base_map;
     {
@@ -542,7 +505,8 @@ std::vector<TabletSharedPtr> StorageEngine::_generate_compaction_tasks(
         // If so, the last Slot can be assigned to Base compaction,
         // otherwise, this Slot needs to be reserved for cumulative compaction.
         int count = copied_cumu_map[data_dir].size() + copied_base_map[data_dir].size();
-        int thread_per_disk = data_dir->is_ssd_disk() ? config::compaction_task_num_per_fast_disk : config::compaction_task_num_per_disk;
+        int thread_per_disk = data_dir->is_ssd_disk() ? config::compaction_task_num_per_fast_disk
+                                                      : config::compaction_task_num_per_disk;
         if (count >= thread_per_disk) {
             // Return if no available slot
             need_pick_tablet = false;
@@ -571,14 +535,18 @@ std::vector<TabletSharedPtr> StorageEngine::_generate_compaction_tasks(
                             ? copied_cumu_map[data_dir]
                             : copied_base_map[data_dir],
                     &disk_max_score, _cumulative_compaction_policy);
-            if (data_dir->is_remote()) {
-                continue;
-            }
-            if (tablet != nullptr) {
+            if (tablet != nullptr &&
+                !tablet->tablet_meta()->tablet_schema()->disable_auto_compaction()) {
                 if (need_pick_tablet) {
                     tablets_compaction.emplace_back(tablet);
                 }
                 max_compaction_score = std::max(max_compaction_score, disk_max_score);
+            } else if (tablet != nullptr &&
+                       tablet->tablet_meta()->tablet_schema()->disable_auto_compaction()) {
+                LOG_EVERY_N(INFO, 500)
+                        << "Tablet " << tablet->full_name()
+                        << " will be ignored by automatic compaction tasks since it's "
+                        << "set to disabled automatic compaction.";
             }
         }
     }
@@ -596,21 +564,9 @@ std::vector<TabletSharedPtr> StorageEngine::_generate_compaction_tasks(
 }
 
 void StorageEngine::_update_cumulative_compaction_policy() {
-    std::string current_policy = "";
-    {
-        std::lock_guard<std::mutex> lock(*config::get_mutable_string_config_lock());
-        current_policy = config::cumulative_compaction_policy;
-    }
-    boost::to_upper(current_policy);
-    if (_cumulative_compaction_policy == nullptr ||
-        _cumulative_compaction_policy->name() != current_policy) {
-        if (current_policy == CUMULATIVE_SIZE_BASED_POLICY) {
-            // check size_based cumulative compaction config
-            check_cumulative_compaction_config();
-        }
+    if (_cumulative_compaction_policy == nullptr) {
         _cumulative_compaction_policy =
-                CumulativeCompactionPolicyFactory::create_cumulative_compaction_policy(
-                        current_policy);
+                CumulativeCompactionPolicyFactory::create_cumulative_compaction_policy();
     }
 }
 
@@ -620,10 +576,14 @@ bool StorageEngine::_push_tablet_into_submitted_compaction(TabletSharedPtr table
     bool already_existed = false;
     switch (compaction_type) {
     case CompactionType::CUMULATIVE_COMPACTION:
-        already_existed = !(_tablet_submitted_cumu_compaction[tablet->data_dir()].insert(tablet->tablet_id()).second);
+        already_existed = !(_tablet_submitted_cumu_compaction[tablet->data_dir()]
+                                    .insert(tablet->tablet_id())
+                                    .second);
         break;
     default:
-        already_existed = !(_tablet_submitted_base_compaction[tablet->data_dir()].insert(tablet->tablet_id()).second);
+        already_existed = !(_tablet_submitted_base_compaction[tablet->data_dir()]
+                                    .insert(tablet->tablet_id())
+                                    .second);
         break;
     }
     return already_existed;
@@ -649,12 +609,13 @@ void StorageEngine::_pop_tablet_from_submitted_compaction(TabletSharedPtr tablet
     }
 }
 
-Status StorageEngine::_submit_compaction_task(TabletSharedPtr tablet, CompactionType compaction_type) {
+Status StorageEngine::_submit_compaction_task(TabletSharedPtr tablet,
+                                              CompactionType compaction_type) {
     bool already_exist = _push_tablet_into_submitted_compaction(tablet, compaction_type);
     if (already_exist) {
-        return Status::AlreadyExist(strings::Substitute(
-                "compaction task has already been submitted, tablet_id=$0, compaction_type=$1.",
-                tablet->tablet_id(), compaction_type));
+        return Status::AlreadyExist(
+                "compaction task has already been submitted, tablet_id={}, compaction_type={}.",
+                tablet->tablet_id(), compaction_type);
     }
     int64_t permits = 0;
     Status st = tablet->prepare_compaction_and_calculate_permits(compaction_type, tablet, &permits);
@@ -676,9 +637,10 @@ Status StorageEngine::_submit_compaction_task(TabletSharedPtr tablet, Compaction
             // reset compaction
             tablet->reset_compaction(compaction_type);
             _pop_tablet_from_submitted_compaction(tablet, compaction_type);
-            return Status::InternalError(strings::Substitute(
-                    "failed to submit compaction task to thread pool, tablet_id=$0, compaction_type=$1.",
-                    tablet->tablet_id(), compaction_type));
+            return Status::InternalError(
+                    "failed to submit compaction task to thread pool, "
+                    "tablet_id={}, compaction_type={}.",
+                    tablet->tablet_id(), compaction_type);
         }
         return Status::OK();
     } else {
@@ -686,35 +648,127 @@ Status StorageEngine::_submit_compaction_task(TabletSharedPtr tablet, Compaction
         tablet->reset_compaction(compaction_type);
         _pop_tablet_from_submitted_compaction(tablet, compaction_type);
         if (!st.ok()) {
-            return Status::InternalError(strings::Substitute(
-                        "failed to prepare compaction task and calculate permits, tablet_id=$0, compaction_type=$1, "
-                        "permit=$2, current_permit=$3, status=$4",
-                        tablet->tablet_id(), compaction_type, permits, _permit_limiter.usage(), st.get_error_msg()));
+            return Status::InternalError(
+                    "failed to prepare compaction task and calculate permits, "
+                    "tablet_id={}, compaction_type={}, "
+                    "permit={}, current_permit={}, status={}",
+                    tablet->tablet_id(), compaction_type, permits, _permit_limiter.usage(),
+                    st.to_string());
         }
         return st;
     }
 }
 
-Status StorageEngine::submit_compaction_task(TabletSharedPtr tablet, CompactionType compaction_type) {
+Status StorageEngine::submit_compaction_task(TabletSharedPtr tablet,
+                                             CompactionType compaction_type) {
     _update_cumulative_compaction_policy();
-    if (tablet->get_cumulative_compaction_policy() == nullptr ||
-        tablet->get_cumulative_compaction_policy()->name() != _cumulative_compaction_policy->name()) {
+    if (tablet->get_cumulative_compaction_policy() == nullptr) {
         tablet->set_cumulative_compaction_policy(_cumulative_compaction_policy);
     }
     return _submit_compaction_task(tablet, compaction_type);
 }
 
 Status StorageEngine::_handle_quick_compaction(TabletSharedPtr tablet) {
-    std::shared_ptr<CumulativeCompaction> compact; 
-    StorageEngine::instance()->create_cumulative_compaction(tablet, compact);
-    compact->quick_rowsets_compact();
+    CumulativeCompaction compact(tablet);
+    compact.quick_rowsets_compact();
+    _pop_tablet_from_submitted_compaction(tablet, CompactionType::CUMULATIVE_COMPACTION);
     return Status::OK();
 }
 
 Status StorageEngine::submit_quick_compaction_task(TabletSharedPtr tablet) {
+    bool already_exist =
+            _push_tablet_into_submitted_compaction(tablet, CompactionType::CUMULATIVE_COMPACTION);
+    if (already_exist) {
+        return Status::AlreadyExist(
+                "compaction task has already been submitted, tablet_id={}, compaction_type={}.",
+                tablet->tablet_id(), CompactionType::CUMULATIVE_COMPACTION);
+    }
     _quick_compaction_thread_pool->submit_func(
             std::bind<void>(&StorageEngine::_handle_quick_compaction, this, tablet));
     return Status::OK();
 }
 
+Status StorageEngine::_handle_seg_compaction(BetaRowsetWriter* writer,
+                                             SegCompactionCandidatesSharedPtr segments) {
+    writer->compact_segments(segments);
+    // return OK here. error will be reported via BetaRowsetWriter::_segcompaction_status
+    return Status::OK();
+}
+
+Status StorageEngine::submit_seg_compaction_task(BetaRowsetWriter* writer,
+                                                 SegCompactionCandidatesSharedPtr segments) {
+    return _seg_compaction_thread_pool->submit_func(
+            std::bind<void>(&StorageEngine::_handle_seg_compaction, this, writer, segments));
+}
+
+void StorageEngine::_cooldown_tasks_producer_callback() {
+    int64_t interval = config::generate_cooldown_task_interval_sec;
+    do {
+        if (_cooldown_thread_pool->get_queue_size() > 0) {
+            continue;
+        }
+        std::vector<TabletSharedPtr> tablets;
+        // TODO(luwei) : a more efficient way to get cooldown tablets
+        _tablet_manager->get_cooldown_tablets(&tablets);
+        LOG(INFO) << "cooldown producer get tablet num: " << tablets.size();
+        for (const auto& tablet : tablets) {
+            Status st = _cooldown_thread_pool->submit_func([=]() {
+                {
+                    // Cooldown tasks on the same tablet cannot be executed concurrently
+                    std::lock_guard<std::mutex> lock(_running_cooldown_mutex);
+                    auto it = _running_cooldown_tablets.find(tablet->tablet_id());
+                    if (it != _running_cooldown_tablets.end()) {
+                        return;
+                    }
+
+                    // the number of concurrent cooldown tasks in each directory
+                    // cannot exceed the configured value
+                    auto dir_it = _running_cooldown_tasks_cnt.find(tablet->data_dir());
+                    if (dir_it != _running_cooldown_tasks_cnt.end() &&
+                        dir_it->second >= config::concurrency_per_dir) {
+                        return;
+                    }
+
+                    _running_cooldown_tablets.insert(tablet->tablet_id());
+                    dir_it = _running_cooldown_tasks_cnt.find(tablet->data_dir());
+                    if (dir_it != _running_cooldown_tasks_cnt.end()) {
+                        _running_cooldown_tasks_cnt[tablet->data_dir()]++;
+                    } else {
+                        _running_cooldown_tasks_cnt[tablet->data_dir()] = 1;
+                    }
+                }
+
+                Status st = tablet->cooldown();
+                if (!st.ok()) {
+                    LOG(WARNING) << "failed to cooldown, tablet: " << tablet->tablet_id()
+                                 << " err: " << st.to_string();
+                } else {
+                    LOG(INFO) << "succeed to cooldown, tablet: " << tablet->tablet_id()
+                              << " cooldown progress ("
+                              << tablets.size() - _cooldown_thread_pool->get_queue_size() << "/"
+                              << tablets.size() << ")";
+                }
+
+                {
+                    std::lock_guard<std::mutex> lock(_running_cooldown_mutex);
+                    _running_cooldown_tasks_cnt[tablet->data_dir()]--;
+                    _running_cooldown_tablets.erase(tablet->tablet_id());
+                }
+            });
+
+            if (!st.ok()) {
+                LOG(INFO) << "failed to submit cooldown task, err msg: " << st;
+            }
+        }
+    } while (!_stop_background_threads_latch.wait_for(std::chrono::seconds(interval)));
+}
+
+void StorageEngine::_cache_file_cleaner_tasks_producer_callback() {
+    int64_t interval = config::generate_cache_cleaner_task_interval_sec;
+    do {
+        LOG(INFO) << "Begin to Clean cache files";
+        FileCacheManager::instance()->gc_file_caches();
+    } while (!_stop_background_threads_latch.wait_for(std::chrono::seconds(interval)));
+}
+
 } // namespace doris
