@@ -15,13 +15,16 @@
 // specific language governing permissions and limitations
 // under the License.
 
-#ifndef DORIS_BE_SRC_OLAP_TABLET_H
-#define DORIS_BE_SRC_OLAP_TABLET_H
+#pragma once
+
 #include <functional>
 #include <memory>
+#include <mutex>
 #include <set>
+#include <shared_mutex>
 #include <string>
 #include <unordered_map>
+#include <unordered_set>
 #include <vector>
 
 #include "gen_cpp/AgentService_types.h"
@@ -30,9 +33,12 @@
 #include "olap/base_tablet.h"
 #include "olap/cumulative_compaction_policy.h"
 #include "olap/data_dir.h"
+#include "olap/olap_common.h"
 #include "olap/olap_define.h"
 #include "olap/rowset/rowset.h"
 #include "olap/rowset/rowset_reader.h"
+#include "olap/rowset/rowset_tree.h"
+#include "olap/rowset/segment_v2/segment.h"
 #include "olap/tablet_meta.h"
 #include "olap/tuple.h"
 #include "olap/utils.h"
@@ -47,9 +53,15 @@ class TabletMeta;
 class CumulativeCompactionPolicy;
 class CumulativeCompaction;
 class BaseCompaction;
+class RowsetWriter;
+
+struct TabletTxnInfo;
+struct RowsetWriterContext;
 
 using TabletSharedPtr = std::shared_ptr<Tablet>;
 
+enum TabletStorageType { STORAGE_TYPE_LOCAL, STORAGE_TYPE_REMOTE, STORAGE_TYPE_REMOTE_AND_LOCAL };
+
 class Tablet : public BaseTablet {
 public:
     static TabletSharedPtr create_tablet_from_meta(TabletMetaSharedPtr tablet_meta,
@@ -58,8 +70,8 @@ public:
     Tablet(TabletMetaSharedPtr tablet_meta, DataDir* data_dir,
            const std::string& cumulative_compaction_type = "");
 
-    OLAPStatus init();
-    inline bool init_succeeded();
+    Status init();
+    bool init_succeeded();
 
     bool is_used();
 
@@ -68,41 +80,51 @@ public:
 
     void save_meta();
     // Used in clone task, to update local meta when finishing a clone job
-    OLAPStatus revise_tablet_meta(const std::vector<RowsetMetaSharedPtr>& rowsets_to_clone,
-                              const std::vector<Version>& versions_to_delete);
-    OLAPStatus pick_quick_compaction_rowsets(std::vector<RowsetSharedPtr>* input_rowsets,
+    Status revise_tablet_meta(const std::vector<RowsetMetaSharedPtr>& rowsets_to_clone,
+                              const std::vector<Version>& versions_to_delete,
+                              bool is_incremental_clone);
+    Status pick_quick_compaction_rowsets(std::vector<RowsetSharedPtr>* input_rowsets,
                                          int64_t* permits);
 
-    inline const int64_t cumulative_layer_point() const;
-    inline void set_cumulative_layer_point(int64_t new_point);
+    const int64_t cumulative_layer_point() const;
+    void set_cumulative_layer_point(int64_t new_point);
     inline const int64_t cumulative_promotion_size() const;
     inline void set_cumulative_promotion_size(int64_t new_size);
 
-    inline size_t tablet_footprint(); // disk space occupied by tablet
-    inline size_t num_rows();
-    inline int version_count() const;
-    inline Version max_version() const;
-    inline CumulativeCompactionPolicy* cumulative_compaction_policy();
+    // Disk space occupied by tablet, contain local and remote.
+    size_t tablet_footprint();
+    // Local disk space occupied by tablet.
+    size_t tablet_local_size();
+    // Remote disk space occupied by tablet.
+    size_t tablet_remote_size();
+
+    size_t num_rows();
+    int version_count() const;
+    uint64_t segment_count() const;
+    Version max_version() const;
+    Version max_version_unlocked() const;
+    CumulativeCompactionPolicy* cumulative_compaction_policy();
+    bool enable_unique_key_merge_on_write() const;
 
     // properties encapsulated in TabletSchema
-    inline KeysType keys_type() const;
-    inline SortType sort_type() const;
-    inline size_t sort_col_num() const;
-    inline size_t num_columns() const;
-    inline size_t num_null_columns() const;
-    inline size_t num_key_columns() const;
-    inline size_t num_short_key_columns() const;
-    inline size_t num_rows_per_row_block() const;
-    inline CompressKind compress_kind() const;
-    inline double bloom_filter_fpp() const;
-    inline size_t next_unique_id() const;
-    inline size_t row_size() const;
-    inline int32_t field_index(const std::string& field_name) const;
+    KeysType keys_type() const;
+    SortType sort_type() const;
+    size_t sort_col_num() const;
+    size_t num_columns() const;
+    size_t num_null_columns() const;
+    size_t num_key_columns() const;
+    size_t num_short_key_columns() const;
+    size_t num_rows_per_row_block() const;
+    CompressKind compress_kind() const;
+    double bloom_filter_fpp() const;
+    size_t next_unique_id() const;
+    size_t row_size() const;
 
     // operation in rowsets
-    OLAPStatus add_rowset(RowsetSharedPtr rowset, bool need_persist = true);
-    OLAPStatus modify_rowsets(std::vector<RowsetSharedPtr>& to_add,
-                        std::vector<RowsetSharedPtr>& to_delete, bool check_delete = false);
+    Status add_rowset(RowsetSharedPtr rowset);
+    Status create_initial_rowset(const int64_t version);
+    Status modify_rowsets(std::vector<RowsetSharedPtr>& to_add,
+                          std::vector<RowsetSharedPtr>& to_delete, bool check_delete = false);
 
     // _rs_version_map and _stale_rs_version_map should be protected by _meta_lock
     // The caller must call hold _meta_lock when call this two function.
@@ -112,7 +134,10 @@ public:
 
     const RowsetSharedPtr rowset_with_max_version() const;
 
-    OLAPStatus add_inc_rowset(const RowsetSharedPtr& rowset);
+    static RowsetMetaSharedPtr rowset_meta_with_max_schema_version(
+            const std::vector<RowsetMetaSharedPtr>& rowset_metas);
+
+    Status add_inc_rowset(const RowsetSharedPtr& rowset);
     /// Delete stale rowset by timing. This delete policy uses now() minutes
     /// config::tablet_rowset_expired_stale_sweep_time_sec to compute the deadline of expired rowset
     /// to delete.  When rowset is deleted, it will be added to StorageEngine unused map and record
@@ -121,49 +146,38 @@ public:
 
     // Given spec_version, find a continuous version path and store it in version_path.
     // If quiet is true, then only "does this path exist" is returned.
-    OLAPStatus capture_consistent_versions(const Version& spec_version,
-                                           std::vector<Version>* version_path,
-                                           bool quiet = false) const;
+    Status capture_consistent_versions(const Version& spec_version,
+                                       std::vector<Version>* version_path,
+                                       bool quiet = false) const;
     // if quiet is true, no error log will be printed if there are missing versions
-    OLAPStatus check_version_integrity(const Version& version, bool quiet = false);
+    Status check_version_integrity(const Version& version, bool quiet = false);
     bool check_version_exist(const Version& version) const;
     void acquire_version_and_rowsets(
             std::vector<std::pair<Version, RowsetSharedPtr>>* version_rowsets) const;
 
-    OLAPStatus capture_consistent_rowsets(const Version& spec_version,
-                                          std::vector<RowsetSharedPtr>* rowsets) const;
-    OLAPStatus capture_rs_readers(const Version& spec_version,
-                                  std::vector<RowsetReaderSharedPtr>* rs_readers,
-                                  std::shared_ptr<MemTracker> parent_tracker = nullptr) const;
+    Status capture_consistent_rowsets(const Version& spec_version,
+                                      std::vector<RowsetSharedPtr>* rowsets) const;
+    Status capture_rs_readers(const Version& spec_version,
+                              std::vector<RowsetReaderSharedPtr>* rs_readers) const;
 
-    OLAPStatus capture_rs_readers(const std::vector<Version>& version_path,
-                                  std::vector<RowsetReaderSharedPtr>* rs_readers,
-                                  std::shared_ptr<MemTracker> parent_tracker = nullptr) const;
+    Status capture_rs_readers(const std::vector<Version>& version_path,
+                              std::vector<RowsetReaderSharedPtr>* rs_readers) const;
 
-    DelPredicateArray delete_predicates() { return _tablet_meta->delete_predicates(); }
-    void add_delete_predicate(const DeletePredicatePB& delete_predicate, int64_t version);
+    const std::vector<RowsetMetaSharedPtr> delete_predicates() {
+        return _tablet_meta->delete_predicates();
+    }
     bool version_for_delete_predicate(const Version& version);
-    bool version_for_load_deletion(const Version& version);
 
     // meta lock
-    inline std::shared_mutex& get_header_lock() { return _meta_lock; }
+    std::shared_mutex& get_header_lock() { return _meta_lock; }
+    std::mutex& get_rowset_update_lock() { return _rowset_update_lock; }
+    std::mutex& get_push_lock() { return _ingest_lock; }
+    std::mutex& get_base_compaction_lock() { return _base_compaction_lock; }
+    std::mutex& get_cumulative_compaction_lock() { return _cumulative_compaction_lock; }
 
-    // ingest lock
-    inline void obtain_push_lock() { _ingest_lock.lock(); }
-    inline void release_push_lock() { _ingest_lock.unlock(); }
-    inline Mutex* get_push_lock() { return &_ingest_lock; }
+    std::shared_mutex& get_migration_lock() { return _migration_lock; }
 
-    // base lock
-    inline void obtain_base_compaction_lock() { _base_lock.lock(); }
-    inline void release_base_compaction_lock() { _base_lock.unlock(); }
-    inline Mutex* get_base_lock() { return &_base_lock; }
-
-    // cumulative lock
-    inline void obtain_cumulative_lock() { _cumulative_lock.lock(); }
-    inline void release_cumulative_lock() { _cumulative_lock.unlock(); }
-    inline Mutex* get_cumulative_lock() { return &_cumulative_lock; }
-
-    inline std::shared_mutex& get_migration_lock() { return _migration_lock; }
+    std::mutex& get_schema_change_lock() { return _schema_change_lock; }
 
     // operation for compaction
     bool can_do_compaction(size_t path_hash, CompactionType compaction_type);
@@ -182,8 +196,8 @@ public:
     void max_continuous_version_from_beginning(Version* version, Version* max_version = nullptr);
 
     // operation for query
-    OLAPStatus split_range(const OlapTuple& start_key_strings, const OlapTuple& end_key_strings,
-                           uint64_t request_block_row_count, std::vector<OlapTuple>* ranges);
+    Status split_range(const OlapTuple& start_key_strings, const OlapTuple& end_key_strings,
+                       uint64_t request_block_row_count, std::vector<OlapTuple>* ranges);
 
     void set_bad(bool is_bad) { _is_bad = is_bad; }
 
@@ -216,22 +230,25 @@ public:
     bool check_path(const std::string& check_path) const;
     bool check_rowset_id(const RowsetId& rowset_id);
 
-    OLAPStatus set_partition_id(int64_t partition_id);
+    Status set_partition_id(int64_t partition_id);
 
     TabletInfo get_tablet_info() const;
 
     void pick_candidate_rowsets_to_cumulative_compaction(
-            std::vector<RowsetSharedPtr>* candidate_rowsets);
-    void pick_candidate_rowsets_to_base_compaction(std::vector<RowsetSharedPtr>* candidate_rowsets);
+            std::vector<RowsetSharedPtr>* candidate_rowsets,
+            std::shared_lock<std::shared_mutex>& /* meta lock*/);
+    void pick_candidate_rowsets_to_base_compaction(
+            std::vector<RowsetSharedPtr>* candidate_rowsets,
+            std::shared_lock<std::shared_mutex>& /* meta lock*/);
 
     void calculate_cumulative_point();
     // TODO(ygl):
-    inline bool is_primary_replica() { return false; }
+    bool is_primary_replica() { return false; }
 
     // TODO(ygl):
     // eco mode means power saving in new energy car
     // eco mode also means save money in palo
-    inline bool in_eco_mode() { return false; }
+    bool in_eco_mode() { return false; }
 
     // return true if the checkpoint is actually done
     bool do_tablet_meta_checkpoint();
@@ -272,16 +289,103 @@ public:
     }
 
     inline bool all_beta() const {
-        ReadLock rdlock(_meta_lock);
+        std::shared_lock rdlock(_meta_lock);
         return _tablet_meta->all_beta();
     }
 
-    void find_alpha_rowsets(std::vector<RowsetSharedPtr>* rowsets) const;
+    TabletSchemaSPtr tablet_schema() const override;
+
+    TabletSchemaSPtr get_max_version_schema(std::lock_guard<std::shared_mutex>&);
+
+    // Find the related rowset with specified version and return its tablet schema
+    TabletSchemaSPtr tablet_schema(Version version) const {
+        return _tablet_meta->tablet_schema(version);
+    }
+
+    Status create_rowset_writer(const Version& version, const RowsetStatePB& rowset_state,
+                                const SegmentsOverlapPB& overlap, TabletSchemaSPtr tablet_schema,
+                                int64_t oldest_write_timestamp, int64_t newest_write_timestamp,
+                                std::unique_ptr<RowsetWriter>* rowset_writer);
+
+    Status create_rowset_writer(const Version& version, const RowsetStatePB& rowset_state,
+                                const SegmentsOverlapPB& overlap, TabletSchemaSPtr tablet_schema,
+                                int64_t oldest_write_timestamp, int64_t newest_write_timestamp,
+                                io::FileSystemSPtr fs,
+                                std::unique_ptr<RowsetWriter>* rowset_writer);
+
+    Status create_rowset_writer(const int64_t& txn_id, const PUniqueId& load_id,
+                                const RowsetStatePB& rowset_state, const SegmentsOverlapPB& overlap,
+                                TabletSchemaSPtr tablet_schema,
+                                std::unique_ptr<RowsetWriter>* rowset_writer);
+
+    Status create_vertical_rowset_writer(const Version& version, const RowsetStatePB& rowset_state,
+                                         const SegmentsOverlapPB& overlap,
+                                         TabletSchemaSPtr tablet_schema,
+                                         int64_t oldest_write_timestamp,
+                                         int64_t newest_write_timestamp,
+                                         std::unique_ptr<RowsetWriter>* rowset_writer);
+
+    Status create_rowset(RowsetMetaSharedPtr rowset_meta, RowsetSharedPtr* rowset);
+    // Cooldown to remote fs.
+    Status cooldown();
+
+    RowsetSharedPtr pick_cooldown_rowset();
+
+    bool need_cooldown(int64_t* cooldown_timestamp, size_t* file_size);
+
+    Status remove_all_remote_rowsets();
+
+    // Lookup the row location of `encoded_key`, the function sets `row_location` on success.
+    // NOTE: the method only works in unique key model with primary key index, you will got a
+    //       not supported error in other data model.
+    Status lookup_row_key(const Slice& encoded_key, const RowsetIdUnorderedSet* rowset_ids,
+                          RowLocation* row_location, uint32_t version);
+
+    // calc delete bitmap when flush memtable, use a fake version to calc
+    // For example, cur max version is 5, and we use version 6 to calc but
+    // finally this rowset publish version with 8, we should make up data
+    // for rowset 6-7. Also, if a compaction happens between commit_txn and
+    // publish_txn, we should remove compaction input rowsets' delete_bitmap
+    // and build newly generated rowset's delete_bitmap
+    Status calc_delete_bitmap(RowsetId rowset_id,
+                              const std::vector<segment_v2::SegmentSharedPtr>& segments,
+                              const RowsetIdUnorderedSet* specified_rowset_ids,
+                              DeleteBitmapPtr delete_bitmap, int64_t version,
+                              bool check_pre_segments = false);
+
+    Status update_delete_bitmap_without_lock(const RowsetSharedPtr& rowset);
+    Status update_delete_bitmap(const RowsetSharedPtr& rowset, const TabletTxnInfo* load_info);
+    void calc_compaction_output_rowset_delete_bitmap(
+            const std::vector<RowsetSharedPtr>& input_rowsets,
+            const RowIdConversion& rowid_conversion, uint64_t start_version, uint64_t end_version,
+            std::set<RowLocation>* missed_rows,
+            std::map<RowsetSharedPtr, std::list<std::pair<RowLocation, RowLocation>>>* location_map,
+            DeleteBitmap* output_rowset_delete_bitmap);
+    void merge_delete_bitmap(const DeleteBitmap& delete_bitmap);
+    Status check_rowid_conversion(
+            RowsetSharedPtr dst_rowset,
+            const std::map<RowsetSharedPtr, std::list<std::pair<RowLocation, RowLocation>>>&
+                    location_map);
+    RowsetIdUnorderedSet all_rs_id(int64_t max_version) const;
+
+    void remove_self_owned_remote_rowsets();
+
+    // Erase entries in `_self_owned_remote_rowsets` iff they are in `rowsets_in_snapshot`.
+    // REQUIRES: held _meta_lock
+    void update_self_owned_remote_rowsets(const std::vector<RowsetSharedPtr>& rowsets_in_snapshot);
+
+    void record_unused_remote_rowset(const RowsetId& rowset_id, const io::ResourceId& resource,
+                                     int64_t num_segments);
+
+    bool check_all_rowset_segment();
+
+    void update_max_version_schema(const TabletSchemaSPtr& tablet_schema);
+
 private:
-    OLAPStatus _init_once_action();
+    Status _init_once_action();
     void _print_missed_versions(const std::vector<Version>& missed_versions) const;
     bool _contains_rowset(const RowsetId rowset_id);
-    OLAPStatus _contains_version(const Version& version);
+    Status _contains_version(const Version& version);
 
     // Returns:
     // version: the max continuous version from beginning
@@ -292,8 +396,8 @@ private:
     /// Delete stale rowset by version. This method not only delete the version in expired rowset map,
     /// but also delete the version in rowset meta vector.
     void _delete_stale_rowset_by_version(const Version& version);
-    OLAPStatus _capture_consistent_rowsets_unlocked(const std::vector<Version>& version_path,
-                                                    std::vector<RowsetSharedPtr>* rowsets) const;
+    Status _capture_consistent_rowsets_unlocked(const std::vector<Version>& version_path,
+                                                std::vector<RowsetSharedPtr>* rowsets) const;
 
     const uint32_t _calc_cumulative_compaction_score(
             std::shared_ptr<CumulativeCompactionPolicy> cumulative_compaction_policy);
@@ -302,6 +406,16 @@ private:
     // When the proportion of empty edges in the adjacency matrix used to represent the version graph
     // in the version tracker is greater than the threshold, rebuild the version tracker
     bool _reconstruct_version_tracker_if_necessary();
+    void _init_context_common_fields(RowsetWriterContext& context);
+
+    Status _check_pk_in_pre_segments(RowsetId rowset_id,
+                                     const std::vector<segment_v2::SegmentSharedPtr>& pre_segments,
+                                     const Slice& key, DeleteBitmapPtr delete_bitmap,
+                                     RowLocation* loc);
+    void _rowset_ids_difference(const RowsetIdUnorderedSet& cur, const RowsetIdUnorderedSet& pre,
+                                RowsetIdUnorderedSet* to_add, RowsetIdUnorderedSet* to_del);
+    Status _load_rowset_segments(const RowsetSharedPtr& rowset,
+                                 std::vector<segment_v2::SegmentSharedPtr>* segments);
 
 public:
     static const int64_t K_INVALID_CUMULATIVE_POINT = -1;
@@ -309,18 +423,26 @@ public:
 private:
     TimestampedVersionTracker _timestamped_version_tracker;
 
-    DorisCallOnce<OLAPStatus> _init_once;
+    DorisCallOnce<Status> _init_once;
     // meta store lock is used for prevent 2 threads do checkpoint concurrently
     // it will be used in econ-mode in the future
     std::shared_mutex _meta_store_lock;
-    Mutex _ingest_lock;
-    Mutex _base_lock;
-    Mutex _cumulative_lock;
+    std::mutex _ingest_lock;
+    std::mutex _base_compaction_lock;
+    std::mutex _cumulative_compaction_lock;
+    std::mutex _schema_change_lock;
     std::shared_mutex _migration_lock;
 
     // TODO(lingbin): There is a _meta_lock TabletMeta too, there should be a comment to
     // explain how these two locks work together.
     mutable std::shared_mutex _meta_lock;
+
+    // In unique key table with MoW, we should guarantee that only one
+    // writer can update rowset and delete bitmap at the same time.
+    // We use a separate lock rather than _meta_lock, to avoid blocking read queries
+    // during publish_txn, which might take hundreds of milliseconds
+    mutable std::mutex _rowset_update_lock;
+
     // After version 0.13, all newly created rowsets are saved in _rs_version_map.
     // And if rowset being compacted, the old rowsetis will be saved in _stale_rs_version_map;
     std::unordered_map<Version, RowsetSharedPtr, HashOfVersion> _rs_version_map;
@@ -328,6 +450,9 @@ private:
     // These _stale rowsets are been removed when rowsets' pathVersion is expired,
     // this policy is judged and computed by TimestampedVersionTracker.
     std::unordered_map<Version, RowsetSharedPtr, HashOfVersion> _stale_rs_version_map;
+    // RowsetTree is used to locate rowsets containing a key or a key range quickly.
+    // It's only used in UNIQUE_KEYS data model.
+    std::unique_ptr<RowsetTree> _rowset_tree;
     // if this tablet is broken, set to true. default is false
     std::atomic<bool> _is_bad;
     // timestamp of last cumu compaction failure
@@ -364,11 +489,17 @@ private:
     int64_t _last_missed_version;
     int64_t _last_missed_time_s;
 
+    // Remote rowsets not shared by other BE. We can delete them when drop tablet.
+    std::unordered_set<RowsetSharedPtr> _self_owned_remote_rowsets; // guarded by _meta_lock
+
+    // Max schema_version schema from Rowset or FE
+    TabletSchemaSPtr _max_version_schema;
+
     DISALLOW_COPY_AND_ASSIGN(Tablet);
 
 public:
     IntCounter* flush_bytes;
-    IntCounter* flush_count;
+    IntCounter* flush_finish_count;
     std::atomic<int64_t> publised_count = 0;
 };
 
@@ -377,7 +508,7 @@ inline CumulativeCompactionPolicy* Tablet::cumulative_compaction_policy() {
 }
 
 inline bool Tablet::init_succeeded() {
-    return _init_once.has_called() && _init_once.stored_result() == OLAP_SUCCESS;
+    return _init_once.has_called() && _init_once.stored_result().ok();
 }
 
 inline bool Tablet::is_used() {
@@ -398,13 +529,12 @@ inline const int64_t Tablet::cumulative_layer_point() const {
 
 inline void Tablet::set_cumulative_layer_point(int64_t new_point) {
     // cumulative point should only be reset to -1, or be increased
-    if (new_point == Tablet::K_INVALID_CUMULATIVE_POINT || new_point >= _cumulative_point){
-        _cumulative_point = new_point;
-    } else {
-        LOG(WARNING) << "Unexpected cumulative point: " << new_point
+    CHECK(new_point == Tablet::K_INVALID_CUMULATIVE_POINT || new_point >= _cumulative_point)
+            << "Unexpected cumulative point: " << new_point
             << ", origin: " << _cumulative_point.load();
-    }
+    _cumulative_point = new_point;
 }
+
 inline const int64_t Tablet::cumulative_promotion_size() const {
     return _cumulative_promotion_size;
 }
@@ -413,21 +543,41 @@ inline void Tablet::set_cumulative_promotion_size(int64_t new_size) {
     _cumulative_promotion_size = new_size;
 }
 
+inline bool Tablet::enable_unique_key_merge_on_write() const {
+#ifdef BE_TEST
+    if (_tablet_meta == nullptr) {
+        return false;
+    }
+#endif
+    return _tablet_meta->enable_unique_key_merge_on_write();
+}
+
 // TODO(lingbin): Why other methods that need to get information from _tablet_meta
 // are not locked, here needs a comment to explain.
 inline size_t Tablet::tablet_footprint() {
-    ReadLock rdlock(_meta_lock);
+    std::shared_lock rdlock(_meta_lock);
     return _tablet_meta->tablet_footprint();
 }
 
+inline size_t Tablet::tablet_local_size() {
+    std::shared_lock rdlock(_meta_lock);
+    return _tablet_meta->tablet_local_size();
+}
+
+inline size_t Tablet::tablet_remote_size() {
+    std::shared_lock rdlock(_meta_lock);
+    return _tablet_meta->tablet_remote_size();
+}
+
 // TODO(lingbin): Why other methods which need to get information from _tablet_meta
 // are not locked, here needs a comment to explain.
 inline size_t Tablet::num_rows() {
-    ReadLock rdlock(_meta_lock);
+    std::shared_lock rdlock(_meta_lock);
     return _tablet_meta->num_rows();
 }
 
 inline int Tablet::version_count() const {
+    std::shared_lock rdlock(_meta_lock);
     return _tablet_meta->version_count();
 }
 
@@ -435,58 +585,65 @@ inline Version Tablet::max_version() const {
     return _tablet_meta->max_version();
 }
 
+inline uint64_t Tablet::segment_count() const {
+    std::shared_lock rdlock(_meta_lock);
+    uint64_t segment_nums = 0;
+    for (auto& rs_meta : _tablet_meta->all_rs_metas()) {
+        segment_nums += rs_meta->num_segments();
+    }
+    return segment_nums;
+}
+
+inline Version Tablet::max_version_unlocked() const {
+    return _tablet_meta->max_version();
+}
+
 inline KeysType Tablet::keys_type() const {
-    return _schema.keys_type();
+    return _schema->keys_type();
 }
 
 inline SortType Tablet::sort_type() const {
-    return _schema.sort_type();
+    return _schema->sort_type();
 }
 
 inline size_t Tablet::sort_col_num() const {
-    return _schema.sort_col_num();
+    return _schema->sort_col_num();
 }
 
 inline size_t Tablet::num_columns() const {
-    return _schema.num_columns();
+    return _schema->num_columns();
 }
 
 inline size_t Tablet::num_null_columns() const {
-    return _schema.num_null_columns();
+    return _schema->num_null_columns();
 }
 
 inline size_t Tablet::num_key_columns() const {
-    return _schema.num_key_columns();
+    return _schema->num_key_columns();
 }
 
 inline size_t Tablet::num_short_key_columns() const {
-    return _schema.num_short_key_columns();
+    return _schema->num_short_key_columns();
 }
 
 inline size_t Tablet::num_rows_per_row_block() const {
-    return _schema.num_rows_per_row_block();
+    return _schema->num_rows_per_row_block();
 }
 
 inline CompressKind Tablet::compress_kind() const {
-    return _schema.compress_kind();
+    return _schema->compress_kind();
 }
 
 inline double Tablet::bloom_filter_fpp() const {
-    return _schema.bloom_filter_fpp();
+    return _schema->bloom_filter_fpp();
 }
 
 inline size_t Tablet::next_unique_id() const {
-    return _schema.next_column_unique_id();
-}
-
-inline int32_t Tablet::field_index(const std::string& field_name) const {
-    return _schema.field_index(field_name);
+    return _schema->next_column_unique_id();
 }
 
 inline size_t Tablet::row_size() const {
-    return _schema.row_size();
+    return _schema->row_size();
 }
 
 } // namespace doris
-
-#endif // DORIS_BE_SRC_OLAP_TABLET_H
