@@ -35,16 +35,17 @@
 #include "olap/base_compaction.h"
 #include "olap/cumulative_compaction.h"
 #include "olap/data_dir.h"
+#include "olap/delta_writer.h"
 #include "olap/lru_cache.h"
 #include "olap/push_handler.h"
 #include "olap/reader.h"
-#include "olap/rowset/column_data_writer.h"
 #include "olap/rowset/rowset_meta_manager.h"
 #include "olap/schema_change.h"
 #include "olap/storage_engine.h"
 #include "olap/tablet_meta.h"
 #include "olap/tablet_meta_manager.h"
 #include "olap/utils.h"
+#include "rowset/beta_rowset.h"
 #include "util/doris_metrics.h"
 #include "util/pretty_printer.h"
 #include "util/time.h"
@@ -69,6 +70,7 @@ using std::stringstream;
 using std::vector;
 
 namespace doris {
+using namespace ErrorCode;
 
 TxnManager::TxnManager(int32_t txn_map_shard_size, int32_t txn_shard_size)
         : _txn_map_shard_size(txn_map_shard_size), _txn_shard_size(txn_shard_size) {
@@ -79,39 +81,40 @@ TxnManager::TxnManager(int32_t txn_map_shard_size, int32_t txn_shard_size)
     _txn_map_locks = new std::shared_mutex[_txn_map_shard_size];
     _txn_tablet_maps = new txn_tablet_map_t[_txn_map_shard_size];
     _txn_partition_maps = new txn_partition_map_t[_txn_map_shard_size];
-    _txn_mutex = new Mutex[_txn_shard_size];
+    _txn_mutex = new std::mutex[_txn_shard_size];
+    _txn_tablet_delta_writer_map = new txn_tablet_delta_writer_map_t[_txn_map_shard_size];
+    _txn_tablet_delta_writer_map_locks = new std::shared_mutex[_txn_map_shard_size];
 }
 
-OLAPStatus TxnManager::prepare_txn(TPartitionId partition_id, const TabletSharedPtr& tablet,
-                                   TTransactionId transaction_id, const PUniqueId& load_id) {
+Status TxnManager::prepare_txn(TPartitionId partition_id, const TabletSharedPtr& tablet,
+                               TTransactionId transaction_id, const PUniqueId& load_id) {
     return prepare_txn(partition_id, transaction_id, tablet->tablet_id(), tablet->schema_hash(),
                        tablet->tablet_uid(), load_id);
 }
 
-OLAPStatus TxnManager::commit_txn(TPartitionId partition_id, const TabletSharedPtr& tablet,
-                                  TTransactionId transaction_id, const PUniqueId& load_id,
-                                  const RowsetSharedPtr& rowset_ptr, bool is_recovery) {
+Status TxnManager::commit_txn(TPartitionId partition_id, const TabletSharedPtr& tablet,
+                              TTransactionId transaction_id, const PUniqueId& load_id,
+                              const RowsetSharedPtr& rowset_ptr, bool is_recovery) {
     return commit_txn(tablet->data_dir()->get_meta(), partition_id, transaction_id,
                       tablet->tablet_id(), tablet->schema_hash(), tablet->tablet_uid(), load_id,
                       rowset_ptr, is_recovery);
 }
 
-OLAPStatus TxnManager::publish_txn(TPartitionId partition_id, const TabletSharedPtr& tablet,
-                                   TTransactionId transaction_id, const Version& version,
-                                   PublishStatistic* stat) {
+Status TxnManager::publish_txn(TPartitionId partition_id, const TabletSharedPtr& tablet,
+                               TTransactionId transaction_id, const Version& version) {
     return publish_txn(tablet->data_dir()->get_meta(), partition_id, transaction_id,
-                       tablet->tablet_id(), tablet->schema_hash(), tablet->tablet_uid(), version, stat);
+                       tablet->tablet_id(), tablet->schema_hash(), tablet->tablet_uid(), version);
 }
 
 // delete the txn from manager if it is not committed(not have a valid rowset)
-OLAPStatus TxnManager::rollback_txn(TPartitionId partition_id, const TabletSharedPtr& tablet,
-                                    TTransactionId transaction_id) {
+Status TxnManager::rollback_txn(TPartitionId partition_id, const TabletSharedPtr& tablet,
+                                TTransactionId transaction_id) {
     return rollback_txn(partition_id, transaction_id, tablet->tablet_id(), tablet->schema_hash(),
                         tablet->tablet_uid());
 }
 
-OLAPStatus TxnManager::delete_txn(TPartitionId partition_id, const TabletSharedPtr& tablet,
-                                  TTransactionId transaction_id) {
+Status TxnManager::delete_txn(TPartitionId partition_id, const TabletSharedPtr& tablet,
+                              TTransactionId transaction_id) {
     return delete_txn(tablet->data_dir()->get_meta(), partition_id, transaction_id,
                       tablet->tablet_id(), tablet->schema_hash(), tablet->tablet_uid());
 }
@@ -119,12 +122,12 @@ OLAPStatus TxnManager::delete_txn(TPartitionId partition_id, const TabletSharedP
 // prepare txn should always be allowed because ingest task will be retried
 // could not distinguish rollup, schema change or base table, prepare txn successfully will allow
 // ingest retried
-OLAPStatus TxnManager::prepare_txn(TPartitionId partition_id, TTransactionId transaction_id,
-                                   TTabletId tablet_id, SchemaHash schema_hash,
-                                   TabletUid tablet_uid, const PUniqueId& load_id) {
+Status TxnManager::prepare_txn(TPartitionId partition_id, TTransactionId transaction_id,
+                               TTabletId tablet_id, SchemaHash schema_hash, TabletUid tablet_uid,
+                               const PUniqueId& load_id) {
     TxnKey key(partition_id, transaction_id);
     TabletInfo tablet_info(tablet_id, schema_hash, tablet_uid);
-    WriteLock txn_wrlock(_get_txn_map_lock(transaction_id));
+    std::lock_guard<std::shared_mutex> txn_wrlock(_get_txn_map_lock(transaction_id));
     txn_tablet_map_t& txn_tablet_map = _get_txn_tablet_map(transaction_id);
     auto it = txn_tablet_map.find(key);
     if (it != txn_tablet_map.end()) {
@@ -139,7 +142,7 @@ OLAPStatus TxnManager::prepare_txn(TPartitionId partition_id, TTransactionId tra
                 LOG(WARNING) << "find transaction exists when add to engine."
                              << "partition_id: " << key.first << ", transaction_id: " << key.second
                              << ", tablet: " << tablet_info.to_string();
-                return OLAP_SUCCESS;
+                return Status::OK();
             }
         }
     }
@@ -150,7 +153,7 @@ OLAPStatus TxnManager::prepare_txn(TPartitionId partition_id, TTransactionId tra
     if (txn_partition_map.size() > config::max_runnings_transactions_per_txn_map) {
         LOG(WARNING) << "too many transactions: " << txn_tablet_map.size()
                      << ", limit: " << config::max_runnings_transactions_per_txn_map;
-        return OLAP_ERR_TOO_MANY_TRANSACTIONS;
+        return Status::Error<TOO_MANY_TRANSACTIONS>();
     }
 
     // not found load id
@@ -163,14 +166,38 @@ OLAPStatus TxnManager::prepare_txn(TPartitionId partition_id, TTransactionId tra
     VLOG_NOTICE << "add transaction to engine successfully."
                 << "partition_id: " << key.first << ", transaction_id: " << key.second
                 << ", tablet: " << tablet_info.to_string();
-    return OLAP_SUCCESS;
+    return Status::OK();
 }
 
-OLAPStatus TxnManager::commit_txn(OlapMeta* meta, TPartitionId partition_id,
-                                  TTransactionId transaction_id, TTabletId tablet_id,
-                                  SchemaHash schema_hash, TabletUid tablet_uid,
-                                  const PUniqueId& load_id, const RowsetSharedPtr& rowset_ptr,
-                                  bool is_recovery) {
+void TxnManager::set_txn_related_delete_bitmap(
+        TPartitionId partition_id, TTransactionId transaction_id, TTabletId tablet_id,
+        SchemaHash schema_hash, TabletUid tablet_uid, bool unique_key_merge_on_write,
+        DeleteBitmapPtr delete_bitmap, const RowsetIdUnorderedSet& rowset_ids, uint64_t num_keys) {
+    pair<int64_t, int64_t> key(partition_id, transaction_id);
+    TabletInfo tablet_info(tablet_id, schema_hash, tablet_uid);
+
+    std::unique_lock<std::mutex> txn_lock(_get_txn_lock(transaction_id));
+    {
+        // get tx
+        std::shared_lock rdlock(_get_txn_map_lock(transaction_id));
+        txn_tablet_map_t& txn_tablet_map = _get_txn_tablet_map(transaction_id);
+        auto it = txn_tablet_map.find(key);
+        DCHECK(it != txn_tablet_map.end());
+        auto load_itr = it->second.find(tablet_info);
+        DCHECK(load_itr != it->second.end());
+        TabletTxnInfo& load_info = load_itr->second;
+        load_info.unique_key_merge_on_write = unique_key_merge_on_write;
+        load_info.delete_bitmap = delete_bitmap;
+        load_info.rowset_ids = rowset_ids;
+        load_info.num_keys = num_keys;
+    }
+}
+
+Status TxnManager::commit_txn(OlapMeta* meta, TPartitionId partition_id,
+                              TTransactionId transaction_id, TTabletId tablet_id,
+                              SchemaHash schema_hash, TabletUid tablet_uid,
+                              const PUniqueId& load_id, const RowsetSharedPtr& rowset_ptr,
+                              bool is_recovery) {
     if (partition_id < 1 || transaction_id < 1 || tablet_id < 1) {
         LOG(FATAL) << "invalid commit req "
                    << " partition_id=" << partition_id << " transaction_id=" << transaction_id
@@ -182,13 +209,13 @@ OLAPStatus TxnManager::commit_txn(OlapMeta* meta, TPartitionId partition_id,
         LOG(WARNING) << "could not commit txn because rowset ptr is null. "
                      << "partition_id: " << key.first << ", transaction_id: " << key.second
                      << ", tablet: " << tablet_info.to_string();
-        return OLAP_ERR_ROWSET_INVALID;
+        return Status::Error<ROWSET_INVALID>();
     }
 
-    MutexLock txn_lock(&_get_txn_lock(transaction_id));
+    std::unique_lock<std::mutex> txn_lock(_get_txn_lock(transaction_id));
     {
         // get tx
-        ReadLock rdlock(_get_txn_map_lock(transaction_id));
+        std::shared_lock rdlock(_get_txn_map_lock(transaction_id));
         txn_tablet_map_t& txn_tablet_map = _get_txn_tablet_map(transaction_id);
         auto it = txn_tablet_map.find(key);
         if (it != txn_tablet_map.end()) {
@@ -206,19 +233,20 @@ OLAPStatus TxnManager::commit_txn(OlapMeta* meta, TPartitionId partition_id,
                               << "partition_id: " << key.first << ", transaction_id: " << key.second
                               << ", tablet: " << tablet_info.to_string()
                               << ", rowset_id: " << load_info.rowset->rowset_id();
-                    return OLAP_SUCCESS;
+                    return Status::OK();
                 } else if (load_info.load_id.hi() == load_id.hi() &&
                            load_info.load_id.lo() == load_id.lo() && load_info.rowset != nullptr &&
                            load_info.rowset->rowset_id() != rowset_ptr->rowset_id()) {
                     // find a rowset with different rowset id, then it should not happen, just return errors
-                    LOG(WARNING) << "find rowset exists when commit transaction to engine. but rowset ids "
+                    LOG(WARNING) << "find rowset exists when commit transaction to engine. but "
+                                    "rowset ids "
                                     "are not same."
                                  << "partition_id: " << key.first
                                  << ", transaction_id: " << key.second
                                  << ", tablet: " << tablet_info.to_string()
                                  << ", exist rowset_id: " << load_info.rowset->rowset_id()
                                  << ", new rowset_id: " << rowset_ptr->rowset_id();
-                    return OLAP_ERR_PUSH_TRANSACTION_ALREADY_EXIST;
+                    return Status::Error<PUSH_TRANSACTION_ALREADY_EXIST>();
                 }
             }
         }
@@ -228,20 +256,28 @@ OLAPStatus TxnManager::commit_txn(OlapMeta* meta, TPartitionId partition_id,
     // save meta need access disk, it maybe very slow, so that it is not in global txn lock
     // it is under a single txn lock
     if (!is_recovery) {
-        OLAPStatus save_status =
-                RowsetMetaManager::save(meta, tablet_uid, rowset_ptr->rowset_id(),
-                                        rowset_ptr->rowset_meta()->get_rowset_pb());
-        if (save_status != OLAP_SUCCESS) {
+        Status save_status = RowsetMetaManager::save(meta, tablet_uid, rowset_ptr->rowset_id(),
+                                                     rowset_ptr->rowset_meta()->get_rowset_pb());
+        if (save_status != Status::OK()) {
             LOG(WARNING) << "save committed rowset failed. when commit txn rowset_id:"
                          << rowset_ptr->rowset_id() << "tablet id: " << tablet_id
                          << "txn id:" << transaction_id;
-            return OLAP_ERR_ROWSET_SAVE_FAILED;
+            return Status::Error<ROWSET_SAVE_FAILED>();
         }
     }
 
     {
-        WriteLock wrlock(_get_txn_map_lock(transaction_id));
+        std::lock_guard<std::shared_mutex> wrlock(_get_txn_map_lock(transaction_id));
         TabletTxnInfo load_info(load_id, rowset_ptr);
+        if (is_recovery) {
+            TabletSharedPtr tablet = StorageEngine::instance()->tablet_manager()->get_tablet(
+                    tablet_info.tablet_id, tablet_info.tablet_uid);
+            if (tablet != nullptr && tablet->enable_unique_key_merge_on_write()) {
+                load_info.unique_key_merge_on_write = true;
+                load_info.delete_bitmap.reset(new DeleteBitmap(tablet->tablet_id()));
+                load_info.num_keys = 0;
+            }
+        }
         txn_tablet_map_t& txn_tablet_map = _get_txn_tablet_map(transaction_id);
         txn_tablet_map[key][tablet_info] = load_info;
         _insert_txn_partition_map_unlocked(transaction_id, partition_id);
@@ -251,57 +287,69 @@ OLAPStatus TxnManager::commit_txn(OlapMeta* meta, TPartitionId partition_id,
                     << ", rowsetid: " << rowset_ptr->rowset_id()
                     << ", version: " << rowset_ptr->version().first;
     }
-    return OLAP_SUCCESS;
+    return Status::OK();
 }
 
 // remove a txn from txn manager
-OLAPStatus TxnManager::publish_txn(OlapMeta* meta, TPartitionId partition_id,
-                                   TTransactionId transaction_id, TTabletId tablet_id,
-                                   SchemaHash schema_hash, TabletUid tablet_uid,
-                                   const Version& version, PublishStatistic* stat) {
-    MonotonicStopWatch watch;
-    watch.start();
+Status TxnManager::publish_txn(OlapMeta* meta, TPartitionId partition_id,
+                               TTransactionId transaction_id, TTabletId tablet_id,
+                               SchemaHash schema_hash, TabletUid tablet_uid,
+                               const Version& version) {
     pair<int64_t, int64_t> key(partition_id, transaction_id);
     TabletInfo tablet_info(tablet_id, schema_hash, tablet_uid);
     RowsetSharedPtr rowset_ptr = nullptr;
-    MutexLock txn_lock(&_get_txn_lock(transaction_id));
+    TabletTxnInfo load_info;
     {
-        ReadLock rlock(_get_txn_map_lock(transaction_id));
-        stat->get_lock_time += watch.elapsed_time();
-        txn_tablet_map_t& txn_tablet_map = _get_txn_tablet_map(transaction_id);
-        auto it = txn_tablet_map.find(key);
-        if (it != txn_tablet_map.end()) {
-            auto load_itr = it->second.find(tablet_info);
-            if (load_itr != it->second.end()) {
-                // found load for txn,tablet
-                // case 1: user commit rowset, then the load id must be equal
-                TabletTxnInfo& load_info = load_itr->second;
-                rowset_ptr = load_info.rowset;
+        {
+            std::unique_lock<std::mutex> txn_rlock(_get_txn_lock(transaction_id));
+            std::shared_lock txn_map_rlock(_get_txn_map_lock(transaction_id));
+            txn_tablet_map_t& txn_tablet_map = _get_txn_tablet_map(transaction_id);
+            auto it = txn_tablet_map.find(key);
+            if (it != txn_tablet_map.end()) {
+                auto load_itr = it->second.find(tablet_info);
+                if (load_itr != it->second.end()) {
+                    // found load for txn,tablet
+                    // case 1: user commit rowset, then the load id must be equal
+                    load_info = load_itr->second;
+                    rowset_ptr = load_info.rowset;
+                }
             }
         }
-    }
-    // save meta need access disk, it maybe very slow, so that it is not in global txn lock
-    // it is under a single txn lock
-    if (rowset_ptr != nullptr) {
-        // TODO(ygl): rowset is already set version here, memory is changed, if save failed
-        // it maybe a fatal error
-        watch.reset();
-        rowset_ptr->make_visible(version);
-        OLAPStatus save_status =
-                RowsetMetaManager::save(meta, tablet_uid, rowset_ptr->rowset_id(),
-                                        rowset_ptr->rowset_meta()->get_rowset_pb());
-        if (save_status != OLAP_SUCCESS) {
-            LOG(WARNING) << "save committed rowset failed. when publish txn rowset_id:"
-                         << rowset_ptr->rowset_id() << ", tablet id: " << tablet_id
-                         << ", txn id:" << transaction_id;
-            return OLAP_ERR_ROWSET_SAVE_FAILED;
+        // save meta need access disk, it maybe very slow, so that it is not in global txn lock
+        // it is under a single txn lock
+        if (rowset_ptr != nullptr) {
+            // TODO(ygl): rowset is already set version here, memory is changed, if save failed
+            // it maybe a fatal error
+            rowset_ptr->make_visible(version);
+            // update delete_bitmap
+            {
+                if (load_info.unique_key_merge_on_write) {
+                    auto tablet =
+                            StorageEngine::instance()->tablet_manager()->get_tablet(tablet_id);
+                    if (tablet == nullptr) {
+                        return Status::OK();
+                    }
+                    RETURN_IF_ERROR(tablet->update_delete_bitmap(rowset_ptr, &load_info));
+                    std::shared_lock rlock(tablet->get_header_lock());
+                    tablet->save_meta();
+                }
+            }
+            Status save_status =
+                    RowsetMetaManager::save(meta, tablet_uid, rowset_ptr->rowset_id(),
+                                            rowset_ptr->rowset_meta()->get_rowset_pb());
+            if (save_status != Status::OK()) {
+                LOG(WARNING) << "save committed rowset failed. when publish txn rowset_id:"
+                             << rowset_ptr->rowset_id() << ", tablet id: " << tablet_id
+                             << ", txn id:" << transaction_id;
+                return Status::Error<ROWSET_SAVE_FAILED>();
+            }
+        } else {
+            return Status::Error<TRANSACTION_NOT_EXIST>();
         }
-        stat->save_meta_time += watch.elapsed_time();
-    } else {
-        return OLAP_ERR_TRANSACTION_NOT_EXIST;
     }
     {
-        WriteLock wrlock(_get_txn_map_lock(transaction_id));
+        std::unique_lock<std::mutex> txn_lock(_get_txn_lock(transaction_id));
+        std::lock_guard<std::shared_mutex> wrlock(_get_txn_map_lock(transaction_id));
         txn_tablet_map_t& txn_tablet_map = _get_txn_tablet_map(transaction_id);
         auto it = txn_tablet_map.find(key);
         if (it != txn_tablet_map.end()) {
@@ -309,27 +357,26 @@ OLAPStatus TxnManager::publish_txn(OlapMeta* meta, TPartitionId partition_id,
             VLOG_NOTICE << "publish txn successfully."
                         << " partition_id: " << key.first << ", txn_id: " << key.second
                         << ", tablet: " << tablet_info.to_string()
-                        << ", rowsetid: " << rowset_ptr->rowset_id() << ", version: " << version.first
-                        << "," << version.second;
+                        << ", rowsetid: " << rowset_ptr->rowset_id()
+                        << ", version: " << version.first << "," << version.second;
             if (it->second.empty()) {
                 txn_tablet_map.erase(it);
                 _clear_txn_partition_map_unlocked(transaction_id, partition_id);
             }
         }
-        return OLAP_SUCCESS;
     }
+    return Status::OK();
 }
 
 // txn could be rollbacked if it does not have related rowset
 // if the txn has related rowset then could not rollback it, because it
 // may be committed in another thread and our current thread meets errors when writing to data file
 // BE has to wait for fe call clear txn api
-OLAPStatus TxnManager::rollback_txn(TPartitionId partition_id, TTransactionId transaction_id,
-                                    TTabletId tablet_id, SchemaHash schema_hash,
-                                    TabletUid tablet_uid) {
+Status TxnManager::rollback_txn(TPartitionId partition_id, TTransactionId transaction_id,
+                                TTabletId tablet_id, SchemaHash schema_hash, TabletUid tablet_uid) {
     pair<int64_t, int64_t> key(partition_id, transaction_id);
     TabletInfo tablet_info(tablet_id, schema_hash, tablet_uid);
-    WriteLock wrlock(_get_txn_map_lock(transaction_id));
+    std::lock_guard<std::shared_mutex> wrlock(_get_txn_map_lock(transaction_id));
     txn_tablet_map_t& txn_tablet_map = _get_txn_tablet_map(transaction_id);
     auto it = txn_tablet_map.find(key);
     if (it != txn_tablet_map.end()) {
@@ -341,7 +388,7 @@ OLAPStatus TxnManager::rollback_txn(TPartitionId partition_id, TTransactionId tr
             if (load_info.rowset != nullptr) {
                 // if rowset is not null, it means other thread may commit the rowset
                 // should not delete txn any more
-                return OLAP_ERR_TRANSACTION_ALREADY_COMMITTED;
+                return Status::Error<TRANSACTION_ALREADY_COMMITTED>();
             }
         }
         it->second.erase(tablet_info);
@@ -353,21 +400,21 @@ OLAPStatus TxnManager::rollback_txn(TPartitionId partition_id, TTransactionId tr
             _clear_txn_partition_map_unlocked(transaction_id, partition_id);
         }
     }
-    return OLAP_SUCCESS;
+    return Status::OK();
 }
 
 // fe call this api to clear unused rowsets in be
 // could not delete the rowset if it already has a valid version
-OLAPStatus TxnManager::delete_txn(OlapMeta* meta, TPartitionId partition_id,
-                                  TTransactionId transaction_id, TTabletId tablet_id,
-                                  SchemaHash schema_hash, TabletUid tablet_uid) {
+Status TxnManager::delete_txn(OlapMeta* meta, TPartitionId partition_id,
+                              TTransactionId transaction_id, TTabletId tablet_id,
+                              SchemaHash schema_hash, TabletUid tablet_uid) {
     pair<int64_t, int64_t> key(partition_id, transaction_id);
     TabletInfo tablet_info(tablet_id, schema_hash, tablet_uid);
-    WriteLock txn_wrlock(_get_txn_map_lock(transaction_id));
+    std::lock_guard<std::shared_mutex> txn_wrlock(_get_txn_map_lock(transaction_id));
     txn_tablet_map_t& txn_tablet_map = _get_txn_tablet_map(transaction_id);
     auto it = txn_tablet_map.find(key);
     if (it == txn_tablet_map.end()) {
-        return OLAP_ERR_TRANSACTION_NOT_EXIST;
+        return Status::Error<TRANSACTION_NOT_EXIST>();
     }
     auto load_itr = it->second.find(tablet_info);
     if (load_itr != it->second.end()) {
@@ -383,7 +430,7 @@ OLAPStatus TxnManager::delete_txn(OlapMeta* meta, TPartitionId partition_id,
                              << ", tablet: " << tablet_info.to_string()
                              << ", rowset id: " << load_info.rowset->rowset_id()
                              << ", version: " << load_info.rowset->version().first;
-                return OLAP_ERR_TRANSACTION_ALREADY_VISIBLE;
+                return Status::Error<TRANSACTION_ALREADY_COMMITTED>();
             } else {
                 RowsetMetaManager::remove(meta, tablet_uid, load_info.rowset->rowset_id());
 #ifndef BE_TEST
@@ -403,7 +450,7 @@ OLAPStatus TxnManager::delete_txn(OlapMeta* meta, TPartitionId partition_id,
         txn_tablet_map.erase(it);
         _clear_txn_partition_map_unlocked(transaction_id, partition_id);
     }
-    return OLAP_SUCCESS;
+    return Status::OK();
 }
 
 void TxnManager::get_tablet_related_txns(TTabletId tablet_id, SchemaHash schema_hash,
@@ -416,7 +463,7 @@ void TxnManager::get_tablet_related_txns(TTabletId tablet_id, SchemaHash schema_
 
     TabletInfo tablet_info(tablet_id, schema_hash, tablet_uid);
     for (int32_t i = 0; i < _txn_map_shard_size; i++) {
-        ReadLock txn_rdlock(_txn_map_locks[i]);
+        std::shared_lock txn_rdlock(_txn_map_locks[i]);
         txn_tablet_map_t& txn_tablet_map = _txn_tablet_maps[i];
         for (auto& it : txn_tablet_map) {
             if (it.second.find(tablet_info) != it.second.end()) {
@@ -437,7 +484,7 @@ void TxnManager::force_rollback_tablet_related_txns(OlapMeta* meta, TTabletId ta
                                                     SchemaHash schema_hash, TabletUid tablet_uid) {
     TabletInfo tablet_info(tablet_id, schema_hash, tablet_uid);
     for (int32_t i = 0; i < _txn_map_shard_size; i++) {
-        WriteLock txn_wrlock(_txn_map_locks[i]);
+        std::lock_guard<std::shared_mutex> txn_wrlock(_txn_map_locks[i]);
         txn_tablet_map_t& txn_tablet_map = _txn_tablet_maps[i];
         for (auto it = txn_tablet_map.begin(); it != txn_tablet_map.end();) {
             auto load_itr = it->second.find(tablet_info);
@@ -473,7 +520,7 @@ void TxnManager::get_txn_related_tablets(const TTransactionId transaction_id,
                                          std::map<TabletInfo, RowsetSharedPtr>* tablet_infos) {
     // get tablets in this transaction
     pair<int64_t, int64_t> key(partition_id, transaction_id);
-    ReadLock txn_rdlock(_get_txn_map_lock(transaction_id));
+    std::shared_lock txn_rdlock(_get_txn_map_lock(transaction_id));
     txn_tablet_map_t& txn_tablet_map = _get_txn_tablet_map(transaction_id);
     auto it = txn_tablet_map.find(key);
     if (it == txn_tablet_map.end()) {
@@ -494,7 +541,7 @@ void TxnManager::get_txn_related_tablets(const TTransactionId transaction_id,
 
 void TxnManager::get_all_related_tablets(std::set<TabletInfo>* tablet_infos) {
     for (int32_t i = 0; i < _txn_map_shard_size; i++) {
-        ReadLock txn_rdlock(_txn_map_locks[i]);
+        std::shared_lock txn_rdlock(_txn_map_locks[i]);
         for (auto& it : _txn_tablet_maps[i]) {
             for (auto& tablet_load_it : it.second) {
                 tablet_infos->emplace(tablet_load_it.first);
@@ -507,7 +554,7 @@ bool TxnManager::has_txn(TPartitionId partition_id, TTransactionId transaction_i
                          TTabletId tablet_id, SchemaHash schema_hash, TabletUid tablet_uid) {
     pair<int64_t, int64_t> key(partition_id, transaction_id);
     TabletInfo tablet_info(tablet_id, schema_hash, tablet_uid);
-    ReadLock txn_rdlock(_get_txn_map_lock(transaction_id));
+    std::shared_lock txn_rdlock(_get_txn_map_lock(transaction_id));
     txn_tablet_map_t& txn_tablet_map = _get_txn_tablet_map(transaction_id);
     auto it = txn_tablet_map.find(key);
     bool found = it != txn_tablet_map.end() && it->second.find(tablet_info) != it->second.end();
@@ -519,7 +566,7 @@ void TxnManager::build_expire_txn_map(std::map<TabletInfo, std::vector<int64_t>>
     int64_t now = UnixSeconds();
     // traverse the txn map, and get all expired txns
     for (int32_t i = 0; i < _txn_map_shard_size; i++) {
-        ReadLock txn_rdlock(_txn_map_locks[i]);
+        std::shared_lock txn_rdlock(_txn_map_locks[i]);
         for (auto& it : _txn_tablet_maps[i]) {
             auto txn_id = it.first.second;
             for (auto& t_map : it.second) {
@@ -539,7 +586,7 @@ void TxnManager::build_expire_txn_map(std::map<TabletInfo, std::vector<int64_t>>
 
 void TxnManager::get_partition_ids(const TTransactionId transaction_id,
                                    std::vector<TPartitionId>* partition_ids) {
-    ReadLock txn_rdlock(_get_txn_map_lock(transaction_id));
+    std::shared_lock txn_rdlock(_get_txn_map_lock(transaction_id));
     txn_partition_map_t& txn_partition_map = _get_txn_partition_map(transaction_id);
     auto it = txn_partition_map.find(transaction_id);
     if (it != txn_partition_map.end()) {
@@ -569,4 +616,51 @@ void TxnManager::_clear_txn_partition_map_unlocked(int64_t transaction_id, int64
     }
 }
 
+void TxnManager::add_txn_tablet_delta_writer(int64_t transaction_id, int64_t tablet_id,
+                                             DeltaWriter* delta_writer) {
+    std::lock_guard<std::shared_mutex> txn_wrlock(
+            _get_txn_tablet_delta_writer_map_lock(transaction_id));
+    txn_tablet_delta_writer_map_t& txn_tablet_delta_writer_map =
+            _get_txn_tablet_delta_writer_map(transaction_id);
+    auto find = txn_tablet_delta_writer_map.find(transaction_id);
+    if (find == txn_tablet_delta_writer_map.end()) {
+        txn_tablet_delta_writer_map[transaction_id] = std::map<int64_t, DeltaWriter*>();
+    }
+    txn_tablet_delta_writer_map[transaction_id][tablet_id] = delta_writer;
+}
+
+void TxnManager::finish_slave_tablet_pull_rowset(int64_t transaction_id, int64_t tablet_id,
+                                                 int64_t node_id, bool is_succeed) {
+    std::lock_guard<std::shared_mutex> txn_wrlock(
+            _get_txn_tablet_delta_writer_map_lock(transaction_id));
+    txn_tablet_delta_writer_map_t& txn_tablet_delta_writer_map =
+            _get_txn_tablet_delta_writer_map(transaction_id);
+    auto find_txn = txn_tablet_delta_writer_map.find(transaction_id);
+    if (find_txn == txn_tablet_delta_writer_map.end()) {
+        LOG(WARNING) << "delta writer manager is not exist, txn_id=" << transaction_id
+                     << ", tablet_id=" << tablet_id;
+        return;
+    }
+    auto find_tablet = txn_tablet_delta_writer_map[transaction_id].find(tablet_id);
+    if (find_tablet == txn_tablet_delta_writer_map[transaction_id].end()) {
+        LOG(WARNING) << "delta writer is not exist, txn_id=" << transaction_id
+                     << ", tablet_id=" << tablet_id;
+        return;
+    }
+    DeltaWriter* delta_writer = txn_tablet_delta_writer_map[transaction_id][tablet_id];
+    delta_writer->finish_slave_tablet_pull_rowset(node_id, is_succeed);
+}
+
+void TxnManager::clear_txn_tablet_delta_writer(int64_t transaction_id) {
+    std::lock_guard<std::shared_mutex> txn_wrlock(
+            _get_txn_tablet_delta_writer_map_lock(transaction_id));
+    txn_tablet_delta_writer_map_t& txn_tablet_delta_writer_map =
+            _get_txn_tablet_delta_writer_map(transaction_id);
+    auto it = txn_tablet_delta_writer_map.find(transaction_id);
+    if (it != txn_tablet_delta_writer_map.end()) {
+        txn_tablet_delta_writer_map.erase(it);
+    }
+    VLOG_CRITICAL << "remove delta writer manager, txn_id=" << transaction_id;
+}
+
 } // namespace doris
