@@ -15,8 +15,7 @@
 // specific language governing permissions and limitations
 // under the License.
 
-#ifndef DORIS_BE_SRC_OLAP_TXN_MANAGER_H
-#define DORIS_BE_SRC_OLAP_TXN_MANAGER_H
+#pragma once
 
 #include <pthread.h>
 #include <rapidjson/document.h>
@@ -43,18 +42,38 @@
 #include "olap/options.h"
 #include "olap/rowset/rowset.h"
 #include "olap/rowset/rowset_meta.h"
+#include "olap/rowset/segment_v2/segment.h"
 #include "olap/tablet.h"
+#include "olap/tablet_meta.h"
 #include "util/time.h"
 
 namespace doris {
+class DeltaWriter;
 
 struct TabletTxnInfo {
     PUniqueId load_id;
     RowsetSharedPtr rowset;
+    bool unique_key_merge_on_write;
+    DeleteBitmapPtr delete_bitmap;
+    // records rowsets calc in commit txn
+    RowsetIdUnorderedSet rowset_ids;
     int64_t creation_time;
+    uint64_t num_keys;
 
     TabletTxnInfo(PUniqueId load_id, RowsetSharedPtr rowset)
-            : load_id(load_id), rowset(rowset), creation_time(UnixSeconds()) {}
+            : load_id(load_id),
+              rowset(rowset),
+              unique_key_merge_on_write(false),
+              creation_time(UnixSeconds()) {}
+
+    TabletTxnInfo(PUniqueId load_id, RowsetSharedPtr rowset, bool merge_on_write,
+                  DeleteBitmapPtr delete_bitmap, const RowsetIdUnorderedSet& ids)
+            : load_id(load_id),
+              rowset(rowset),
+              unique_key_merge_on_write(merge_on_write),
+              delete_bitmap(delete_bitmap),
+              rowset_ids(ids),
+              creation_time(UnixSeconds()) {}
 
     TabletTxnInfo() {}
 };
@@ -69,51 +88,53 @@ public:
         delete[] _txn_partition_maps;
         delete[] _txn_map_locks;
         delete[] _txn_mutex;
+        delete[] _txn_tablet_delta_writer_map;
+        delete[] _txn_tablet_delta_writer_map_locks;
     }
 
-    OLAPStatus prepare_txn(TPartitionId partition_id, const TabletSharedPtr& tablet,
-                           TTransactionId transaction_id, const PUniqueId& load_id);
+    Status prepare_txn(TPartitionId partition_id, const TabletSharedPtr& tablet,
+                       TTransactionId transaction_id, const PUniqueId& load_id);
 
-    OLAPStatus commit_txn(TPartitionId partition_id, const TabletSharedPtr& tablet,
-                          TTransactionId transaction_id, const PUniqueId& load_id,
-                          const RowsetSharedPtr& rowset_ptr, bool is_recovery);
+    Status commit_txn(TPartitionId partition_id, const TabletSharedPtr& tablet,
+                      TTransactionId transaction_id, const PUniqueId& load_id,
+                      const RowsetSharedPtr& rowset_ptr, bool is_recovery);
 
-    OLAPStatus publish_txn(TPartitionId partition_id, const TabletSharedPtr& tablet,
-                           TTransactionId transaction_id, const Version& version, PublishStatistic* stat = nullptr);
+    Status publish_txn(TPartitionId partition_id, const TabletSharedPtr& tablet,
+                       TTransactionId transaction_id, const Version& version);
 
     // delete the txn from manager if it is not committed(not have a valid rowset)
-    OLAPStatus rollback_txn(TPartitionId partition_id, const TabletSharedPtr& tablet,
-                            TTransactionId transaction_id);
+    Status rollback_txn(TPartitionId partition_id, const TabletSharedPtr& tablet,
+                        TTransactionId transaction_id);
 
-    OLAPStatus delete_txn(TPartitionId partition_id, const TabletSharedPtr& tablet,
-                          TTransactionId transaction_id);
+    Status delete_txn(TPartitionId partition_id, const TabletSharedPtr& tablet,
+                      TTransactionId transaction_id);
 
     // add a txn to manager
     // partition id is useful in publish version stage because version is associated with partition
-    OLAPStatus prepare_txn(TPartitionId partition_id, TTransactionId transaction_id,
-                           TTabletId tablet_id, SchemaHash schema_hash, TabletUid tablet_uid,
-                           const PUniqueId& load_id);
+    Status prepare_txn(TPartitionId partition_id, TTransactionId transaction_id,
+                       TTabletId tablet_id, SchemaHash schema_hash, TabletUid tablet_uid,
+                       const PUniqueId& load_id);
 
-    OLAPStatus commit_txn(OlapMeta* meta, TPartitionId partition_id, TTransactionId transaction_id,
-                          TTabletId tablet_id, SchemaHash schema_hash, TabletUid tablet_uid,
-                          const PUniqueId& load_id, const RowsetSharedPtr& rowset_ptr,
-                          bool is_recovery);
+    Status commit_txn(OlapMeta* meta, TPartitionId partition_id, TTransactionId transaction_id,
+                      TTabletId tablet_id, SchemaHash schema_hash, TabletUid tablet_uid,
+                      const PUniqueId& load_id, const RowsetSharedPtr& rowset_ptr,
+                      bool is_recovery);
 
     // remove a txn from txn manager
     // not persist rowset meta because
-    OLAPStatus publish_txn(OlapMeta* meta, TPartitionId partition_id, TTransactionId transaction_id,
-                           TTabletId tablet_id, SchemaHash schema_hash, TabletUid tablet_uid,
-                           const Version& version, PublishStatistic* stat = nullptr);
+    Status publish_txn(OlapMeta* meta, TPartitionId partition_id, TTransactionId transaction_id,
+                       TTabletId tablet_id, SchemaHash schema_hash, TabletUid tablet_uid,
+                       const Version& version);
 
     // delete the txn from manager if it is not committed(not have a valid rowset)
-    OLAPStatus rollback_txn(TPartitionId partition_id, TTransactionId transaction_id,
-                            TTabletId tablet_id, SchemaHash schema_hash, TabletUid tablet_uid);
+    Status rollback_txn(TPartitionId partition_id, TTransactionId transaction_id,
+                        TTabletId tablet_id, SchemaHash schema_hash, TabletUid tablet_uid);
 
     // remove the txn from txn manager
     // delete the related rowset if it is not null
     // delete rowset related data if it is not null
-    OLAPStatus delete_txn(OlapMeta* meta, TPartitionId partition_id, TTransactionId transaction_id,
-                          TTabletId tablet_id, SchemaHash schema_hash, TabletUid tablet_uid);
+    Status delete_txn(OlapMeta* meta, TPartitionId partition_id, TTransactionId transaction_id,
+                      TTabletId tablet_id, SchemaHash schema_hash, TabletUid tablet_uid);
 
     void get_tablet_related_txns(TTabletId tablet_id, SchemaHash schema_hash, TabletUid tablet_uid,
                                  int64_t* partition_id, std::set<int64_t>* transaction_ids);
@@ -123,11 +144,11 @@ public:
 
     void get_all_related_tablets(std::set<TabletInfo>* tablet_infos);
 
-    // just check if the txn exists
+    // Just check if the txn exists.
     bool has_txn(TPartitionId partition_id, TTransactionId transaction_id, TTabletId tablet_id,
                  SchemaHash schema_hash, TabletUid tablet_uid);
 
-    // get all expired txns and save them in expire_txn_map.
+    // Get all expired txns and save them in expire_txn_map.
     // This is currently called before reporting all tablet info, to avoid iterating txn map for every tablets.
     void build_expire_txn_map(std::map<TabletInfo, std::vector<int64_t>>* expire_txn_map);
 
@@ -137,10 +158,22 @@ public:
     void get_partition_ids(const TTransactionId transaction_id,
                            std::vector<TPartitionId>* partition_ids);
 
+    void add_txn_tablet_delta_writer(int64_t transaction_id, int64_t tablet_id,
+                                     DeltaWriter* delta_writer);
+    void clear_txn_tablet_delta_writer(int64_t transaction_id);
+    void finish_slave_tablet_pull_rowset(int64_t transaction_id, int64_t tablet_id, int64_t node_id,
+                                         bool is_succeed);
+
+    void set_txn_related_delete_bitmap(TPartitionId partition_id, TTransactionId transaction_id,
+                                       TTabletId tablet_id, SchemaHash schema_hash,
+                                       TabletUid tablet_uid, bool unique_key_merge_on_write,
+                                       DeleteBitmapPtr delete_bitmap,
+                                       const RowsetIdUnorderedSet& rowset_ids, uint64_t num_keys);
+
 private:
     using TxnKey = std::pair<int64_t, int64_t>; // partition_id, transaction_id;
 
-    // implement TxnKey hash function to support TxnKey as a key for unordered_map
+    // Implement TxnKey hash function to support TxnKey as a key for `unordered_map`.
     struct TxnKeyHash {
         template <typename T, typename U>
         size_t operator()(const std::pair<T, U>& e) const {
@@ -148,7 +181,7 @@ private:
         }
     };
 
-    // implement TxnKey equal function to support TxnKey as a key for unordered_map
+    // Implement TxnKey equal function to support TxnKey as a key for `unordered_map`.
     struct TxnKeyEqual {
         template <class T, typename U>
         bool operator()(const std::pair<T, U>& l, const std::pair<T, U>& r) const {
@@ -159,17 +192,23 @@ private:
     typedef std::unordered_map<TxnKey, std::map<TabletInfo, TabletTxnInfo>, TxnKeyHash, TxnKeyEqual>
             txn_tablet_map_t;
     typedef std::unordered_map<int64_t, std::unordered_set<int64_t>> txn_partition_map_t;
+    typedef std::unordered_map<int64_t, std::map<int64_t, DeltaWriter*>>
+            txn_tablet_delta_writer_map_t;
 
-    inline std::shared_mutex& _get_txn_map_lock(TTransactionId transactionId);
+    std::shared_mutex& _get_txn_map_lock(TTransactionId transactionId);
 
-    inline txn_tablet_map_t& _get_txn_tablet_map(TTransactionId transactionId);
+    txn_tablet_map_t& _get_txn_tablet_map(TTransactionId transactionId);
 
-    inline txn_partition_map_t& _get_txn_partition_map(TTransactionId transactionId);
+    txn_partition_map_t& _get_txn_partition_map(TTransactionId transactionId);
 
-    inline Mutex& _get_txn_lock(TTransactionId transactionId);
+    inline std::mutex& _get_txn_lock(TTransactionId transactionId);
 
-    // insert or remove (transaction_id, partition_id) from _txn_partition_map
-    // get _txn_map_lock before calling
+    std::shared_mutex& _get_txn_tablet_delta_writer_map_lock(TTransactionId transactionId);
+
+    txn_tablet_delta_writer_map_t& _get_txn_tablet_delta_writer_map(TTransactionId transactionId);
+
+    // Insert or remove (transaction_id, partition_id) from _txn_partition_map
+    // get _txn_map_lock before calling.
     void _insert_txn_partition_map_unlocked(int64_t transaction_id, int64_t partition_id);
     void _clear_txn_partition_map_unlocked(int64_t transaction_id, int64_t partition_id);
 
@@ -188,7 +227,10 @@ private:
 
     std::shared_mutex* _txn_map_locks;
 
-    Mutex* _txn_mutex;
+    std::mutex* _txn_mutex;
+
+    txn_tablet_delta_writer_map_t* _txn_tablet_delta_writer_map;
+    std::shared_mutex* _txn_tablet_delta_writer_map_locks;
     DISALLOW_COPY_AND_ASSIGN(TxnManager);
 }; // TxnManager
 
@@ -205,9 +247,18 @@ inline TxnManager::txn_partition_map_t& TxnManager::_get_txn_partition_map(
     return _txn_partition_maps[transactionId & (_txn_map_shard_size - 1)];
 }
 
-inline Mutex& TxnManager::_get_txn_lock(TTransactionId transactionId) {
+inline std::mutex& TxnManager::_get_txn_lock(TTransactionId transactionId) {
     return _txn_mutex[transactionId & (_txn_shard_size - 1)];
 }
 
+inline std::shared_mutex& TxnManager::_get_txn_tablet_delta_writer_map_lock(
+        TTransactionId transactionId) {
+    return _txn_tablet_delta_writer_map_locks[transactionId & (_txn_map_shard_size - 1)];
+}
+
+inline TxnManager::txn_tablet_delta_writer_map_t& TxnManager::_get_txn_tablet_delta_writer_map(
+        TTransactionId transactionId) {
+    return _txn_tablet_delta_writer_map[transactionId & (_txn_map_shard_size - 1)];
+}
+
 } // namespace doris
-#endif // DORIS_BE_SRC_OLAP_TXN_MANAGER_H
