@@ -14,29 +14,23 @@
 // KIND, either express or implied.  See the License for the
 // specific language governing permissions and limitations
 // under the License.
+// This file is copied from
+// https://github.com/apache/impala/blob/branch-2.9.0/be/src/runtime/runtime-state.h
+// and modified by Doris
 
-#ifndef DORIS_BE_SRC_QUERY_RUNTIME_RUNTIME_STATE_H
-#define DORIS_BE_SRC_QUERY_RUNTIME_RUNTIME_STATE_H
+#pragma once
 
-#include <atomic>
 #include <fstream>
-#include <memory>
-#include <mutex>
-#include <sstream>
-#include <string>
-#include <vector>
 
 #include "cctz/time_zone.h"
 #include "common/global_types.h"
 #include "common/object_pool.h"
 #include "gen_cpp/PaloInternalService_types.h" // for TQueryOptions
 #include "gen_cpp/Types_types.h"               // for TUniqueId
-#include "runtime/mem_pool.h"
 #include "runtime/query_fragments_ctx.h"
 #include "runtime/thread_resource_mgr.h"
-#include "util/logging.h"
 #include "util/runtime_profile.h"
-#include "runtime/thread_context.h"
+#include "util/telemetry/telemetry.h"
 
 namespace doris {
 
@@ -72,6 +66,9 @@ public:
     // RuntimeState for executing expr in fe-support.
     RuntimeState(const TQueryGlobals& query_globals);
 
+    // for job task only
+    RuntimeState();
+
     // Empty d'tor to avoid issues with unique_ptr.
     ~RuntimeState();
 
@@ -79,15 +76,8 @@ public:
     Status init(const TUniqueId& fragment_instance_id, const TQueryOptions& query_options,
                 const TQueryGlobals& query_globals, ExecEnv* exec_env);
 
-    // Set up four-level hierarchy of mem trackers: process, query, fragment instance.
-    // The instance tracker is tied to our profile.
-    // Specific parts of the fragment (i.e. exec nodes, sinks, data stream senders, etc)
-    // will add a fourth level when they are initialized.
-    // This function also initializes a user function mem tracker (in the fourth level).
-    Status init_mem_trackers(const TUniqueId& query_id);
-
-    // for ut only
-    Status init_instance_mem_tracker();
+    // for ut and non-query.
+    Status init_mem_trackers(const TUniqueId& query_id = TUniqueId());
 
     // Gets/Creates the query wide block mgr.
     Status create_block_mgr();
@@ -100,18 +90,19 @@ public:
     std::shared_ptr<ObjectPool> obj_pool_ptr() const { return _obj_pool; }
 
     const DescriptorTbl& desc_tbl() const { return *_desc_tbl; }
-    void set_desc_tbl(DescriptorTbl* desc_tbl) { _desc_tbl = desc_tbl; }
+    void set_desc_tbl(const DescriptorTbl* desc_tbl) { _desc_tbl = desc_tbl; }
     int batch_size() const { return _query_options.batch_size; }
-    int num_free_block_in_scan() const { return _query_options.num_free_block_in_scan; }
     bool abort_on_error() const { return _query_options.abort_on_error; }
     bool abort_on_default_limit_exceeded() const {
         return _query_options.abort_on_default_limit_exceeded;
     }
     int max_errors() const { return _query_options.max_errors; }
+    int query_timeout() const { return _query_options.query_timeout; }
     int max_io_buffers() const { return _query_options.max_io_buffers; }
     int num_scanner_threads() const { return _query_options.num_scanner_threads; }
     TQueryType::type query_type() const { return _query_options.query_type; }
     int64_t timestamp_ms() const { return _timestamp_ms; }
+    int32_t nano_seconds() const { return _nano_seconds; }
     const std::string& timezone() const { return _timezone; }
     const cctz::time_zone& timezone_obj() const { return _timezone_obj; }
     const std::string& user() const { return _user; }
@@ -119,12 +110,7 @@ public:
     const TUniqueId& query_id() const { return _query_id; }
     const TUniqueId& fragment_instance_id() const { return _fragment_instance_id; }
     ExecEnv* exec_env() { return _exec_env; }
-    const std::vector<std::shared_ptr<MemTracker>>& mem_trackers() { return _mem_trackers; }
-    std::shared_ptr<MemTracker> fragment_mem_tracker() { return _fragment_mem_tracker; }
-    std::shared_ptr<MemTracker> query_mem_tracker() { return _query_mem_tracker; }
-    std::shared_ptr<MemTracker> instance_mem_tracker() { return _instance_mem_tracker; }
-    std::shared_ptr<MemTrackerLimiter> new_query_mem_tracker() { return _new_query_mem_tracker; }
-    std::shared_ptr<MemTrackerLimiter> new_instance_mem_tracker() { return _new_instance_mem_tracker; }
+    std::shared_ptr<MemTrackerLimiter> query_mem_tracker() { return _query_mem_tracker; }
     ThreadResourceMgr::ResourcePool* resource_pool() { return _resource_pool; }
 
     void set_fragment_root_id(PlanNodeId id) {
@@ -142,6 +128,16 @@ public:
     // Returns true if codegen is enabled for this query.
     bool codegen_enabled() const { return !_query_options.disable_codegen; }
 
+    bool enable_function_pushdown() const {
+        return _query_options.__isset.enable_function_pushdown &&
+               _query_options.enable_function_pushdown;
+    }
+
+    bool check_overflow_for_decimal() const {
+        return _query_options.__isset.check_overflow_for_decimal &&
+               _query_options.check_overflow_for_decimal;
+    }
+
     // Create a codegen object in _codegen. No-op if it has already been called.
     // If codegen is enabled for the query, this is created when the runtime
     // state is created. If codegen is disabled for the query, this is created
@@ -158,28 +154,9 @@ public:
         return _process_status;
     };
 
-    //    MemPool* udf_pool() {
-    //        return _udf_pool.get();
-    //    };
-
-    // Create and return a stream receiver for _fragment_instance_id
-    // from the data stream manager. The receiver is added to _data_stream_recvrs_pool.
-    DataStreamRecvr* create_recvr(const RowDescriptor& row_desc, PlanNodeId dest_node_id,
-                                  int num_senders, int buffer_size, RuntimeProfile* profile);
-
-    // Sets the fragment memory limit and adds it to _mem_trackers
-    void set_fragment_mem_tracker(std::shared_ptr<MemTracker> tracker) {
-        DCHECK(_fragment_mem_tracker == nullptr);
-        _fragment_mem_tracker = tracker;
-        _mem_trackers.push_back(tracker);
-    }
-
     // Appends error to the _error_log if there is space
     bool log_error(const std::string& error);
 
-    // If !status.ok(), appends the error to the _error_log
-    void log_error(const Status& status);
-
     // Returns true if the error log has not reached _max_errors.
     bool log_has_space() {
         std::lock_guard<std::mutex> l(_error_log_lock);
@@ -204,7 +181,7 @@ public:
     int64_t backend_id() const { return _backend_id; }
 
     void set_be_number(int be_number) { _be_number = be_number; }
-    int be_number(void) { return _be_number; }
+    int be_number(void) const { return _be_number; }
 
     // Sets _process_status with err_msg if no error has been set yet.
     void set_process_status(const std::string& err_msg) {
@@ -255,7 +232,7 @@ public:
 
     void set_load_job_id(int64_t job_id) { _load_job_id = job_id; }
 
-    const int64_t load_job_id() { return _load_job_id; }
+    const int64_t load_job_id() const { return _load_job_id; }
 
     // we only initialize object for load jobs
     void set_load_error_hub_info(const TLoadErrorHubInfo& hub_info) {
@@ -282,8 +259,9 @@ public:
     // append error msg and error line to file when loading data.
     // is_summary is true, means we are going to write the summary line
     // If we need to stop the processing, set stop_processing to true
-    Status append_error_msg_to_file(std::function<std::string()> line, std::function<std::string()> error_msg,
-                                    bool* stop_processing, bool is_summary = false);
+    Status append_error_msg_to_file(std::function<std::string()> line,
+                                    std::function<std::string()> error_msg, bool* stop_processing,
+                                    bool is_summary = false);
 
     int64_t num_bytes_load_total() { return _num_bytes_load_total.load(); }
 
@@ -331,12 +309,21 @@ public:
 
     bool enable_spill() const { return _query_options.enable_spilling; }
 
-    int32_t runtime_filter_wait_time_ms() { return _query_options.runtime_filter_wait_time_ms; }
+    int32_t runtime_filter_wait_time_ms() const {
+        return _query_options.runtime_filter_wait_time_ms;
+    }
 
-    int32_t runtime_filter_max_in_num() { return _query_options.runtime_filter_max_in_num; }
+    int32_t runtime_filter_max_in_num() const { return _query_options.runtime_filter_max_in_num; }
 
     bool enable_vectorized_exec() const { return _query_options.enable_vectorized_engine; }
 
+    int be_exec_version() const {
+        if (!_query_options.__isset.be_exec_version) {
+            return 0;
+        }
+        return _query_options.be_exec_version;
+    }
+
     bool trim_tailing_spaces_for_external_table_query() const {
         return _query_options.trim_tailing_spaces_for_external_table_query;
     }
@@ -349,15 +336,42 @@ public:
         return _query_options.enable_enable_exchange_node_parallel_merge;
     }
 
+    segment_v2::CompressionTypePB fragement_transmission_compression_type() const {
+        if (_query_options.__isset.fragment_transmission_compression_codec) {
+            if (_query_options.fragment_transmission_compression_codec == "lz4") {
+                return segment_v2::CompressionTypePB::LZ4;
+            }
+        }
+        return segment_v2::CompressionTypePB::SNAPPY;
+    }
+
+    bool skip_storage_engine_merge() const {
+        return _query_options.__isset.skip_storage_engine_merge &&
+               _query_options.skip_storage_engine_merge;
+    }
+
+    bool skip_delete_predicate() const {
+        return _query_options.__isset.skip_delete_predicate && _query_options.skip_delete_predicate;
+    }
+
+    bool skip_delete_bitmap() const {
+        return _query_options.__isset.skip_delete_bitmap && _query_options.skip_delete_bitmap;
+    }
+
+    int partitioned_hash_join_rows_threshold() const {
+        if (!_query_options.__isset.partitioned_hash_join_rows_threshold) {
+            return 0;
+        }
+        return _query_options.partitioned_hash_join_rows_threshold;
+    }
+
     const std::vector<TTabletCommitInfo>& tablet_commit_infos() const {
         return _tablet_commit_infos;
     }
 
     std::vector<TTabletCommitInfo>& tablet_commit_infos() { return _tablet_commit_infos; }
 
-    const std::vector<TErrorTabletInfo>& error_tablet_infos() const {
-        return _error_tablet_infos;
-    }
+    const std::vector<TErrorTabletInfo>& error_tablet_infos() const { return _error_tablet_infos; }
 
     std::vector<TErrorTabletInfo>& error_tablet_infos() { return _error_tablet_infos; }
 
@@ -374,6 +388,21 @@ public:
 
     QueryFragmentsCtx* get_query_fragments_ctx() { return _query_ctx; }
 
+    void set_query_mem_tracker(const std::shared_ptr<MemTrackerLimiter>& tracker) {
+        _query_mem_tracker = tracker;
+    }
+
+    OpentelemetryTracer get_tracer() { return _tracer; }
+
+    void set_tracer(OpentelemetryTracer&& tracer) { _tracer = std::move(tracer); }
+
+    bool enable_profile() const { return _query_options.is_report_success; }
+
+    bool enable_share_hash_table_for_broadcast_join() const {
+        return _query_options.__isset.enable_share_hash_table_for_broadcast_join &&
+               _query_options.enable_share_hash_table_for_broadcast_join;
+    }
+
 private:
     // Use a custom block manager for the query for testing purposes.
     void set_block_mgr2(const std::shared_ptr<BufferedBlockMgr2>& block_mgr) {
@@ -384,31 +413,13 @@ private:
 
     static const int DEFAULT_BATCH_SIZE = 2048;
 
-    // all mem limits that apply to this query
-    std::vector<std::shared_ptr<MemTracker>> _mem_trackers;
-
-    // Fragment memory limit.  Also contained in _mem_trackers
-    std::shared_ptr<MemTracker> _fragment_mem_tracker;
-
-    // MemTracker that is shared by all fragment instances running on this host.
-    // The query mem tracker must be released after the _instance_mem_tracker.
-    std::shared_ptr<MemTracker> _query_mem_tracker;
-
-    // Memory usage of this fragment instance
-    std::shared_ptr<MemTracker> _instance_mem_tracker;
-
-    // MemTracker that is shared by all fragment instances running on this host.
-    // The query mem tracker must be released after the _instance_mem_tracker.
-    std::shared_ptr<MemTrackerLimiter> _new_query_mem_tracker;
-
-    // Memory usage of this fragment instance
-    std::shared_ptr<MemTrackerLimiter> _new_instance_mem_tracker;
+    std::shared_ptr<MemTrackerLimiter> _query_mem_tracker;
 
     // put runtime state before _obj_pool, so that it will be deconstructed after
     // _obj_pool. Because some of object in _obj_pool will use profile when deconstructing.
     RuntimeProfile _profile;
 
-    DescriptorTbl* _desc_tbl;
+    const DescriptorTbl* _desc_tbl;
     std::shared_ptr<ObjectPool> _obj_pool;
 
     // runtime filter
@@ -438,6 +449,7 @@ private:
 
     //Query-global timestamp_ms
     int64_t _timestamp_ms;
+    int32_t _nano_seconds;
     std::string _timezone;
     cctz::time_zone _timezone_obj;
 
@@ -515,6 +527,8 @@ private:
     // true if max_filter_ratio is 0
     bool _load_zero_tolerance = false;
 
+    OpentelemetryTracer _tracer = telemetry::get_noop_tracer();
+
     // prohibit copies
     RuntimeState(const RuntimeState&);
 };
@@ -525,5 +539,3 @@ private:
     } while (false)
 
 } // namespace doris
-
-#endif // end of DORIS_BE_SRC_QUERY_RUNTIME_RUNTIME_STATE_H
