@@ -15,13 +15,13 @@
 // specific language governing permissions and limitations
 // under the License.
 
-#ifndef DORIS_BE_SRC_UTIL_BITMAP_VALUE_H
-#define DORIS_BE_SRC_UTIL_BITMAP_VALUE_H
+#pragma once
 
 #include <parallel_hashmap/btree.h>
 
 #include <algorithm>
 #include <cstdarg>
+#include <cstdint>
 #include <cstdio>
 #include <limits>
 #include <map>
@@ -33,9 +33,11 @@
 #include <utility>
 
 #include "common/logging.h"
+#include "gutil/integral_types.h"
 #include "udf/udf.h"
 #include "util/coding.h"
-
+#include "vec/common/pod_array.h"
+#include "vec/common/pod_array_fwd.h"
 namespace doris {
 
 // serialized bitmap := TypeCode(1), Payload
@@ -72,6 +74,16 @@ struct BitmapTypeCode {
         // added in 0.12
         BITMAP64 = 4
     };
+    Status static inline validate(int bitmap_type) {
+        if (UNLIKELY(bitmap_type < type::EMPTY || bitmap_type > type::BITMAP64)) {
+            std::string err_msg =
+                    fmt::format("BitmapTypeCode invalid, should between: {} and {} actrual is {}",
+                                BitmapTypeCode::EMPTY, BitmapTypeCode::BITMAP64, bitmap_type);
+            LOG(ERROR) << err_msg;
+            return Status::IOError(err_msg);
+        }
+        return Status::OK();
+    }
 };
 
 namespace detail {
@@ -1023,13 +1035,13 @@ public:
         return false;
     }
 
-    bool operator==(const Roaring64MapSetBitForwardIterator& o) {
+    bool operator==(const Roaring64MapSetBitForwardIterator& o) const {
         if (map_iter == map_end && o.map_iter == o.map_end) return true;
         if (o.map_iter == o.map_end) return false;
         return **this == *o;
     }
 
-    bool operator!=(const Roaring64MapSetBitForwardIterator& o) {
+    bool operator!=(const Roaring64MapSetBitForwardIterator& o) const {
         if (map_iter == map_end && o.map_iter == o.map_end) return false;
         if (o.map_iter == o.map_end) return true;
         return **this != *o;
@@ -1253,6 +1265,54 @@ public:
         return *this;
     }
 
+    BitmapValue& fastunion(const std::vector<const BitmapValue*>& values) {
+        std::vector<const detail::Roaring64Map*> bitmaps;
+        std::vector<uint64_t> single_values;
+        for (int i = 0; i < values.size(); ++i) {
+            auto* value = values[i];
+            switch (value->_type) {
+            case EMPTY:
+                break;
+            case SINGLE:
+                single_values.push_back(value->_sv);
+                break;
+            case BITMAP:
+                bitmaps.push_back(&value->_bitmap);
+                break;
+            }
+        }
+
+        if (!bitmaps.empty()) {
+            switch (_type) {
+            case EMPTY:
+                _bitmap = detail::Roaring64Map::fastunion(bitmaps.size(), bitmaps.data());
+                _type = BITMAP;
+                break;
+            case SINGLE:
+                _bitmap = detail::Roaring64Map::fastunion(bitmaps.size(), bitmaps.data());
+                _bitmap.add(_sv);
+                _type = BITMAP;
+                break;
+            case BITMAP:
+                _bitmap |= detail::Roaring64Map::fastunion(bitmaps.size(), bitmaps.data());
+                break;
+            }
+        }
+
+        if (_type == EMPTY && single_values.size() == 1) {
+            _sv = single_values[0];
+            _type = SINGLE;
+        } else if (!single_values.empty()) {
+            _bitmap.addMany(single_values.size(), single_values.data());
+            if (_type == SINGLE) {
+                _bitmap.add(_sv);
+            }
+            _type = BITMAP;
+        }
+
+        return *this;
+    }
+
     // Compute the intersection between the current bitmap and the provided bitmap.
     // Possible type transitions are:
     // SINGLE -> EMPTY
@@ -1372,6 +1432,9 @@ public:
         return false;
     }
 
+    // true if contains a value that belongs to the range [left, right].
+    bool contains_any(uint64_t left, uint64_t right) const;
+
     uint64_t cardinality() const {
         switch (_type) {
         case EMPTY:
@@ -1537,7 +1600,6 @@ public:
     // Deserialize a bitmap value from `src`.
     // Return false if `src` begins with unknown type code, true otherwise.
     bool deserialize(const char* src) {
-        DCHECK(*src >= BitmapTypeCode::EMPTY && *src <= BitmapTypeCode::BITMAP64);
         switch (*src) {
         case BitmapTypeCode::EMPTY:
             _type = EMPTY;
@@ -1556,6 +1618,9 @@ public:
             _bitmap = detail::Roaring64Map::read(src);
             break;
         default:
+            LOG(ERROR) << "BitmapTypeCode invalid, should between: " << BitmapTypeCode::EMPTY
+                       << " and " << BitmapTypeCode::BITMAP64 << " actrual is "
+                       << static_cast<int>(*src);
             return false;
         }
         return true;
@@ -1617,6 +1682,16 @@ public:
         }
     }
 
+    uint64_t max(bool* empty) const {
+        return min_or_max(empty, [&]() { return _bitmap.maximum(); });
+    }
+
+    uint64_t min(bool* empty) const {
+        return min_or_max(empty, [&]() { return _bitmap.minimum(); });
+    }
+
+    bool empty() const { return _type == EMPTY; }
+
     /**
      * Return new set with specified range (not include the range_end)
      */
@@ -1733,6 +1808,24 @@ public:
         return count;
     }
 
+    //for function bitmap_to_array
+    void to_array(vectorized::PaddedPODArray<int64_t>& data) const {
+        switch (_type) {
+        case EMPTY:
+            break;
+        case SINGLE: {
+            data.emplace_back(_sv);
+            break;
+        }
+        case BITMAP: {
+            for (auto it = _bitmap.begin(); it != _bitmap.end(); ++it) {
+                data.emplace_back(*it);
+            }
+            break;
+        }
+        }
+    }
+
     void clear() {
         _type = EMPTY;
         _bitmap.clear();
@@ -1745,6 +1838,7 @@ public:
 
     b_iterator begin() const;
     b_iterator end() const;
+    b_iterator lower_bound(uint64_t val) const;
 
 private:
     void _convert_to_smaller_type() {
@@ -1761,6 +1855,25 @@ private:
         }
     }
 
+    uint64_t min_or_max(bool* empty, std::function<uint64_t()> func) const {
+        bool is_empty = false;
+        uint64_t result = 0;
+        switch (_type) {
+        case SINGLE:
+            result = _sv;
+            break;
+        case BITMAP:
+            result = func();
+            break;
+        default:
+            is_empty = true;
+        }
+        if (empty) {
+            *empty = is_empty;
+        }
+        return result;
+    }
+
     enum BitmapDataType {
         EMPTY = 0,
         SINGLE = 1, // single element
@@ -1798,7 +1911,9 @@ public:
     }
 
     BitmapValueIterator(const BitmapValueIterator& other)
-            : _bitmap(other._bitmap), _iter(other._iter), _sv(other._sv), _end(other._end) {}
+            : _bitmap(other._bitmap), _sv(other._sv), _end(other._end) {
+        _iter = other._iter ? new detail::Roaring64MapSetBitForwardIterator(*other._iter) : nullptr;
+    }
 
     ~BitmapValueIterator() {
         if (_iter != nullptr) {
@@ -1852,13 +1967,15 @@ public:
     }
 
     bool operator==(const BitmapValueIterator& other) const {
-        if (_end && other._end) return true;
+        if (_end && other._end) {
+            return true;
+        }
 
         switch (_bitmap._type) {
         case BitmapValue::BitmapDataType::EMPTY:
             return other._bitmap._type == BitmapValue::BitmapDataType::EMPTY;
         case BitmapValue::BitmapDataType::SINGLE:
-            return _sv == other._sv;
+            return _end == other._end && _sv == other._sv;
         case BitmapValue::BitmapDataType::BITMAP:
             return *_iter == *(other._iter);
         default:
@@ -1869,6 +1986,27 @@ public:
 
     bool operator!=(const BitmapValueIterator& other) const { return !(*this == other); }
 
+    /**
+    * Move the iterator to the first value >= `val`.
+    */
+    BitmapValueIterator& move(uint64_t val) {
+        switch (_bitmap._type) {
+        case BitmapValue::BitmapDataType::SINGLE:
+            if (_sv < val) {
+                _end = true;
+            }
+            break;
+        case BitmapValue::BitmapDataType::BITMAP:
+            if (!_iter->move(val)) {
+                _end = true;
+            }
+            break;
+        default:
+            break;
+        }
+        return *this;
+    }
+
 private:
     const BitmapValue& _bitmap;
     detail::Roaring64MapSetBitForwardIterator* _iter = nullptr;
@@ -1884,6 +2022,16 @@ inline BitmapValueIterator BitmapValue::end() const {
     return BitmapValueIterator(*this, true);
 }
 
-} // namespace doris
+inline BitmapValueIterator BitmapValue::lower_bound(uint64_t val) const {
+    return BitmapValueIterator(*this).move(val);
+}
 
-#endif //DORIS_BE_SRC_UTIL_BITMAP_VALUE_H
+inline bool BitmapValue::contains_any(uint64_t left, uint64_t right) const {
+    if (left > right) {
+        return false;
+    }
+    auto it = lower_bound(left);
+    return it != end() && *it <= right;
+}
+
+} // namespace doris
