@@ -47,6 +47,7 @@ public:
     bool is_numeric() const override { return false; }
 
     bool is_bitmap() const override { return std::is_same_v<T, BitmapValue>; }
+    bool is_hll() const override { return std::is_same_v<T, HyperLogLog>; }
 
     size_t size() const override { return data.size(); }
 
@@ -55,47 +56,48 @@ public:
     }
 
     void insert_from(const IColumn& src, size_t n) override {
-        data.push_back(static_cast<const Self&>(src).get_data()[n]);
+        data.push_back(assert_cast<const Self&>(src).get_data()[n]);
     }
 
     void insert_data(const char* pos, size_t /*length*/) override {
         data.push_back(*reinterpret_cast<const T*>(pos));
     }
 
-    void insert_many_binary_data(char* data_array, uint32_t* len_array, uint32_t* start_offset_array, size_t num) override {
+    void insert_binary_data(const char* pos, size_t length) {
+        insert_default();
+        T* pvalue = &get_element(size() - 1);
+        if (!length) {
+            *pvalue = *reinterpret_cast<const T*>(pos);
+            return;
+        }
+
         if constexpr (std::is_same_v<T, BitmapValue>) {
-            for (size_t i = 0; i < num; i++) {
-                uint32_t len = len_array[i];
-                uint32_t start_offset = start_offset_array[i];
-                insert_default();
-                BitmapValue* pvalue = &get_element(size() - 1);
-                if (len != 0) {
-                    BitmapValue value;
-                    value.deserialize(data_array + start_offset);
-                    *pvalue = std::move(value);
-                } else {
-                    *pvalue = std::move(*reinterpret_cast<BitmapValue*>(data_array + start_offset));   
-                }
-            }
+            pvalue->deserialize(pos);
         } else if constexpr (std::is_same_v<T, HyperLogLog>) {
-            for (size_t i = 0; i < num; i++) {
-                uint32_t len = len_array[i];
-                uint32_t start_offset = start_offset_array[i];
-                insert_default();
-                HyperLogLog* pvalue = &get_element(size() - 1);
-                if (len != 0) {
-                    HyperLogLog value;
-                    value.deserialize(Slice(data_array + start_offset, len));
-                    *pvalue = std::move(value);
-                } else {
-                    *pvalue = std::move(*reinterpret_cast<HyperLogLog*>(data_array + start_offset));
-                }
-            }
+            pvalue->deserialize(Slice(pos, length));
         } else {
             LOG(FATAL) << "Unexpected type in column complex";
         }
     }
 
+    void insert_many_continuous_binary_data(const char* data, const uint32_t* offsets,
+                                            const size_t num) override {
+        if (UNLIKELY(num == 0)) {
+            return;
+        }
+
+        for (size_t i = 0; i != num; ++i) {
+            insert_binary_data(data + offsets[i], offsets[i + 1] - offsets[i]);
+        }
+    }
+
+    void insert_many_binary_data(char* data_array, uint32_t* len_array,
+                                 uint32_t* start_offset_array, size_t num) override {
+        for (size_t i = 0; i < num; i++) {
+            insert_binary_data(data_array + start_offset_array[i], len_array[i]);
+        }
+    }
+
     void insert_default() override { data.push_back(T()); }
 
     void insert_many_defaults(size_t length) override {
@@ -160,7 +162,7 @@ public:
     }
 
     void insert_range_from(const IColumn& src, size_t start, size_t length) override {
-        auto& col = static_cast<const Self&>(src);
+        auto& col = assert_cast<const Self&>(src);
         auto& src_data = col.get_data();
         auto st = src_data.begin() + start;
         auto ed = st + length;
@@ -175,7 +177,7 @@ public:
         for (int i = 0; i < new_size; ++i) {
             auto offset = *(indices_begin + i);
             if (offset == -1) {
-                data.emplace_back(T{});
+                data.emplace_back(T {});
             } else {
                 data.emplace_back(src_vec.get_element(offset));
             }
@@ -183,7 +185,7 @@ public:
     }
 
     void pop_back(size_t n) override { data.erase(data.end() - n, data.end()); }
-    // it's impossable to use ComplexType as key , so we don't have to implemnt them
+    // it's impossible to use ComplexType as key , so we don't have to implement them
     [[noreturn]] StringRef serialize_value_into_arena(size_t n, Arena& arena,
                                                       char const*& begin) const override {
         LOG(FATAL) << "serialize_value_into_arena not implemented";
@@ -193,10 +195,23 @@ public:
         LOG(FATAL) << "deserialize_and_insert_from_arena not implemented";
     }
 
+    // maybe we do not need to impl the function
     void update_hash_with_value(size_t n, SipHash& hash) const override {
         // TODO add hash function
     }
 
+    virtual void update_hashes_with_value(
+            std::vector<SipHash>& hashes,
+            const uint8_t* __restrict null_data = nullptr) const override {
+            // TODO add hash function
+    };
+
+    virtual void update_hashes_with_value(
+            uint64_t* __restrict hashes,
+            const uint8_t* __restrict null_data = nullptr) const override {
+        // TODO add hash function
+    }
+
     [[noreturn]] int compare_at(size_t n, size_t m, const IColumn& rhs,
                                 int nan_direction_hint) const override {
         LOG(FATAL) << "compare_at not implemented";
@@ -233,16 +248,22 @@ public:
 
     ColumnPtr replicate(const IColumn::Offsets& replicate_offsets) const override;
 
-    void replicate(const uint32_t* counts, size_t target_size, IColumn& column) const override;
+    void replicate(const uint32_t* counts, size_t target_size, IColumn& column, size_t begin = 0,
+                   int count_sz = -1) const override;
 
     [[noreturn]] MutableColumns scatter(IColumn::ColumnIndex num_columns,
                                         const IColumn::Selector& selector) const override {
         LOG(FATAL) << "scatter not implemented";
     }
 
+    void append_data_by_selector(MutableColumnPtr& res,
+                                 const IColumn::Selector& selector) const override {
+        this->template append_data_by_selector_impl<ColumnComplexType<T>>(res, selector);
+    }
+
     void replace_column_data(const IColumn& rhs, size_t row, size_t self_row = 0) override {
         DCHECK(size() > self_row);
-        data[self_row] = static_cast<const Self&>(rhs).data[row];
+        data[self_row] = assert_cast<const Self&>(rhs).data[row];
     }
 
     void replace_column_data_default(size_t self_row = 0) override {
@@ -259,7 +280,7 @@ MutableColumnPtr ColumnComplexType<T>::clone_resized(size_t size) const {
     auto res = this->create();
 
     if (size > 0) {
-        auto& new_col = static_cast<Self&>(*res);
+        auto& new_col = assert_cast<Self&>(*res);
         new_col.data = this->data;
     }
 
@@ -298,10 +319,7 @@ template <typename T>
 ColumnPtr ColumnComplexType<T>::permute(const IColumn::Permutation& perm, size_t limit) const {
     size_t size = data.size();
 
-    if (limit == 0)
-        limit = size;
-    else
-        limit = std::min(size, limit);
+    limit = limit ? std::min(size, limit) : size;
 
     if (perm.size() < limit) {
         LOG(FATAL) << "Size of permutation is less than required.";
@@ -343,16 +361,17 @@ ColumnPtr ColumnComplexType<T>::replicate(const IColumn::Offsets& offsets) const
 }
 
 template <typename T>
-void ColumnComplexType<T>::replicate(const uint32_t* counts, size_t target_size,
-                                     IColumn& column) const {
-    size_t size = data.size();
+void ColumnComplexType<T>::replicate(const uint32_t* counts, size_t target_size, IColumn& column,
+                                     size_t begin, int count_sz) const {
+    size_t size = count_sz < 0 ? data.size() : count_sz;
     if (0 == size) return;
 
     auto& res = reinterpret_cast<ColumnComplexType<T>&>(column);
     typename Self::Container& res_data = res.get_data();
     res_data.reserve(target_size);
 
-    for (size_t i = 0; i < size; ++i) {
+    size_t end = size + begin;
+    for (size_t i = begin; i < end; ++i) {
         size_t size_to_replicate = counts[i];
         for (size_t j = 0; j < size_to_replicate; ++j) {
             res_data.push_back(data[i]);
@@ -367,11 +386,11 @@ template <typename T>
 struct is_complex : std::false_type {};
 
 template <>
-struct is_complex<BitmapValue> : std::true_type {};  
+struct is_complex<BitmapValue> : std::true_type {};
 //DataTypeBitMap::FieldType = BitmapValue
 
 template <>
-struct is_complex<HyperLogLog> : std::true_type {};  
+struct is_complex<HyperLogLog> : std::true_type {};
 //DataTypeHLL::FieldType = HyperLogLog
 
 template <class T>
