@@ -19,23 +19,27 @@
 
 #include <iostream>
 
-#include "common/logging.h"
 #include "env/env.h"
+#include "io/fs/file_system.h"
+#include "io/fs/file_writer.h"
+#include "io/fs/local_file_system.h"
 #include "olap/column_block.h"
 #include "olap/decimal12.h"
-#include "olap/fs/fs_util.h"
 #include "olap/olap_common.h"
 #include "olap/rowset/segment_v2/column_reader.h"
 #include "olap/rowset/segment_v2/column_writer.h"
 #include "olap/tablet_schema_helper.h"
 #include "olap/types.h"
 #include "runtime/mem_pool.h"
-#include "runtime/mem_tracker.h"
 #include "testutil/test_util.h"
 #include "util/file_utils.h"
 #include "vec/core/types.h"
+#include "vec/data_types/data_type_date.h"
+#include "vec/data_types/data_type_date_time.h"
+#include "vec/data_types/data_type_decimal.h"
 #include "vec/data_types/data_type_nothing.h"
 #include "vec/data_types/data_type_number.h"
+#include "vec/data_types/data_type_string.h"
 
 using std::string;
 
@@ -46,8 +50,8 @@ static const std::string TEST_DIR = "./ut_dir/column_reader_writer_test";
 
 class ColumnReaderWriterTest : public testing::Test {
 public:
-    ColumnReaderWriterTest() : _tracker(new MemTracker()), _pool(_tracker.get()) {}
-    virtual ~ColumnReaderWriterTest() {}
+    ColumnReaderWriterTest() : _pool() {}
+    ~ColumnReaderWriterTest() override = default;
 
 protected:
     void SetUp() override {
@@ -65,7 +69,6 @@ protected:
     }
 
 private:
-    std::shared_ptr<MemTracker> _tracker;
     MemPool _pool;
 };
 
@@ -79,11 +82,11 @@ void test_nullable_data(uint8_t* src_data, uint8_t* src_is_null, int num_rows,
 
     // write data
     std::string fname = TEST_DIR + "/" + test_name;
+    auto fs = io::global_local_filesystem();
     {
-        std::unique_ptr<fs::WritableBlock> wblock;
-        fs::CreateBlockOptions opts(fname);
-        Status st = fs::fs_util::block_manager(TStorageMedium::HDD)->create_block(opts, &wblock);
-        EXPECT_TRUE(st.ok()) << st.get_error_msg();
+        io::FileWriterPtr file_writer;
+        Status st = fs->create_file(fname, &file_writer);
+        EXPECT_TRUE(st.ok()) << st;
 
         ColumnWriterOptions writer_opts;
         writer_opts.meta = &meta;
@@ -107,7 +110,7 @@ void test_nullable_data(uint8_t* src_data, uint8_t* src_is_null, int num_rows,
             column = create_char_key(1);
         }
         std::unique_ptr<ColumnWriter> writer;
-        ColumnWriter::create(writer_opts, &column, wblock.get(), &writer);
+        ColumnWriter::create(writer_opts, &column, file_writer.get(), &writer);
         st = writer->init();
         EXPECT_TRUE(st.ok()) << st.to_string();
 
@@ -122,41 +125,35 @@ void test_nullable_data(uint8_t* src_data, uint8_t* src_is_null, int num_rows,
         EXPECT_TRUE(writer->write_zone_map().ok());
 
         // close the file
-        EXPECT_TRUE(wblock->close().ok());
+        EXPECT_TRUE(file_writer->close().ok());
     }
     auto type_info = get_scalar_type_info(type);
+    io::FileReaderSPtr file_reader;
+    ASSERT_EQ(fs->open_file(fname, &file_reader), Status::OK());
     // read and check
     {
         // sequence read
         {
             ColumnReaderOptions reader_opts;
-            FilePathDesc path_desc;
-            path_desc.filepath = fname;
             std::unique_ptr<ColumnReader> reader;
-            auto st = ColumnReader::create(reader_opts, meta, num_rows, path_desc, &reader);
+            auto st = ColumnReader::create(reader_opts, meta, num_rows, file_reader, &reader);
             EXPECT_TRUE(st.ok());
 
             ColumnIterator* iter = nullptr;
             st = reader->new_iterator(&iter);
             EXPECT_TRUE(st.ok());
-            std::unique_ptr<fs::ReadableBlock> rblock;
-            fs::BlockManager* block_manager = fs::fs_util::block_manager(TStorageMedium::HDD);
-            block_manager->open_block(path_desc, &rblock);
 
-            EXPECT_TRUE(st.ok());
             ColumnIteratorOptions iter_opts;
             OlapReaderStatistics stats;
             iter_opts.stats = &stats;
-            iter_opts.rblock = rblock.get();
-            iter_opts.mem_tracker = std::make_shared<MemTracker>();
+            iter_opts.file_reader = file_reader.get();
             st = iter->init(iter_opts);
             EXPECT_TRUE(st.ok());
 
             st = iter->seek_to_first();
             EXPECT_TRUE(st.ok()) << st.to_string();
 
-            auto tracker = std::make_shared<MemTracker>();
-            MemPool pool(tracker.get());
+            MemPool pool;
             std::unique_ptr<ColumnVectorBatch> cvb;
             ColumnVectorBatch::create(0, true, type_info, nullptr, &cvb);
             cvb->resize(1024);
@@ -191,30 +188,23 @@ void test_nullable_data(uint8_t* src_data, uint8_t* src_is_null, int num_rows,
 
         {
             ColumnReaderOptions reader_opts;
-            FilePathDesc path_desc;
-            path_desc.filepath = fname;
             std::unique_ptr<ColumnReader> reader;
-            auto st = ColumnReader::create(reader_opts, meta, num_rows, path_desc, &reader);
+            auto st = ColumnReader::create(reader_opts, meta, num_rows, file_reader, &reader);
             EXPECT_TRUE(st.ok());
 
             ColumnIterator* iter = nullptr;
             st = reader->new_iterator(&iter);
             EXPECT_TRUE(st.ok());
-            std::unique_ptr<fs::ReadableBlock> rblock;
-            fs::BlockManager* block_manager = fs::fs_util::block_manager(TStorageMedium::HDD);
-            block_manager->open_block(path_desc, &rblock);
 
             EXPECT_TRUE(st.ok());
             ColumnIteratorOptions iter_opts;
             OlapReaderStatistics stats;
             iter_opts.stats = &stats;
-            iter_opts.rblock = rblock.get();
-            iter_opts.mem_tracker = std::make_shared<MemTracker>();
+            iter_opts.file_reader = file_reader.get();
             st = iter->init(iter_opts);
             EXPECT_TRUE(st.ok());
 
-            auto tracker = std::make_shared<MemTracker>();
-            MemPool pool(tracker.get());
+            MemPool pool;
             std::unique_ptr<ColumnVectorBatch> cvb;
             ColumnVectorBatch::create(0, true, type_info, nullptr, &cvb);
             cvb->resize(1024);
@@ -265,11 +255,11 @@ void test_array_nullable_data(CollectionValue* src_data, uint8_t* src_is_null, i
 
     // write data
     std::string fname = TEST_DIR + "/" + test_name;
+    auto fs = io::global_local_filesystem();
     {
-        std::unique_ptr<fs::WritableBlock> wblock;
-        fs::CreateBlockOptions opts(fname);
-        Status st = fs::fs_util::block_manager(TStorageMedium::HDD)->create_block(opts, &wblock);
-        EXPECT_TRUE(st.ok()) << st.get_error_msg();
+        io::FileWriterPtr file_writer;
+        Status st = fs->create_file(fname, &file_writer);
+        EXPECT_TRUE(st.ok()) << st;
 
         ColumnWriterOptions writer_opts;
         writer_opts.meta = &meta;
@@ -293,7 +283,7 @@ void test_array_nullable_data(CollectionValue* src_data, uint8_t* src_is_null, i
         child_meta->set_is_nullable(true);
 
         std::unique_ptr<ColumnWriter> writer;
-        ColumnWriter::create(writer_opts, &list_column, wblock.get(), &writer);
+        ColumnWriter::create(writer_opts, &list_column, file_writer.get(), &writer);
         st = writer->init();
         EXPECT_TRUE(st.ok()) << st.to_string();
 
@@ -311,31 +301,26 @@ void test_array_nullable_data(CollectionValue* src_data, uint8_t* src_is_null, i
         EXPECT_TRUE(st.ok());
 
         // close the file
-        EXPECT_TRUE(wblock->close().ok());
+        EXPECT_TRUE(file_writer->close().ok());
     }
     auto type_info = get_type_info(&meta);
-
+    io::FileReaderSPtr file_reader;
+    ASSERT_EQ(fs->open_file(fname, &file_reader), Status::OK());
     // read and check
     {
         ColumnReaderOptions reader_opts;
-        FilePathDesc path_desc;
-        path_desc.filepath = fname;
         std::unique_ptr<ColumnReader> reader;
-        auto st = ColumnReader::create(reader_opts, meta, num_rows, path_desc, &reader);
+        auto st = ColumnReader::create(reader_opts, meta, num_rows, file_reader, &reader);
         EXPECT_TRUE(st.ok());
 
         ColumnIterator* iter = nullptr;
         st = reader->new_iterator(&iter);
         EXPECT_TRUE(st.ok());
-        std::unique_ptr<fs::ReadableBlock> rblock;
-        fs::BlockManager* block_manager = fs::fs_util::block_manager(TStorageMedium::HDD);
-        st = block_manager->open_block(path_desc, &rblock);
-        EXPECT_TRUE(st.ok());
+
         ColumnIteratorOptions iter_opts;
         OlapReaderStatistics stats;
         iter_opts.stats = &stats;
-        iter_opts.rblock = rblock.get();
-        iter_opts.mem_tracker = std::make_shared<MemTracker>();
+        iter_opts.file_reader = file_reader.get();
         st = iter->init(iter_opts);
         EXPECT_TRUE(st.ok());
         // sequence read
@@ -343,10 +328,9 @@ void test_array_nullable_data(CollectionValue* src_data, uint8_t* src_is_null, i
             st = iter->seek_to_first();
             EXPECT_TRUE(st.ok()) << st.to_string();
 
-            MemTracker tracker;
-            MemPool pool(&tracker);
+            MemPool pool;
             std::unique_ptr<ColumnVectorBatch> cvb;
-            ColumnVectorBatch::create(0, true, type_info, field, &cvb);
+            ColumnVectorBatch::create(0, true, type_info.get(), field, &cvb);
             cvb->resize(1024);
             ColumnBlock col(cvb.get(), &pool);
 
@@ -370,10 +354,9 @@ void test_array_nullable_data(CollectionValue* src_data, uint8_t* src_is_null, i
         }
         // seek read
         {
-            MemTracker tracker;
-            MemPool pool(&tracker);
+            MemPool pool;
             std::unique_ptr<ColumnVectorBatch> cvb;
-            ColumnVectorBatch::create(0, true, type_info, field, &cvb);
+            ColumnVectorBatch::create(0, true, type_info.get(), field, &cvb);
             cvb->resize(1024);
             ColumnBlock col(cvb.get(), &pool);
 
@@ -463,15 +446,15 @@ TEST_F(ColumnReaderWriterTest, test_array_type) {
 template <FieldType type>
 void test_read_default_value(string value, void* result) {
     using Type = typename TypeTraits<type>::CppType;
-    const auto* type_info = get_scalar_type_info(type);
+    const auto* scalar_type_info = get_scalar_type_info<type>();
     // read and check
     {
         TabletColumn tablet_column = create_with_default_value<type>(value);
-        DefaultValueColumnIterator iter(tablet_column.has_default_value(),
-                                        tablet_column.default_value(), tablet_column.is_nullable(),
-                                        type_info, tablet_column.length());
+        DefaultValueColumnIterator iter(
+                tablet_column.has_default_value(), tablet_column.default_value(),
+                tablet_column.is_nullable(), create_static_type_info_ptr(scalar_type_info),
+                tablet_column.length(), tablet_column.precision(), tablet_column.frac());
         ColumnIteratorOptions iter_opts;
-        iter_opts.mem_tracker = std::make_shared<MemTracker>();
         auto st = iter.init(iter_opts);
         EXPECT_TRUE(st.ok());
         // sequence read
@@ -479,14 +462,12 @@ void test_read_default_value(string value, void* result) {
             st = iter.seek_to_first();
             EXPECT_TRUE(st.ok()) << st.to_string();
 
-            auto tracker = std::make_shared<MemTracker>();
-            MemPool pool(tracker.get());
+            MemPool pool;
             std::unique_ptr<ColumnVectorBatch> cvb;
-            ColumnVectorBatch::create(0, true, type_info, nullptr, &cvb);
+            ColumnVectorBatch::create(0, true, scalar_type_info, nullptr, &cvb);
             cvb->resize(1024);
             ColumnBlock col(cvb.get(), &pool);
 
-            int idx = 0;
             size_t rows_read = 1024;
             ColumnBlockView dst(&col);
             bool has_null;
@@ -505,15 +486,13 @@ void test_read_default_value(string value, void* result) {
                     ;
                     EXPECT_EQ(*(Type*)result, *(reinterpret_cast<const Type*>(col.cell_ptr(j))));
                 }
-                idx++;
             }
         }
 
         {
-            auto tracker = std::make_shared<MemTracker>();
-            MemPool pool(tracker.get());
+            MemPool pool;
             std::unique_ptr<ColumnVectorBatch> cvb;
-            ColumnVectorBatch::create(0, true, type_info, nullptr, &cvb);
+            ColumnVectorBatch::create(0, true, scalar_type_info, nullptr, &cvb);
             cvb->resize(1024);
             ColumnBlock col(cvb.get(), &pool);
 
@@ -521,7 +500,6 @@ void test_read_default_value(string value, void* result) {
                 st = iter.seek_to_ordinal(rowid);
                 EXPECT_TRUE(st.ok());
 
-                int idx = rowid;
                 size_t rows_read = 1024;
                 ColumnBlockView dst(&col);
                 bool has_null;
@@ -540,7 +518,6 @@ void test_read_default_value(string value, void* result) {
                         EXPECT_EQ(*(Type*)result,
                                   *(reinterpret_cast<const Type*>(col.cell_ptr(j))));
                     }
-                    idx++;
                 }
             }
         }
@@ -575,15 +552,15 @@ static vectorized::MutableColumnPtr create_vectorized_column_ptr(FieldType type)
 template <FieldType type>
 void test_v_read_default_value(string value, void* result) {
     using Type = typename TypeTraits<type>::CppType;
-    const auto* type_info = get_scalar_type_info(type);
+    const auto* scalar_type_info = get_scalar_type_info<type>();
     // read and check
     {
         TabletColumn tablet_column = create_with_default_value<type>(value);
-        DefaultValueColumnIterator iter(tablet_column.has_default_value(),
-                                        tablet_column.default_value(), tablet_column.is_nullable(),
-                                        type_info, tablet_column.length());
+        DefaultValueColumnIterator iter(
+                tablet_column.has_default_value(), tablet_column.default_value(),
+                tablet_column.is_nullable(), create_static_type_info_ptr(scalar_type_info),
+                tablet_column.length(), tablet_column.precision(), tablet_column.frac());
         ColumnIteratorOptions iter_opts;
-        iter_opts.mem_tracker = std::make_shared<MemTracker>();
         auto st = iter.init(iter_opts);
         EXPECT_TRUE(st.ok());
 
@@ -822,5 +799,29 @@ TEST_F(ColumnReaderWriterTest, test_v_default_value) {
     test_v_read_default_value<OLAP_FIELD_TYPE_DECIMAL>(v_decimal, &decimal);
 }
 
+TEST_F(ColumnReaderWriterTest, test_single_empty_array) {
+    size_t num_array = 1;
+    std::unique_ptr<uint8_t[]> array_is_null(new uint8_t[BitmapSize(num_array)]());
+    CollectionValue array(0);
+    test_array_nullable_data<OLAP_FIELD_TYPE_TINYINT, BIT_SHUFFLE, BIT_SHUFFLE>(
+            &array, array_is_null.get(), num_array, "test_single_empty_array");
+}
+
+TEST_F(ColumnReaderWriterTest, test_mixed_empty_arrays) {
+    size_t num_array = 3;
+    std::unique_ptr<uint8_t[]> array_is_null(new uint8_t[BitmapSize(num_array)]());
+    std::unique_ptr<CollectionValue[]> collection_values(new CollectionValue[num_array]);
+    int data[] = {1, 2, 3};
+    for (int i = 0; i < num_array; ++i) {
+        if (i % 2 == 1) {
+            new (&collection_values[i]) CollectionValue(0);
+        } else {
+            new (&collection_values[i]) CollectionValue(&data, 3, false, nullptr);
+        }
+    }
+    test_array_nullable_data<OLAP_FIELD_TYPE_INT, BIT_SHUFFLE, BIT_SHUFFLE>(
+            collection_values.get(), array_is_null.get(), num_array, "test_mixed_empty_arrays");
+}
+
 } // namespace segment_v2
 } // namespace doris
