@@ -30,7 +30,6 @@ import org.apache.doris.thrift.TTableType;
 
 import com.google.common.base.Strings;
 import com.google.common.collect.Maps;
-
 import org.apache.commons.codec.digest.DigestUtils;
 import org.apache.logging.log4j.LogManager;
 import org.apache.logging.log4j.Logger;
@@ -45,8 +44,6 @@ import java.util.List;
 import java.util.Map;
 import java.util.stream.Collectors;
 
-import static java.util.stream.Collectors.joining;
-
 public class OdbcTable extends Table {
     private static final Logger LOG = LogManager.getLogger(OlapTable.class);
 
@@ -77,11 +74,17 @@ public class OdbcTable extends Table {
 
     // For different databases, special characters need to be escaped
     private static String mysqlProperName(String name) {
-        return "`" + name + "`";
+        // In JdbcExternalTable, the name contains databaseName, like: db.table
+        // So, we should split db and table, then switch to `db`.`table`.
+        List<String> list = Arrays.asList(name.split("\\."));
+        return list.stream().map(s -> "`" + s + "`").collect(Collectors.joining("."));
     }
 
     private static String mssqlProperName(String name) {
-        return "[" + name + "]";
+        // In JdbcExternalTable, the name contains databaseName, like: db.table
+        // So, we should split db and table, then switch to [db].[table].
+        List<String> list = Arrays.asList(name.split("\\."));
+        return list.stream().map(s -> "[" + s + "]").collect(Collectors.joining("."));
     }
 
     private static String psqlProperName(String name) {
@@ -89,6 +92,26 @@ public class OdbcTable extends Table {
         return list.stream().map(s -> "\"" + s + "\"").collect(Collectors.joining("."));
     }
 
+    private static String oracleProperName(String name) {
+        List<String> list = Arrays.asList(name.split("\\."));
+        return list.stream().map(s -> "\"" + s.toUpperCase() + "\"").collect(Collectors.joining("."));
+    }
+
+    private static String clickhouseProperName(String name) {
+        List<String> list = Arrays.asList(name.split("\\."));
+        return list.stream().map(s -> "\"" + s + "\"").collect(Collectors.joining("."));
+    }
+
+    private static String saphanaProperName(String name) {
+        List<String> list = Arrays.asList(name.split("\\."));
+        return list.stream().map(s -> "\"" + s + "\"").collect(Collectors.joining("."));
+    }
+
+    private static String trinoProperName(String name) {
+        List<String> list = Arrays.asList(name.split("\\."));
+        return list.stream().map(s -> "\"" + s + "\"").collect(Collectors.joining("."));
+    }
+
     public static String databaseProperName(TOdbcTableType tableType, String name) {
         switch (tableType) {
             case MYSQL:
@@ -97,6 +120,14 @@ public class OdbcTable extends Table {
                 return mssqlProperName(name);
             case POSTGRESQL:
                 return psqlProperName(name);
+            case ORACLE:
+                return oracleProperName(name);
+            case CLICKHOUSE:
+                return clickhouseProperName(name);
+            case SAP_HANA:
+                return saphanaProperName(name);
+            case TRINO:
+                return trinoProperName(name);
             default:
                 return name;
         }
@@ -128,20 +159,20 @@ public class OdbcTable extends Table {
     private void validate(Map<String, String> properties) throws DdlException {
         if (properties == null) {
             throw new DdlException("Please set properties of odbc table, "
-                    + "they are: odbc_catalog_resource or [host, port, user, password, driver, odbc_type]" +
-                    " and database and table");
+                    + "they are: odbc_catalog_resource or [host, port, user, password, driver, odbc_type]"
+                    + " and database and table");
         }
         if (properties.containsKey(ODBC_CATALOG_RESOURCE)) {
             odbcCatalogResourceName = properties.get(ODBC_CATALOG_RESOURCE);
 
             // 1. check whether resource exist
-            Resource oriResource = Catalog.getCurrentCatalog().getResourceMgr().getResource(odbcCatalogResourceName);
+            Resource oriResource = Env.getCurrentEnv().getResourceMgr().getResource(odbcCatalogResourceName);
             if (oriResource == null) {
                 throw new DdlException("Resource does not exist. name: " + odbcCatalogResourceName);
             }
 
             // 2. check resource usage privilege
-            if (!Catalog.getCurrentCatalog().getAuth().checkResourcePriv(ConnectContext.get(),
+            if (!Env.getCurrentEnv().getAuth().checkResourcePriv(ConnectContext.get(),
                     odbcCatalogResourceName,
                     PrivPredicate.USAGE)) {
                 throw new DdlException("USAGE denied to user '" + ConnectContext.get().getQualifiedUser()
@@ -209,7 +240,7 @@ public class OdbcTable extends Table {
 
             charset = properties.get(ODBC_CHARSET);
             copiedProperties.remove(ODBC_CHARSET);
-            
+
             String tableType = properties.get(ODBC_TYPE);
             if (Strings.isNullOrEmpty(tableType)) {
                 throw new DdlException("Type of Odbc table is null. "
@@ -242,17 +273,19 @@ public class OdbcTable extends Table {
 
     private String getPropertyFromResource(String propertyName) {
         OdbcCatalogResource odbcCatalogResource = (OdbcCatalogResource)
-                (Catalog.getCurrentCatalog().getResourceMgr().getResource(odbcCatalogResourceName));
+                (Env.getCurrentEnv().getResourceMgr().getResource(odbcCatalogResourceName));
         if (odbcCatalogResource == null) {
             throw new RuntimeException("Resource does not exist. name: " + odbcCatalogResourceName);
         }
 
         String property = odbcCatalogResource.getProperty(propertyName);
         if (property == null) {
-            throw new RuntimeException("The property:" + propertyName + " do not set in resource " + odbcCatalogResourceName);
+            throw new RuntimeException("The property:" + propertyName
+                    + " do not set in resource " + odbcCatalogResourceName);
         }
         return property;
     }
+
     public String getExtraParameter(Map<String, String> extraMap) {
         if (extraMap == null || extraMap.isEmpty()) {
             return "";
@@ -260,7 +293,7 @@ public class OdbcTable extends Table {
         return ";" + extraMap.entrySet()
                 .stream()
                 .map(e -> e.getKey() + "=" + e.getValue())
-                .collect(joining(";"));
+                .collect(Collectors.joining(";"));
     }
 
     public String getExtraParam() {
@@ -269,6 +302,7 @@ public class OdbcTable extends Table {
         }
         return getExtraParameter(resourceProperties);
     }
+
     public String getOdbcCatalogResourceName() {
         return odbcCatalogResourceName;
     }
@@ -353,7 +387,8 @@ public class OdbcTable extends Table {
                         getCharset());
                 break;
             case POSTGRESQL:
-                connectString = String.format("Driver=%s;Server=%s;Port=%s;DataBase=%s;Uid=%s;Pwd=%s;charset=%s;UseDeclareFetch=1;Fetch=4096",
+                connectString = String.format("Driver=%s;Server=%s;Port=%s;DataBase=%s;"
+                                + "Uid=%s;Pwd=%s;charset=%s;UseDeclareFetch=1;Fetch=4096",
                         getOdbcDriver(),
                         getHost(),
                         getPort(),
@@ -363,7 +398,8 @@ public class OdbcTable extends Table {
                         getCharset());
                 break;
             case MYSQL:
-                connectString = String.format("Driver=%s;Server=%s;Port=%s;DataBase=%s;Uid=%s;Pwd=%s;charset=%s;forward_cursor=1;no_cache=1",
+                connectString = String.format("Driver=%s;Server=%s;Port=%s;DataBase=%s;"
+                                + "Uid=%s;Pwd=%s;charset=%s;forward_cursor=1;no_cache=1",
                         getOdbcDriver(),
                         getHost(),
                         getPort(),
@@ -400,8 +436,8 @@ public class OdbcTable extends Table {
         return copied;
     }
 
-    public void resetIdsForRestore(Catalog catalog){
-        id = catalog.getNextId();
+    public void resetIdsForRestore(Env env) {
+        id = env.getNextId();
     }
 
     public TTableDescriptor toThrift() {
