@@ -18,18 +18,18 @@
 package org.apache.doris.common.util;
 
 import org.apache.doris.analysis.BrokerDesc;
-import org.apache.doris.analysis.StorageBackend;
+import org.apache.doris.backup.BlobStorage;
 import org.apache.doris.backup.RemoteFile;
-import org.apache.doris.backup.S3Storage;
 import org.apache.doris.backup.Status;
-import org.apache.doris.catalog.AuthType;
-import org.apache.doris.catalog.Catalog;
+import org.apache.doris.catalog.Env;
 import org.apache.doris.catalog.FsBroker;
 import org.apache.doris.common.AnalysisException;
 import org.apache.doris.common.ClientPool;
 import org.apache.doris.common.Config;
+import org.apache.doris.common.FeConstants;
 import org.apache.doris.common.Pair;
 import org.apache.doris.common.UserException;
+import org.apache.doris.datasource.hive.HiveMetaStoreCache;
 import org.apache.doris.service.FrontendOptions;
 import org.apache.doris.thrift.TBrokerCheckPathExistRequest;
 import org.apache.doris.thrift.TBrokerCheckPathExistResponse;
@@ -49,69 +49,30 @@ import org.apache.doris.thrift.TBrokerOperationStatus;
 import org.apache.doris.thrift.TBrokerOperationStatusCode;
 import org.apache.doris.thrift.TBrokerPReadRequest;
 import org.apache.doris.thrift.TBrokerPWriteRequest;
-import org.apache.doris.thrift.TBrokerRangeDesc;
 import org.apache.doris.thrift.TBrokerReadResponse;
 import org.apache.doris.thrift.TBrokerRenamePathRequest;
 import org.apache.doris.thrift.TBrokerVersion;
-import org.apache.doris.thrift.THdfsConf;
-import org.apache.doris.thrift.THdfsParams;
 import org.apache.doris.thrift.TNetworkAddress;
 import org.apache.doris.thrift.TPaloBrokerService;
 
 import com.google.common.base.Preconditions;
 import com.google.common.collect.Lists;
-
-import org.apache.hadoop.conf.Configuration;
-import org.apache.hadoop.fs.FileStatus;
-import org.apache.hadoop.fs.FileSystem;
-import org.apache.hadoop.fs.Path;
-import org.apache.hadoop.security.UserGroupInformation;
 import org.apache.logging.log4j.LogManager;
 import org.apache.logging.log4j.Logger;
 import org.apache.thrift.TException;
 
 import java.io.FileInputStream;
 import java.io.IOException;
-import java.net.URI;
-import java.net.URISyntaxException;
 import java.nio.ByteBuffer;
 import java.nio.channels.FileChannel;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.List;
-import java.util.Map;
 
 public class BrokerUtil {
     private static final Logger LOG = LogManager.getLogger(BrokerUtil.class);
 
     private static final int READ_BUFFER_SIZE_B = 1024 * 1024;
-    public static String HADOOP_FS_NAME = "fs.defaultFS";
-    // simple or kerberos
-    public static String HADOOP_SECURITY_AUTHENTICATION = "hadoop.security.authentication";
-    public static String HADOOP_USER_NAME = "hadoop.username";
-    public static String HADOOP_KERBEROS_PRINCIPAL = "hadoop.kerberos.principal";
-    public static String HADOOP_KERBEROS_KEYTAB = "hadoop.kerberos.keytab";
-
-    public static void generateHdfsParam(Map<String, String> properties, TBrokerRangeDesc rangeDesc) {
-        rangeDesc.setHdfsParams(new THdfsParams());
-        rangeDesc.hdfs_params.setHdfsConf(new ArrayList<>());
-        for (Map.Entry<String, String> property : properties.entrySet()) {
-            if (property.getKey().equalsIgnoreCase(HADOOP_FS_NAME)) {
-                rangeDesc.hdfs_params.setFsName(property.getValue());
-            } else if (property.getKey().equalsIgnoreCase(HADOOP_USER_NAME)) {
-                rangeDesc.hdfs_params.setUser(property.getValue());
-            } else if (property.getKey().equalsIgnoreCase(HADOOP_KERBEROS_PRINCIPAL)) {
-                rangeDesc.hdfs_params.setHdfsKerberosPrincipal(property.getValue());
-            } else if (property.getKey().equalsIgnoreCase(HADOOP_KERBEROS_KEYTAB)) {
-                rangeDesc.hdfs_params.setHdfsKerberosKeytab(property.getValue());
-            } else {
-                THdfsConf hdfsConf = new THdfsConf();
-                hdfsConf.setKey(property.getKey());
-                hdfsConf.setValue(property.getValue());
-                rangeDesc.hdfs_params.hdfs_conf.add(hdfsConf);
-            }
-        }
-    }
 
     /**
      * Parse file status in path with broker, except directory
@@ -122,90 +83,23 @@ public class BrokerUtil {
      */
     public static void parseFile(String path, BrokerDesc brokerDesc, List<TBrokerFileStatus> fileStatuses)
             throws UserException {
-        if (brokerDesc.getStorageType() == StorageBackend.StorageType.BROKER) {
-            TNetworkAddress address = getAddress(brokerDesc);
-            TPaloBrokerService.Client client = borrowClient(address);
-            boolean failed = true;
-            try {
-                TBrokerListPathRequest request = new TBrokerListPathRequest(
-                    TBrokerVersion.VERSION_ONE, path, false, brokerDesc.getProperties());
-                TBrokerListResponse tBrokerListResponse = null;
-                try {
-                    tBrokerListResponse = client.listPath(request);
-                } catch (TException e) {
-                    reopenClient(client);
-                    tBrokerListResponse = client.listPath(request);
-                }
-                if (tBrokerListResponse.getOpStatus().getStatusCode() != TBrokerOperationStatusCode.OK) {
-                    throw new UserException("Broker list path failed. path=" + path
-                        + ",broker=" + address + ",msg=" + tBrokerListResponse.getOpStatus().getMessage());
-                }
-                failed = false;
-                for (TBrokerFileStatus tBrokerFileStatus : tBrokerListResponse.getFiles()) {
-                    if (tBrokerFileStatus.isDir) {
-                        continue;
-                    }
-                    fileStatuses.add(tBrokerFileStatus);
-                }
-            } catch (TException e) {
-                LOG.warn("Broker list path exception, path={}, address={}", path, address, e);
-                throw new UserException("Broker list path exception. path=" + path + ", broker=" + address);
-            } finally {
-                returnClient(client, address, failed);
+        List<RemoteFile> rfiles = new ArrayList<>();
+        try {
+            BlobStorage storage = BlobStorage.create(
+                    brokerDesc.getName(), brokerDesc.getStorageType(), brokerDesc.getProperties());
+            Status st = storage.list(path, rfiles, false);
+            if (!st.ok()) {
+                throw new UserException(brokerDesc.getName() + " list path failed. path=" + path
+                        + ",msg=" + st.getErrMsg());
             }
-        } else if (brokerDesc.getStorageType() == StorageBackend.StorageType.S3) {
-            S3Storage s3 = new S3Storage(brokerDesc.getProperties());
-            List<RemoteFile> rfiles = new ArrayList<>();
-            try {
-                Status st = s3.list(path, rfiles, false);
-                if (!st.ok()) {
-                    throw new UserException("S3 list path failed. path=" + path
-                            + ",msg=" + st.getErrMsg());
-                }
-            } catch (Exception e) {
-                LOG.warn("s3 list path exception, path={}", path, e);
-                throw new UserException("s3 list path exception. path=" + path + ", err: " + e.getMessage());
-            }
-            for (RemoteFile r : rfiles) {
-                if (r.isFile()) {
-                    fileStatuses.add(new TBrokerFileStatus(r.getName(), !r.isFile(), r.getSize(), r.isFile()));
-                }
-            }
-        } else if (brokerDesc.getStorageType() == StorageBackend.StorageType.HDFS) {
-            if (!brokerDesc.getProperties().containsKey(HADOOP_FS_NAME)
-                || !brokerDesc.getProperties().containsKey(HADOOP_USER_NAME)) {
-                throw new UserException(String.format(
-                    "The properties of hdfs is invalid. %s and %s are needed", HADOOP_FS_NAME, HADOOP_USER_NAME));
-            }
-            String fsName = brokerDesc.getProperties().get(HADOOP_FS_NAME);
-            String userName = brokerDesc.getProperties().get(HADOOP_USER_NAME);
-            Configuration conf = new Configuration();
-            boolean isSecurityEnabled = false;
-            for (Map.Entry<String, String> propEntry : brokerDesc.getProperties().entrySet()) {
-                conf.set(propEntry.getKey(), propEntry.getValue());
-                if (propEntry.getKey().equals(BrokerUtil.HADOOP_SECURITY_AUTHENTICATION)
-                    && propEntry.getValue().equals(AuthType.KERBEROS.getDesc())) {
-                    isSecurityEnabled = true;
-                }
-            }
-            try {
-                if (isSecurityEnabled) {
-                    UserGroupInformation.setConfiguration(conf);
-                    UserGroupInformation.loginUserFromKeytab(
-                        brokerDesc.getProperties().get(BrokerUtil.HADOOP_KERBEROS_PRINCIPAL),
-                        brokerDesc.getProperties().get(BrokerUtil.HADOOP_KERBEROS_KEYTAB));
-                }
-                FileSystem fs = FileSystem.get(new URI(fsName), conf, userName);
-                FileStatus[] statusList = fs.globStatus(new Path(path));
-                for (FileStatus status : statusList) {
-                    if (status.isFile()) {
-                        fileStatuses.add(new TBrokerFileStatus(status.getPath().toUri().getPath(),
-                            status.isDirectory(), status.getLen(), status.isFile()));
-                    }
-                }
-            } catch (IOException | InterruptedException | URISyntaxException e) {
-                LOG.warn("hdfs check error: ", e);
-                throw new UserException(e.getMessage());
+        } catch (Exception e) {
+            LOG.warn("{} list path exception, path={}", brokerDesc.getName(), path, e);
+            throw new UserException(brokerDesc.getName() +  " list path exception. path="
+                    + path + ", err: " + e.getMessage());
+        }
+        for (RemoteFile r : rfiles) {
+            if (r.isFile()) {
+                fileStatuses.add(new TBrokerFileStatus(r.getName(), !r.isFile(), r.getSize(), r.isFile()));
             }
         }
     }
@@ -214,13 +108,29 @@ public class BrokerUtil {
         return brokerName + "[" + address.toString() + "]";
     }
 
-    public static List<String> parseColumnsFromPath(String filePath, List<String> columnsFromPath) throws UserException {
+    public static List<String> parseColumnsFromPath(String filePath, List<String> columnsFromPath)
+            throws UserException {
+        return parseColumnsFromPath(filePath, columnsFromPath, true);
+    }
+
+    public static List<String> parseColumnsFromPath(
+            String filePath,
+            List<String> columnsFromPath,
+            boolean caseSensitive)
+            throws UserException {
         if (columnsFromPath == null || columnsFromPath.isEmpty()) {
             return Collections.emptyList();
         }
+        if (!caseSensitive) {
+            for (int i = 0; i < columnsFromPath.size(); i++) {
+                String path = columnsFromPath.remove(i);
+                columnsFromPath.add(i, path.toLowerCase());
+            }
+        }
         String[] strings = filePath.split("/");
         if (strings.length < 2) {
-            throw new UserException("Fail to parse columnsFromPath, expected: " + columnsFromPath + ", filePath: " + filePath);
+            throw new UserException("Fail to parse columnsFromPath, expected: "
+                    + columnsFromPath + ", filePath: " + filePath);
         }
         String[] columns = new String[columnsFromPath.size()];
         int size = 0;
@@ -230,24 +140,29 @@ public class BrokerUtil {
                 continue;
             }
             if (str == null || !str.contains("=")) {
-                throw new UserException("Fail to parse columnsFromPath, expected: " + columnsFromPath + ", filePath: " + filePath);
+                throw new UserException("Fail to parse columnsFromPath, expected: "
+                        + columnsFromPath + ", filePath: " + filePath);
             }
             String[] pair = str.split("=", 2);
             if (pair.length != 2) {
-                throw new UserException("Fail to parse columnsFromPath, expected: " + columnsFromPath + ", filePath: " + filePath);
+                throw new UserException("Fail to parse columnsFromPath, expected: "
+                        + columnsFromPath + ", filePath: " + filePath);
             }
-            int index = columnsFromPath.indexOf(pair[0]);
+            String parsedColumnName = caseSensitive ? pair[0] : pair[0].toLowerCase();
+            int index = columnsFromPath.indexOf(parsedColumnName);
             if (index == -1) {
                 continue;
             }
-            columns[index] = pair[1];
+            columns[index] = HiveMetaStoreCache.HIVE_DEFAULT_PARTITION.equals(pair[1])
+                ? FeConstants.null_string : pair[1];
             size++;
             if (size >= columnsFromPath.size()) {
                 break;
             }
         }
         if (size != columnsFromPath.size()) {
-            throw new UserException("Fail to parse columnsFromPath, expected: " + columnsFromPath + ", filePath: " + filePath);
+            throw new UserException("Fail to parse columnsFromPath, expected: "
+                    + columnsFromPath + ", filePath: " + filePath);
         }
         return Lists.newArrayList(columns);
     }
@@ -467,8 +382,8 @@ public class BrokerUtil {
                     remotePath, brokerDesc.getProperties());
             TBrokerCheckPathExistResponse rep = client.checkPathExist(req);
             if (rep.getOpStatus().getStatusCode() != TBrokerOperationStatusCode.OK) {
-                throw new UserException("Broker check path exist failed. path=" + remotePath + ", broker=" + address +
-                        ", msg=" + rep.getOpStatus().getMessage());
+                throw new UserException("Broker check path exist failed. path=" + remotePath + ", broker=" + address
+                        + ", msg=" + rep.getOpStatus().getMessage());
             }
             failed = false;
             return rep.isPathExist;
@@ -490,22 +405,23 @@ public class BrokerUtil {
                     destFilePath, brokerDesc.getProperties());
             TBrokerOperationStatus rep = client.renamePath(req);
             if (rep.getStatusCode() != TBrokerOperationStatusCode.OK) {
-                throw new UserException("failed to rename " + origFilePath + " to " + destFilePath +
-                        ", msg: " + rep.getMessage() + ", broker: " + address);
+                throw new UserException("failed to rename " + origFilePath + " to " + destFilePath
+                        + ", msg: " + rep.getMessage() + ", broker: " + address);
             }
             failed = false;
         } catch (TException e) {
             LOG.warn("Broker rename file failed, origin path={}, dest path={}, address={}, exception={}",
                     origFilePath, destFilePath, address, e);
-            throw new UserException("Broker rename file exception. origin path=" + origFilePath +
-                    ", dest path=" + destFilePath + ", broker=" + address);
+            throw new UserException("Broker rename file exception. origin path=" + origFilePath
+                    + ", dest path=" + destFilePath + ", broker=" + address);
         } finally {
             returnClient(client, address, failed);
         }
     }
 
-    public static Pair<TPaloBrokerService.Client, TNetworkAddress> getBrokerAddressAndClient(BrokerDesc brokerDesc) throws UserException {
-        Pair<TPaloBrokerService.Client, TNetworkAddress> pair = new Pair<TPaloBrokerService.Client, TNetworkAddress>(null, null);
+    public static Pair<TPaloBrokerService.Client, TNetworkAddress> getBrokerAddressAndClient(BrokerDesc brokerDesc)
+            throws UserException {
+        Pair<TPaloBrokerService.Client, TNetworkAddress> pair = Pair.of(null, null);
         TNetworkAddress address = getAddress(brokerDesc);
         TPaloBrokerService.Client client = borrowClient(address);
         pair.first = client;
@@ -517,7 +433,7 @@ public class BrokerUtil {
         FsBroker broker = null;
         try {
             String localIP = FrontendOptions.getLocalHostAddress();
-            broker = Catalog.getCurrentCatalog().getBrokerMgr().getBroker(brokerDesc.getName(), localIP);
+            broker = Env.getCurrentEnv().getBrokerMgr().getBroker(brokerDesc.getName(), localIP);
         } catch (AnalysisException e) {
             throw new UserException(e.getMessage());
         }
@@ -601,7 +517,8 @@ public class BrokerUtil {
 
         public void write(ByteBuffer byteBuffer, long bufferSize) throws UserException {
             if (!isReady) {
-                throw new UserException("Broker writer is not ready. filePath=" + brokerFilePath + ", broker=" + address);
+                throw new UserException("Broker writer is not ready. filePath="
+                        + brokerFilePath + ", broker=" + address);
             }
 
             failed = true;
@@ -660,3 +577,4 @@ public class BrokerUtil {
 
     }
 }
+
