@@ -17,7 +17,7 @@
 
 package org.apache.doris.journal.bdbje;
 
-import org.apache.doris.catalog.Catalog;
+import org.apache.doris.catalog.Env;
 import org.apache.doris.common.Pair;
 import org.apache.doris.common.io.DataOutputBuffer;
 import org.apache.doris.common.io.Writable;
@@ -38,7 +38,6 @@ import com.sleepycat.je.rep.InsufficientLogException;
 import com.sleepycat.je.rep.NetworkRestore;
 import com.sleepycat.je.rep.NetworkRestoreConfig;
 import com.sleepycat.je.rep.RollbackException;
-
 import org.apache.logging.log4j.LogManager;
 import org.apache.logging.log4j.Logger;
 
@@ -49,38 +48,38 @@ import java.io.IOException;
 import java.util.List;
 import java.util.concurrent.atomic.AtomicLong;
 
-/* 
+/*
  * This is the bdb implementation of Journal interface.
  * First, we open() this journal, then read from or write to the bdb environment
  * We can also get journal id information by calling get***Id functions.
  * Finally, close this journal.
  * This class encapsulates the read, write APIs of bdbje
  */
-public class BDBJEJournal implements Journal {
+public class BDBJEJournal implements Journal { // CHECKSTYLE IGNORE THIS LINE: BDBJE should use uppercase
     public static final Logger LOG = LogManager.getLogger(BDBJEJournal.class);
     private static final int OUTPUT_BUFFER_INIT_SIZE = 128;
     private static final int RETRY_TIME = 3;
-    
+
     private String environmentPath = null;
     private String selfNodeName;
     private String selfNodeHostPort;
-    
+
     private BDBEnvironment bdbEnvironment = null;
     private Database currentJournalDB;
     // the next journal's id. start from 1.
     private AtomicLong nextJournalId = new AtomicLong(1);
-    
+
     public BDBJEJournal(String nodeName) {
         initBDBEnv(nodeName);
     }
-    
-    /* 
+
+    /*
      * Initialize bdb environment.
      * node name is ip_port (the port is edit_log_port)
      */
     private void initBDBEnv(String nodeName) {
-        environmentPath = Catalog.getServingCatalog().getBdbDir();
-        Pair<String, Integer> selfNode = Catalog.getServingCatalog().getSelfNode();
+        environmentPath = Env.getServingEnv().getBdbDir();
+        Pair<String, Integer> selfNode = Env.getServingEnv().getSelfNode();
         selfNodeName = nodeName;
         selfNodeHostPort = selfNode.first + ":" + selfNode.second;
     }
@@ -97,7 +96,7 @@ public class BDBJEJournal implements Journal {
         if (currentJournalDB.count() == 0) {
             return;
         }
-        
+
         long newName = nextJournalId.get();
         String currentDbName = currentJournalDB.getDatabaseName();
         long currentName = Long.parseLong(currentDbName);
@@ -164,15 +163,16 @@ public class BDBJEJournal implements Journal {
             if (op == OperationType.OP_TIMESTAMP) {
                 /*
                  * Do not exit if the write operation is OP_TIMESTAMP.
-                 * If all the followers exit except master, master should continue provide query service.
+                 * If all the followers exit except master, master should continue provide query
+                 * service.
                  * To prevent master exit, we should exempt OP_TIMESTAMP write
                  */
                 nextJournalId.set(id);
                 LOG.warn("master can not achieve quorum. write timestamp fail. but will not exit.");
                 return;
             }
-            String msg = "write bdb failed. will exit. journalId: " + id + ", bdb database Name: " +
-                    currentJournalDB.getDatabaseName();
+            String msg = "write bdb failed. will exit. journalId: " + id + ", bdb database Name: "
+                    + currentJournalDB.getDatabaseName();
             LOG.error(msg);
             Util.stdoutWithTime(msg);
             System.exit(-1);
@@ -194,19 +194,19 @@ public class BDBJEJournal implements Journal {
                 break;
             }
         }
-        
+
         if (dbName == null) {
             return null;
         }
-        
+
         JournalEntity ret = null;
         Long key = new Long(journalId);
         DatabaseEntry theKey = new DatabaseEntry();
         TupleBinding<Long> myBinding = TupleBinding.getPrimitiveBinding(Long.class);
         myBinding.objectToEntry(key, theKey);
-        
+
         DatabaseEntry theData = new DatabaseEntry();
-        
+
         Database database = bdbEnvironment.openDatabase(dbName);
         try {
             // null means perform the operation without transaction protection.
@@ -235,7 +235,7 @@ public class BDBJEJournal implements Journal {
     public JournalCursor read(long fromKey, long toKey) {
         return BDBJournalCursor.getJournalCursor(bdbEnvironment, fromKey, toKey);
     }
-    
+
     @Override
     public long getMaxJournalId() {
         long ret = -1;
@@ -249,13 +249,13 @@ public class BDBJEJournal implements Journal {
         if (dbNames.size() == 0) {
             return ret;
         }
-        
+
         int index = dbNames.size() - 1;
         String dbName = dbNames.get(index).toString();
         long dbNumberName = dbNames.get(index);
         Database database = bdbEnvironment.openDatabase(dbName);
         ret = dbNumberName + database.count() - 1;
-        
+
         return ret;
     }
 
@@ -272,14 +272,14 @@ public class BDBJEJournal implements Journal {
         if (dbNames.size() == 0) {
             return ret;
         }
-        
+
         String dbName = dbNames.get(0).toString();
         Database database = bdbEnvironment.openDatabase(dbName);
         // The database is empty
         if (database.count() == 0) {
             return ret;
         }
-        
+
         return dbNames.get(0);
     }
 
@@ -297,35 +297,37 @@ public class BDBJEJournal implements Journal {
         if (bdbEnvironment == null) {
             File dbEnv = new File(environmentPath);
             bdbEnvironment = new BDBEnvironment();
-            Pair<String, Integer> helperNode = Catalog.getServingCatalog().getHelperNode();
+            Pair<String, Integer> helperNode = Env.getServingEnv().getHelperNode();
             String helperHostPort = helperNode.first + ":" + helperNode.second;
             try {
-                bdbEnvironment.setup(dbEnv, selfNodeName, selfNodeHostPort,
-                        helperHostPort, Catalog.getServingCatalog().isElectable());
+                bdbEnvironment.setup(dbEnv, selfNodeName, selfNodeHostPort, helperHostPort,
+                        Env.getServingEnv().isElectable());
             } catch (Exception e) {
                 LOG.error("catch an exception when setup bdb environment. will exit.", e);
                 System.exit(-1);
             }
         }
-        
-        // Open a new journal database or get last existing one as current journal database
+
+        // Open a new journal database or get last existing one as current journal
+        // database
         List<Long> dbNames = null;
         for (int i = 0; i < RETRY_TIME; i++) {
             try {
                 dbNames = getDatabaseNames();
-                
+
                 if (dbNames == null) {
                     LOG.error("fail to get dbNames while open bdbje journal. will exit");
                     System.exit(-1);
                 }
                 if (dbNames.size() == 0) {
                     /*
-                     *  This is the very first time to open. Usually, we will open a new database named "1".
-                     *  But when we start cluster with an image file copied from other cluster,
-                     *  here we should open database with name image max journal id + 1.
-                     *  (default Catalog.getServingCatalog().getReplayedJournalId() is 0)
+                     * This is the very first time to open. Usually, we will open a new database
+                     * named "1".
+                     * But when we start cluster with an image file copied from other cluster,
+                     * here we should open database with name image max journal id + 1.
+                     * (default Catalog.getServingEnv().getReplayedJournalId() is 0)
                      */
-                    String dbName = Long.toString(Catalog.getServingCatalog().getReplayedJournalId() + 1);
+                    String dbName = Long.toString(Env.getServingEnv().getReplayedJournalId() + 1);
                     LOG.info("the very first time to open bdb, dbname is {}", dbName);
                     currentJournalDB = bdbEnvironment.openDatabase(dbName);
                 } else {
@@ -339,23 +341,28 @@ public class BDBJEJournal implements Journal {
                 break;
             } catch (InsufficientLogException insufficientLogEx) {
                 reSetupBdbEnvironment(insufficientLogEx);
+            } catch (RollbackException rollbackEx) {
+                LOG.warn("catch rollback log exception. will reopen the ReplicatedEnvironment.", rollbackEx);
+                bdbEnvironment.closeReplicatedEnvironment();
+                bdbEnvironment.openReplicatedEnvironment(new File(environmentPath));
             }
         }
     }
 
     private void reSetupBdbEnvironment(InsufficientLogException insufficientLogEx) {
         LOG.warn("catch insufficient log exception. will recover and try again.", insufficientLogEx);
-        // Copy the missing log files from a member of the replication group who owns the files
-        // ATTN: here we use `getServingCatalog()`, because only serving catalog has helper nodes.
-        Pair<String, Integer> helperNode = Catalog.getServingCatalog().getHelperNode();
+        // Copy the missing log files from a member of the replication group who owns
+        // the files
+        // ATTN: here we use `getServingEnv()`, because only serving catalog has
+        // helper nodes.
+        Pair<String, Integer> helperNode = Env.getServingEnv().getHelperNode();
         NetworkRestore restore = new NetworkRestore();
         NetworkRestoreConfig config = new NetworkRestoreConfig();
         config.setRetainLogFiles(false);
         restore.execute(insufficientLogEx, config);
         bdbEnvironment.close();
         bdbEnvironment.setup(new File(environmentPath), selfNodeName, selfNodeHostPort,
-                helperNode.first + ":" + helperNode.second,
-                Catalog.getServingCatalog().isElectable());
+                helperNode.first + ":" + helperNode.second, Env.getServingEnv().isElectable());
     }
 
     @Override
@@ -372,7 +379,7 @@ public class BDBJEJournal implements Journal {
         }
         msg += ", deleteToJournalId is " + deleteToJournalId;
         LOG.info(msg);
-        
+
         for (int i = 1; i < dbNames.size(); i++) {
             if (deleteToJournalId >= dbNames.get(i)) {
                 long name = dbNames.get(i - 1);
@@ -386,7 +393,7 @@ public class BDBJEJournal implements Journal {
             }
         }
     }
-    
+
     @Override
     public long getFinalizedJournalId() {
         List<Long> dbNames = getDatabaseNames();
@@ -394,48 +401,52 @@ public class BDBJEJournal implements Journal {
             LOG.error("database name is null.");
             return 0;
         }
-        
+
         String msg = "database names: ";
         for (long name : dbNames) {
             msg += name + " ";
         }
         LOG.info(msg);
-        
+
         if (dbNames.size() < 2) {
             return 0;
         }
-        
+
         return dbNames.get(dbNames.size() - 1) - 1;
     }
-    
+
     @Override
     public List<Long> getDatabaseNames() {
         if (bdbEnvironment == null) {
             return null;
         }
 
-        // Open a new journal database or get last existing one as current journal database
-        List<Long>  dbNames = null;
+        // Open a new journal database or get last existing one as current journal
+        // database
+        List<Long> dbNames = null;
         for (int i = 0; i < RETRY_TIME; i++) {
             try {
                 dbNames = bdbEnvironment.getDatabaseNames();
                 break;
             } catch (InsufficientLogException insufficientLogEx) {
                 /*
-                 * If this is not a checkpoint thread, which means this maybe the FE startup thread,
-                 * or a replay thread. We will reopen bdbEnvironment for these 2 cases to get valid log
+                 * If this is not a checkpoint thread, which means this maybe the FE startup
+                 * thread,
+                 * or a replay thread. We will reopen bdbEnvironment for these 2 cases to get
+                 * valid log
                  * from helper nodes.
                  *
-                 * The checkpoint thread will only run on Master FE. And Master FE should not encounter
+                 * The checkpoint thread will only run on Master FE. And Master FE should not
+                 * encounter
                  * these exception. So if it happens, throw exception out.
                  */
-                if (!Catalog.isCheckpointThread()) {
+                if (!Env.isCheckpointThread()) {
                     reSetupBdbEnvironment(insufficientLogEx);
                 } else {
                     throw insufficientLogEx;
                 }
             } catch (RollbackException rollbackEx) {
-                if (!Catalog.isCheckpointThread()) {
+                if (!Env.isCheckpointThread()) {
                     LOG.warn("catch rollback log exception. will reopen the ReplicatedEnvironment.", rollbackEx);
                     bdbEnvironment.closeReplicatedEnvironment();
                     bdbEnvironment.openReplicatedEnvironment(new File(environmentPath));
@@ -447,4 +458,8 @@ public class BDBJEJournal implements Journal {
 
         return dbNames;
     }
+
+    public BDBEnvironment getBDBEnvironment() {
+        return this.bdbEnvironment;
+    }
 }
