@@ -14,6 +14,9 @@
 // KIND, either express or implied.  See the License for the
 // specific language governing permissions and limitations
 // under the License.
+// This file is copied from
+// https://github.com/apache/impala/blob/branch-2.9.0/fe/src/main/java/org/apache/impala/FoldConstantsRule.java
+// and modified by Doris
 
 package org.apache.doris.rewrite;
 
@@ -27,7 +30,7 @@ import org.apache.doris.analysis.InformationFunction;
 import org.apache.doris.analysis.LiteralExpr;
 import org.apache.doris.analysis.NullLiteral;
 import org.apache.doris.analysis.SysVariableDesc;
-import org.apache.doris.catalog.Catalog;
+import org.apache.doris.catalog.Env;
 import org.apache.doris.catalog.PrimitiveType;
 import org.apache.doris.catalog.Type;
 import org.apache.doris.common.AnalysisException;
@@ -47,12 +50,12 @@ import org.apache.doris.thrift.TQueryGlobals;
 
 import com.google.common.base.Preconditions;
 import com.google.common.base.Predicates;
-
 import org.apache.logging.log4j.LogManager;
 import org.apache.logging.log4j.Logger;
 
 import java.text.DateFormat;
 import java.text.SimpleDateFormat;
+import java.time.LocalDateTime;
 import java.util.Collections;
 import java.util.Date;
 import java.util.HashMap;
@@ -107,7 +110,7 @@ public class FoldConstantsRule implements ExprRewriteRule {
         if (expr instanceof CastExpr) {
             CastExpr castExpr = (CastExpr) expr;
             if (castExpr.getChild(0) instanceof NullLiteral) {
-                return expr;
+                return castExpr.getChild(0);
             }
         }
 
@@ -119,7 +122,7 @@ public class FoldConstantsRule implements ExprRewriteRule {
                 return expr;
             }
         }
-        return expr.getResultValue();
+        return expr.getResultValue(false);
     }
 
     /**
@@ -141,7 +144,7 @@ public class FoldConstantsRule implements ExprRewriteRule {
         Map<String, Map<String, Expr>> sysVarsMap = new HashMap<>();
         // map to collect InformationFunction
         Map<String, Map<String, Expr>> infoFnsMap = new HashMap<>();
-        for (Map.Entry<String, Expr> entry : exprMap.entrySet()){
+        for (Map.Entry<String, Expr> entry : exprMap.entrySet()) {
             Map<String, TExpr> constMap = new HashMap<>();
             Map<String, Expr> oriConstMap = new HashMap<>();
             Map<String, Expr> sysVarMap = new HashMap<>();
@@ -191,7 +194,7 @@ public class FoldConstantsRule implements ExprRewriteRule {
      * @throws AnalysisException
      */
     // public only for unit test
-    public void getConstExpr(Expr expr, Map<String,TExpr> constExprMap, Map<String, Expr> oriConstMap,
+    public void getConstExpr(Expr expr, Map<String, TExpr> constExprMap, Map<String, Expr> oriConstMap,
                               Analyzer analyzer, Map<String, Expr> sysVarMap, Map<String, Expr> infoFnMap)
             throws AnalysisException {
         if (expr.isConstant()) {
@@ -229,9 +232,9 @@ public class FoldConstantsRule implements ExprRewriteRule {
         }
     }
 
-    private void recursiveGetChildrenConstExpr(Expr expr, Map<String,TExpr> constExprMap, Map<String, Expr> oriConstMap,
-                                               Analyzer analyzer, Map<String, Expr> sysVarMap,
-                                               Map<String, Expr> infoFnMap)throws AnalysisException {
+    private void recursiveGetChildrenConstExpr(Expr expr, Map<String, TExpr> constExprMap,
+            Map<String, Expr> oriConstMap, Analyzer analyzer, Map<String, Expr> sysVarMap, Map<String, Expr> infoFnMap)
+            throws AnalysisException {
         for (int i = 0; i < expr.getChildren().size(); i++) {
             final Expr child = expr.getChildren().get(i);
             getConstExpr(child, constExprMap, oriConstMap, analyzer, sysVarMap, infoFnMap);
@@ -340,17 +343,18 @@ public class FoldConstantsRule implements ExprRewriteRule {
         TNetworkAddress brpcAddress = null;
         Map<String, Map<String, Expr>> resultMap = new HashMap<>();
         try {
-            List<Long> backendIds = Catalog.getCurrentSystemInfo().getBackendIds(true);
+            List<Long> backendIds = Env.getCurrentSystemInfo().getBackendIds(true);
             if (backendIds.isEmpty()) {
                 throw new LoadException("Failed to get all partitions. No alive backends");
             }
             Collections.shuffle(backendIds);
-            Backend be = Catalog.getCurrentSystemInfo().getBackend(backendIds.get(0));
+            Backend be = Env.getCurrentSystemInfo().getBackend(backendIds.get(0));
             brpcAddress = new TNetworkAddress(be.getHost(), be.getBrpcPort());
 
             TQueryGlobals queryGlobals = new TQueryGlobals();
             queryGlobals.setNowString(DATE_FORMAT.format(new Date()));
             queryGlobals.setTimestampMs(System.currentTimeMillis());
+            queryGlobals.setNanoSeconds(LocalDateTime.now().getNano());
             queryGlobals.setTimeZone(TimeUtils.DEFAULT_TIME_ZONE);
             if (context.getSessionVariable().getTimeZone().equals("CST")) {
                 queryGlobals.setTimeZone(TimeUtils.DEFAULT_TIME_ZONE);
@@ -360,14 +364,17 @@ public class FoldConstantsRule implements ExprRewriteRule {
 
             TFoldConstantParams tParams = new TFoldConstantParams(map, queryGlobals);
             tParams.setVecExec(VectorizedUtil.isVectorized());
-            
-            Future<InternalService.PConstantExprResult> future = BackendServiceProxy.getInstance().foldConstantExpr(brpcAddress, tParams);
+
+            Future<InternalService.PConstantExprResult> future
+                    = BackendServiceProxy.getInstance().foldConstantExpr(brpcAddress, tParams);
             InternalService.PConstantExprResult result = future.get(5, TimeUnit.SECONDS);
 
             if (result.getStatus().getStatusCode() == 0) {
-                for (Map.Entry<String, InternalService.PExprResultMap> entry : result.getExprResultMapMap().entrySet()) {
+                for (Map.Entry<String, InternalService.PExprResultMap> entry
+                        : result.getExprResultMapMap().entrySet()) {
                     Map<String, Expr> tmp = new HashMap<>();
-                    for (Map.Entry<String, InternalService.PExprResult> entry1 : entry.getValue().getMapMap().entrySet()) {
+                    for (Map.Entry<String, InternalService.PExprResult> entry1
+                            : entry.getValue().getMapMap().entrySet()) {
                         TPrimitiveType type = TPrimitiveType.findByValue(entry1.getValue().getType().getType());
                         Expr retExpr = null;
                         if (entry1.getValue().getSuccess()) {
@@ -392,4 +399,3 @@ public class FoldConstantsRule implements ExprRewriteRule {
         return resultMap;
     }
 }
-
