@@ -17,11 +17,10 @@
 
 package org.apache.doris.analysis;
 
-import mockit.Expectations;
 import org.apache.doris.catalog.BrokerMgr;
-import org.apache.doris.catalog.Catalog;
 import org.apache.doris.catalog.Column;
 import org.apache.doris.catalog.Database;
+import org.apache.doris.catalog.Env;
 import org.apache.doris.catalog.FakeEditLog;
 import org.apache.doris.catalog.KeysType;
 import org.apache.doris.catalog.MaterializedIndex;
@@ -34,6 +33,8 @@ import org.apache.doris.catalog.SinglePartitionInfo;
 import org.apache.doris.common.AnalysisException;
 import org.apache.doris.common.DdlException;
 import org.apache.doris.common.jmockit.Deencapsulation;
+import org.apache.doris.datasource.CatalogMgr;
+import org.apache.doris.datasource.InternalCatalog;
 import org.apache.doris.load.Load;
 import org.apache.doris.mysql.privilege.PaloAuth;
 import org.apache.doris.mysql.privilege.PrivPredicate;
@@ -44,10 +45,11 @@ import org.apache.doris.thrift.TStorageType;
 
 import com.google.common.collect.Lists;
 import com.google.common.collect.Sets;
-
+import mockit.Expectations;
 
 import java.util.LinkedList;
 import java.util.List;
+import java.util.function.Function;
 
 public class AccessTestUtil {
     private static FakeEditLog fakeEditLog;
@@ -56,7 +58,7 @@ public class AccessTestUtil {
         SystemInfoService clusterInfo = new SystemInfoService();
         return clusterInfo;
     }
-    
+
     public static PaloAuth fetchAdminAccess() {
         PaloAuth auth = new PaloAuth();
         try {
@@ -74,6 +76,10 @@ public class AccessTestUtil {
                     minTimes = 0;
                     result = true;
 
+                    auth.checkTblPriv((ConnectContext) any, anyString, anyString, anyString, (PrivPredicate) any);
+                    minTimes = 0;
+                    result = true;
+
                     auth.setPassword((SetPassVar) any);
                     minTimes = 0;
                 }
@@ -84,15 +90,15 @@ public class AccessTestUtil {
         return auth;
     }
 
-    public static Catalog fetchAdminCatalog() {
+    public static Env fetchAdminCatalog() {
         try {
-            Catalog catalog = Deencapsulation.newInstance(Catalog.class);
+            Env env = Deencapsulation.newInstance(Env.class);
 
-            PaloAuth paloAuth = fetchAdminAccess();
+            PaloAuth auth = fetchAdminAccess();
 
             fakeEditLog = new FakeEditLog();
             EditLog editLog = new EditLog("name");
-            catalog.setEditLog(editLog);
+            env.setEditLog(editLog);
 
             Database db = new Database(50000L, "testCluster:testDb");
             MaterializedIndex baseIndex = new MaterializedIndex(30001, IndexState.NORMAL);
@@ -101,20 +107,17 @@ public class AccessTestUtil {
             List<Column> baseSchema = new LinkedList<Column>();
             Column column = new Column();
             baseSchema.add(column);
-            OlapTable table = new OlapTable(30000, "testTbl", baseSchema,
-                    KeysType.AGG_KEYS, new SinglePartitionInfo(), distributionInfo);
-            table.setIndexMeta(baseIndex.getId(), "testTbl", baseSchema, 0, 1, (short) 1,
-                    TStorageType.COLUMN, KeysType.AGG_KEYS);
+            OlapTable table = new OlapTable(30000, "testTbl", baseSchema, KeysType.AGG_KEYS, new SinglePartitionInfo(),
+                    distributionInfo);
+            table.setIndexMeta(baseIndex.getId(), "testTbl", baseSchema, 0, 1, (short) 1, TStorageType.COLUMN,
+                    KeysType.AGG_KEYS);
             table.addPartition(partition);
             table.setBaseIndexId(baseIndex.getId());
             db.createTable(table);
 
+            InternalCatalog catalog = Deencapsulation.newInstance(InternalCatalog.class);
             new Expectations(catalog) {
                 {
-                    catalog.getAuth();
-                    minTimes = 0;
-                    result = paloAuth;
-
                     catalog.getDbNullable(50000L);
                     minTimes = 0;
                     result = db;
@@ -135,11 +138,48 @@ public class AccessTestUtil {
                     minTimes = 0;
                     result = Lists.newArrayList("testCluster:testDb");
 
-                    catalog.getEditLog();
+                    catalog.getClusterDbNames("testCluster");
+                    minTimes = 0;
+                    result = Lists.newArrayList("testCluster:testDb");
+                }
+            };
+
+            CatalogMgr dsMgr = new CatalogMgr();
+            new Expectations(dsMgr) {
+                {
+                    dsMgr.getCatalog((String) any);
+                    minTimes = 0;
+                    result = catalog;
+
+                    dsMgr.getCatalogOrException((String) any, (Function) any);
+                    minTimes = 0;
+                    result = catalog;
+
+                    dsMgr.getCatalogOrAnalysisException((String) any);
+                    minTimes = 0;
+                    result = catalog;
+                }
+            };
+
+            new Expectations(env, catalog) {
+                {
+                    env.getAuth();
+                    minTimes = 0;
+                    result = auth;
+
+                    env.getCurrentCatalog();
+                    minTimes = 0;
+                    result = catalog;
+
+                    env.getInternalCatalog();
+                    minTimes = 0;
+                    result = catalog;
+
+                    env.getEditLog();
                     minTimes = 0;
                     result = editLog;
 
-                    catalog.getLoadInstance();
+                    env.getLoadInstance();
                     minTimes = 0;
                     result = new Load();
 
@@ -147,19 +187,23 @@ public class AccessTestUtil {
                     minTimes = 0;
                     result = Lists.newArrayList("testCluster:testDb");
 
-                    catalog.changeDb((ConnectContext) any, "blockDb");
+                    env.changeDb((ConnectContext) any, "blockDb");
                     minTimes = 0;
                     result = new DdlException("failed");
 
-                    catalog.changeDb((ConnectContext) any, anyString);
+                    env.changeDb((ConnectContext) any, anyString);
                     minTimes = 0;
 
-                    catalog.getBrokerMgr();
+                    env.getBrokerMgr();
                     minTimes = 0;
                     result = new BrokerMgr();
+
+                    env.getCatalogMgr();
+                    minTimes = 0;
+                    result = dsMgr;
                 }
             };
-            return catalog;
+            return env;
         } catch (DdlException e) {
             return null;
         } catch (AnalysisException e) {
@@ -190,6 +234,10 @@ public class AccessTestUtil {
     public static OlapTable mockTable(String name) {
         Column column1 = new Column("col1", PrimitiveType.BIGINT);
         Column column2 = new Column("col2", PrimitiveType.DOUBLE);
+        Column column3 = new Column("k1", PrimitiveType.VARCHAR);
+        Column column4 = new Column("k2", PrimitiveType.VARCHAR);
+        Column column5 = new Column("k3", PrimitiveType.VARCHAR);
+        Column column6 = new Column("k4", PrimitiveType.BIGINT);
 
         MaterializedIndex index = new MaterializedIndex();
         new Expectations(index) {
@@ -223,6 +271,22 @@ public class AccessTestUtil {
                 table.getPartition(40000L);
                 minTimes = 0;
                 result = partition;
+
+                table.getColumn("k1");
+                minTimes = 0;
+                result = column3;
+
+                table.getColumn("k2");
+                minTimes = 0;
+                result = column4;
+
+                table.getColumn("k3");
+                minTimes = 0;
+                result = column5;
+
+                table.getColumn("k4");
+                minTimes = 0;
+                result = column6;
             }
         };
         return table;
@@ -238,6 +302,10 @@ public class AccessTestUtil {
                 minTimes = 0;
                 result = olapTable;
 
+                db.getTableNullable("t");
+                minTimes = 0;
+                result = olapTable;
+
                 db.getTableNullable("emptyTable");
                 minTimes = 0;
                 result = null;
@@ -264,27 +332,28 @@ public class AccessTestUtil {
         return db;
     }
 
-    public static Catalog fetchBlockCatalog() {
+    public static Env fetchBlockCatalog() {
         try {
-            Catalog catalog = Deencapsulation.newInstance(Catalog.class);
+            Env env = Deencapsulation.newInstance(Env.class);
 
-            PaloAuth paloAuth = fetchBlockAccess();
+            PaloAuth auth = fetchBlockAccess();
             Database db = mockDb("testCluster:testDb");
 
+            InternalCatalog catalog = Deencapsulation.newInstance(InternalCatalog.class);
             new Expectations(catalog) {
                 {
-                    catalog.getAuth();
-                    minTimes = 0;
-                    result = paloAuth;
-
-                    catalog.changeDb((ConnectContext) any, anyString);
-                    minTimes = 0;
-                    result = new DdlException("failed");
-
                     catalog.getDbNullable("testCluster:testDb");
                     minTimes = 0;
                     result = db;
 
+                    catalog.getDbNullable("testCluster:testdb");
+                    minTimes = 0;
+                    result = db;
+
+                    catalog.getDbOrAnalysisException("testdb");
+                    minTimes = 0;
+                    result = db;
+
                     catalog.getDbNullable("testCluster:emptyDb");
                     minTimes = 0;
                     result = null;
@@ -306,7 +375,48 @@ public class AccessTestUtil {
                     result = null;
                 }
             };
-            return catalog;
+
+            CatalogMgr dsMgr = new CatalogMgr();
+            new Expectations(dsMgr) {
+                {
+                    dsMgr.getCatalog((String) any);
+                    minTimes = 0;
+                    result = catalog;
+
+                    dsMgr.getCatalogOrException((String) any, (Function) any);
+                    minTimes = 0;
+                    result = catalog;
+
+                    dsMgr.getCatalogOrAnalysisException((String) any);
+                    minTimes = 0;
+                    result = catalog;
+                }
+            };
+
+            new Expectations(env) {
+                {
+                    env.getAuth();
+                    minTimes = 0;
+                    result = auth;
+
+                    env.changeDb((ConnectContext) any, anyString);
+                    minTimes = 0;
+                    result = new DdlException("failed");
+
+                    env.getInternalCatalog();
+                    minTimes = 0;
+                    result = catalog;
+
+                    env.getCurrentCatalog();
+                    minTimes = 0;
+                    result = catalog;
+
+                    env.getCatalogMgr();
+                    minTimes = 0;
+                    result = dsMgr;
+                }
+            };
+            return env;
         } catch (DdlException e) {
             return null;
         } catch (AnalysisException e) {
@@ -317,16 +427,20 @@ public class AccessTestUtil {
     public static Analyzer fetchAdminAnalyzer(boolean withCluster) {
         final String prefix = "testCluster:";
 
-        Analyzer analyzer = new Analyzer(fetchAdminCatalog(), new ConnectContext(null));
+        Analyzer analyzer = new Analyzer(fetchAdminCatalog(), new ConnectContext());
         new Expectations(analyzer) {
             {
+                analyzer.getDefaultCatalog();
+                minTimes = 0;
+                result = InternalCatalog.INTERNAL_CATALOG_NAME;
+
                 analyzer.getDefaultDb();
                 minTimes = 0;
-                result = withCluster? prefix + "testDb" : "testDb";
+                result = withCluster ? prefix + "testDb" : "testDb";
 
                 analyzer.getQualifiedUser();
-                minTimes = 0 ;
-                result = withCluster? prefix + "testUser" : "testUser";
+                minTimes = 0;
+                result = withCluster ? prefix + "testUser" : "testUser";
 
                 analyzer.getClusterName();
                 minTimes = 0;
@@ -349,9 +463,13 @@ public class AccessTestUtil {
     }
 
     public static Analyzer fetchBlockAnalyzer() throws AnalysisException {
-        Analyzer analyzer = new Analyzer(fetchBlockCatalog(), new ConnectContext(null));
+        Analyzer analyzer = new Analyzer(fetchBlockCatalog(), new ConnectContext());
         new Expectations(analyzer) {
             {
+                analyzer.getDefaultCatalog();
+                minTimes = 0;
+                result = InternalCatalog.INTERNAL_CATALOG_NAME;
+
                 analyzer.getDefaultDb();
                 minTimes = 0;
                 result = "testCluster:testDb";
@@ -369,9 +487,13 @@ public class AccessTestUtil {
     }
 
     public static Analyzer fetchEmptyDbAnalyzer() {
-        Analyzer analyzer = new Analyzer(fetchBlockCatalog(), new ConnectContext(null));
+        Analyzer analyzer = new Analyzer(fetchBlockCatalog(), new ConnectContext());
         new Expectations(analyzer) {
             {
+                analyzer.getDefaultCatalog();
+                minTimes = 0;
+                result = InternalCatalog.INTERNAL_CATALOG_NAME;
+
                 analyzer.getDefaultDb();
                 minTimes = 0;
                 result = "";
@@ -476,25 +598,26 @@ public class AccessTestUtil {
                 result = "testDb";
             }
         };
-        Catalog catalog = fetchBlockCatalog();
-        Analyzer analyzer = new Analyzer(catalog, new ConnectContext(null));
+
+        Env env = fetchBlockCatalog();
+        Analyzer analyzer = new Analyzer(env, new ConnectContext());
         new Expectations(analyzer) {
             {
+                analyzer.getDefaultCatalog();
+                minTimes = 0;
+                result = InternalCatalog.INTERNAL_CATALOG_NAME;
+
                 analyzer.getDefaultDb();
                 minTimes = 0;
                 result = "testDb";
 
-                analyzer.getTableOrAnalysisException((TableName) any);
-                minTimes = 0;
-                result = table;
-
                 analyzer.getQualifiedUser();
                 minTimes = 0;
                 result = "testUser";
 
-                analyzer.getCatalog();
+                analyzer.getEnv();
                 minTimes = 0;
-                result = catalog;
+                result = env;
 
                 analyzer.getClusterName();
                 minTimes = 0;
@@ -514,11 +637,10 @@ public class AccessTestUtil {
 
                 analyzer.getContext();
                 minTimes = 0;
-                result = new ConnectContext(null);
+                result = new ConnectContext();
 
             }
         };
         return analyzer;
     }
 }
-
