@@ -25,7 +25,15 @@
 # contains the build version based on the git hash or svn revision.
 ##############################################################
 
-build_version="1.1.5-rc02"
+set -eo pipefail
+
+build_version_prefix="doris"
+build_version_major=1
+build_version_minor=2
+build_version_patch=4
+build_version_rc_version="rc01"
+
+build_version="${build_version_prefix}-${build_version_major}.${build_version_minor}.${build_version_patch}-${build_version_rc_version}"
 
 unset LANG
 unset LC_CTYPE
@@ -33,21 +41,17 @@ unset LC_CTYPE
 date="$(date +"%a, %d %b %Y %H:%M:%S %Z")"
 hostname="$(hostname)"
 
-cwd=$(pwd)
+cwd="$(pwd)"
 
-if [ -z ${DORIS_HOME+x} ]; then
-    ROOT=$(dirname "$0")
-    ROOT=$(
-        cd "$ROOT"
-        pwd
-    )
-    DORIS_HOME=${ROOT}/../..
+if [[ -z "${DORIS_HOME}" ]]; then
+    ROOT="$(cd "$(dirname "${BASH_SOURCE[0]}")" &>/dev/null && pwd)"
+    DORIS_HOME="${ROOT}/../.."
     echo "DORIS_HOME: ${DORIS_HOME}"
 fi
 
-if [[ -z ${DORIS_TEST_BINARY_DIR} ]]; then
-    if [ -e ${DORIS_HOME}/fe/fe-core/target/generated-sources/build/org/apache/doris/common/Version.java \
-        -a -e ${DORIS_HOME}/gensrc/build/gen_cpp/version.h ]; then
+if [[ -z "${DORIS_TEST_BINARY_DIR}" ]]; then
+    if [[ -e "${DORIS_HOME}/fe/fe-core/target/generated-sources/build/org/apache/doris/common/Version.java" &&
+        -e "${DORIS_HOME}/gensrc/build/gen_cpp/version.h" ]]; then
         exit
     fi
 fi
@@ -64,34 +68,33 @@ else
     url="file://${hostname}"
 fi
 
-cd ${cwd}
+cd "${cwd}"
 
 build_hash="${url}@${revision}"
 build_short_hash="${short_revision}"
 build_time="${date}"
 build_info="${hostname}"
 
-if [ -z "$JAVA_HOME" ]; then
-    java_cmd=$(which java)
+if [[ -z "${JAVA_HOME}" ]]; then
+    java_cmd="$(which java)"
 else
-    java_cmd="$JAVA_HOME/bin/java"
+    java_cmd="${JAVA_HOME}/bin/java"
 fi
 
-if [ ! -x "$java_cmd" ]; then
+if [[ ! -x "${java_cmd}" ]]; then
     echo "The JAVA_HOME environment variable is not defined correctly"
     echo "This environment variable is needed to run this program"
     echo "NB: JAVA_HOME should point to a JDK not a JRE"
     exit 1
 fi
-java_version_str=$($java_cmd -fullversion 2>&1)
-java_version_str=$(echo $java_version_str | sed -e 's/"/\\"/g')
+java_version_str="$("${java_cmd}" -fullversion 2>&1 | sed -e 's/"/\\"/g')"
 
-echo "get java cmd: $java_cmd"
-echo "get java version: $java_version_str"
+echo "get java cmd: ${java_cmd}"
+echo "get java version: ${java_version_str}"
 
 VERSION_PACKAGE="${DORIS_HOME}/fe/fe-core/target/generated-sources/build/org/apache/doris/common"
-mkdir -p ${VERSION_PACKAGE}
-cat > "${VERSION_PACKAGE}/Version.java" << EOF
+mkdir -p "${VERSION_PACKAGE}"
+cat >"${VERSION_PACKAGE}/Version.java" <<EOF
 // Licensed to the Apache Software Foundation (ASF) under one
 // or more contributor license agreements.  See the NOTICE file
 // distributed with this work for additional information
@@ -117,6 +120,12 @@ package org.apache.doris.common;
 
 public class Version {
 
+  public static final String DORIS_BUILD_VERSION_PREFIX = "${build_version_prefix}";
+  public static final int DORIS_BUILD_VERSION_MAJOR = ${build_version_major};
+  public static final int DORIS_BUILD_VERSION_MINOR = ${build_version_minor};
+  public static final int DORIS_BUILD_VERSION_PATCH = ${build_version_patch};
+  public static final String DORIS_BUILD_VERSION_RC_VERSION = "${build_version_rc_version}";
+
   public static final String DORIS_BUILD_VERSION = "${build_version}";
   public static final String DORIS_BUILD_HASH = "${build_hash}";
   public static final String DORIS_BUILD_SHORT_HASH = "${build_short_hash}";
@@ -125,6 +134,12 @@ public class Version {
   public static final String DORIS_JAVA_COMPILE_VERSION = "${java_version_str}";
 
   public static void main(String[] args) {
+    System.out.println("doris_build_version_prefix: " + DORIS_BUILD_VERSION_PREFIX);
+    System.out.println("doris_build_version_major: " + DORIS_BUILD_VERSION_MAJOR);
+    System.out.println("doris_build_version_minor: " + DORIS_BUILD_VERSION_MINOR);
+    System.out.println("doris_build_version_patch: " + DORIS_BUILD_VERSION_PATCH);
+    System.out.println("doris_build_version_rc_version: " + DORIS_BUILD_VERSION_RC_VERSION);
+
     System.out.println("doris_build_version: " + DORIS_BUILD_VERSION);
     System.out.println("doris_build_hash: " + DORIS_BUILD_HASH);
     System.out.println("doris_build_short_hash: " + DORIS_BUILD_SHORT_HASH);
@@ -136,9 +151,9 @@ public class Version {
 }
 EOF
 
-GEN_CPP_DIR=${DORIS_HOME}/gensrc/build/gen_cpp/
-mkdir -p ${GEN_CPP_DIR}
-cat > "${GEN_CPP_DIR}/version.h" << EOF
+GEN_CPP_DIR="${DORIS_HOME}/gensrc/build/gen_cpp"
+mkdir -p "${GEN_CPP_DIR}"
+cat >"${GEN_CPP_DIR}/version.h" <<EOF
 // Licensed to the Apache Software Foundation (ASF) under one
 // or more contributor license agreements.  See the NOTICE file
 // distributed with this work for additional information
@@ -165,11 +180,17 @@ cat > "${GEN_CPP_DIR}/version.h" << EOF
 
 namespace doris {
 
-#define DORIS_BUILD_VERSION    "${build_version}"
-#define DORIS_BUILD_HASH       "${build_hash}"
-#define DORIS_BUILD_SHORT_HASH "${build_short_hash}"
-#define DORIS_BUILD_TIME       "${build_time}"
-#define DORIS_BUILD_INFO       "${build_info}"
+#define DORIS_BUILD_VERSION_PREFIX      "${build_version_prefix}";
+#define DORIS_BUILD_VERSION_MAJOR       ${build_version_major};
+#define DORIS_BUILD_VERSION_MINOR       ${build_version_minor};
+#define DORIS_BUILD_VERSION_PATCH       ${build_version_patch};
+#define DORIS_BUILD_VERSION_RC_VERSION  "${build_version_rc_version}";
+
+#define DORIS_BUILD_VERSION             "${build_version}"
+#define DORIS_BUILD_HASH                "${build_hash}"
+#define DORIS_BUILD_SHORT_HASH          "${build_short_hash}"
+#define DORIS_BUILD_TIME                "${build_time}"
+#define DORIS_BUILD_INFO                "${build_info}"
 
 } // namespace doris
 
