@@ -36,12 +36,20 @@ curdir="$(cd "$(dirname "${BASH_SOURCE[0]}")" &>/dev/null && pwd)"
 export DORIS_HOME="${curdir}/.."
 export TP_DIR="${curdir}"
 
+# include custom environment variables
+if [[ -f "${DORIS_HOME}/env.sh" ]]; then
+    export BUILD_THIRDPARTY_WIP=1
+    . "${DORIS_HOME}/env.sh"
+    export BUILD_THIRDPARTY_WIP=
+fi
+
 # Check args
 usage() {
     echo "
 Usage: $0 <options>
   Optional options:
      -j                 build thirdparty parallel
+     --clean            clean the extracted data
   "
     exit 1
 }
@@ -51,6 +59,7 @@ if ! OPTS="$(getopt \
     -o '' \
     -o 'h' \
     -l 'help' \
+    -l 'clean' \
     -o 'j:' \
     -- "$@")"; then
     usage
@@ -81,6 +90,10 @@ if [[ "$#" -ne 1 ]]; then
             HELP=1
             shift
             ;;
+        --clean)
+            CLEAN=1
+            shift
+            ;;
         --)
             shift
             break
@@ -95,20 +108,13 @@ fi
 
 if [[ "${HELP}" -eq 1 ]]; then
     usage
-    exit
 fi
 
 echo "Get params:
     PARALLEL            -- ${PARALLEL}
+    CLEAN               -- ${CLEAN}
 "
 
-# include custom environment variables
-if [[ -f "${DORIS_HOME}/env.sh" ]]; then
-    export BUILD_THIRDPARTY_WIP=1
-    . "${DORIS_HOME}/env.sh"
-    export BUILD_THIRDPARTY_WIP=
-fi
-
 if [[ ! -f "${TP_DIR}/download-thirdparty.sh" ]]; then
     echo "Download thirdparty script is missing".
     exit 1
@@ -123,6 +129,12 @@ fi
 
 cd "${TP_DIR}"
 
+if [[ "${CLEAN}" -eq 1 ]] && [[ -d "${TP_SOURCE_DIR}" ]]; then
+    echo 'Clean the extracted data ...'
+    find "${TP_SOURCE_DIR}" -mindepth 1 -maxdepth 1 -type d -exec rm -rf {} \;
+    echo 'Success!'
+fi
+
 # Download thirdparties.
 "${TP_DIR}/download-thirdparty.sh"
 
@@ -146,6 +158,11 @@ elif [[ "${CC}" == *clang ]]; then
     warning_option_ignored='-Wno-option-ignored'
     boost_toolset='clang'
     libhdfs_cxx17='-std=c++1z'
+
+    test_warning_result="$("${CC}" -xc++ "${warning_unused_but_set_variable}" /dev/null 2>&1 || true)"
+    if echo "${test_warning_result}" | grep 'unknown warning option' >/dev/null; then
+        warning_unused_but_set_variable=''
+    fi
 fi
 
 # prepare installed prefix
@@ -154,10 +171,17 @@ pushd "${TP_DIR}/installed"/
 ln -sf lib64 lib
 popd
 
+# Configure the search paths for pkg-config and cmake
+export PKG_CONFIG_PATH="${TP_DIR}/installed/lib64/pkgconfig"
+export CMAKE_PREFIX_PATH="${TP_DIR}/installed"
+
+echo "PKG_CONFIG_PATH: ${PKG_CONFIG_PATH}"
+echo "CMAKE_PREFIX_PATH: ${CMAKE_PREFIX_PATH}"
+
 check_prerequest() {
     local CMD="$1"
     local NAME="$2"
-    if ! ${CMD}; then
+    if ! eval "${CMD}"; then
         echo "${NAME} is missing"
         exit 1
     else
@@ -246,6 +270,34 @@ remove_all_dylib() {
     fi
 }
 
+if [[ -z "${STRIP_TP_LIB}" ]]; then
+    if [[ "${KERNEL}" != 'Darwin' ]]; then
+        STRIP_TP_LIB='ON'
+    else
+        STRIP_TP_LIB='OFF'
+    fi
+fi
+
+if [[ "${STRIP_TP_LIB}" = "ON" ]]; then
+    echo "Strip thirdparty libraries"
+else
+    echo "Do not strip thirdparty libraries"
+fi
+
+strip_lib() {
+    if [[ "${STRIP_TP_LIB}" = "ON" ]]; then
+        if [[ -z $1 ]]; then
+            echo "Must specify the library to be stripped."
+            exit 1
+        fi
+        if [[ ! -f "${TP_LIB_DIR}/$1" ]]; then
+            echo "Library to be stripped (${TP_LIB_DIR}/$1) does not exist."
+            exit 1
+        fi
+        strip --strip-debug --strip-unneeded "${TP_LIB_DIR}/$1"
+    fi
+}
+
 #libbacktrace
 build_libbacktrace() {
     check_if_source_exist "${LIBBACKTRACE_SOURCE}"
@@ -278,7 +330,7 @@ build_libevent() {
     "${BUILD_SYSTEM}" install
 
     remove_all_dylib
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libevent.a
+    strip_lib libevent.a
 }
 
 build_openssl() {
@@ -319,15 +371,17 @@ build_thrift() {
     cd "${TP_SOURCE_DIR}/${THRIFT_SOURCE}"
 
     if [[ "${KERNEL}" != 'Darwin' ]]; then
-        cppflags="-I${TP_INCLUDE_DIR}"
+        cflags="-I${TP_INCLUDE_DIR}"
+        cxxflags="-I${TP_INCLUDE_DIR} ${warning_unused_but_set_variable}"
         ldflags="-L${TP_LIB_DIR} --static"
     else
-        cppflags="-I${TP_INCLUDE_DIR} -Wno-implicit-function-declaration"
+        cflags="-I${TP_INCLUDE_DIR} -Wno-implicit-function-declaration"
+        cxxflags="-I${TP_INCLUDE_DIR} ${warning_unused_but_set_variable}"
         ldflags="-L${TP_LIB_DIR}"
     fi
 
     # NOTE(amos): libtool discard -static. --static works.
-    ./configure CPPFLAGS="${cppflags}" LDFLAGS="${ldflags}" LIBS="-lcrypto -ldl -lssl" \
+    ./configure CFLAGS="${cflags}" CXXFLAGS="${cxxflags}" LDFLAGS="${ldflags}" LIBS="-lcrypto -ldl -lssl" \
         --prefix="${TP_INSTALL_DIR}" --docdir="${TP_INSTALL_DIR}/doc" --enable-static --disable-shared --disable-tests \
         --disable-tutorial --without-qt4 --without-qt5 --without-csharp --without-erlang --without-nodejs --without-nodets --without-swift \
         --without-lua --without-perl --without-php --without-php_extension --without-dart --without-ruby --without-cl \
@@ -340,8 +394,8 @@ build_thrift() {
 
     make -j "${PARALLEL}"
     make install
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libthrift.a
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libthriftnb.a
+    strip_lib libthrift.a
+    strip_lib libthriftnb.a
 }
 
 # protobuf
@@ -386,8 +440,8 @@ build_protobuf() {
 
     make -j "${PARALLEL}"
     make install
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libprotobuf.a
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libprotoc.a
+    strip_lib libprotobuf.a
+    strip_lib libprotoc.a
 }
 
 # gflags
@@ -423,7 +477,7 @@ build_glog() {
 
     make -j "${PARALLEL}"
     make install
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libglog.a
+    strip_lib libglog.a
 }
 
 # gtest
@@ -441,7 +495,7 @@ build_gtest() {
 
     "${BUILD_SYSTEM}" -j "${PARALLEL}"
     "${BUILD_SYSTEM}" install
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libgtest.a
+    strip_lib libgtest.a
 }
 
 # rapidjson
@@ -510,7 +564,7 @@ build_zlib() {
     check_if_source_exist "${ZLIB_SOURCE}"
     cd "${TP_SOURCE_DIR}/${ZLIB_SOURCE}"
 
-    CFLAGS="-fPIC" \
+    CFLAGS="-O3 -fPIC" \
         CPPFLAGS="-I${TP_INCLUDE_DIR}" \
         LDFLAGS="-L${TP_LIB_DIR}" \
         ./configure --prefix="${TP_INSTALL_DIR}" --static
@@ -552,7 +606,7 @@ build_zstd() {
         -DZSTD_BUILD_PROGRAMS=OFF -DZSTD_BUILD_SHARED=OFF -DCMAKE_INSTALL_PREFIX="${TP_INSTALL_DIR}" ..
 
     "${BUILD_SYSTEM}" -j "${PARALLEL}" install
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libzstd.a
+    strip_lib libzstd.a
 }
 
 # bzip
@@ -574,7 +628,7 @@ build_lzo2() {
 
     make -j "${PARALLEL}"
     make install
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/liblzo2.a
+    strip_lib liblzo2.a
 }
 
 # curl
@@ -597,7 +651,7 @@ build_curl() {
 
     make curl_LDFLAGS=-all-static -j "${PARALLEL}"
     make curl_LDFLAGS=-all-static install
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libcurl.a
+    strip_lib libcurl.a
 }
 
 # re2
@@ -607,7 +661,7 @@ build_re2() {
 
     "${CMAKE_CMD}" -DCMAKE_BUILD_TYPE=Release -G "${GENERATOR}" -DBUILD_SHARED_LIBS=0 -DCMAKE_INSTALL_PREFIX="${TP_INSTALL_DIR}"
     "${BUILD_SYSTEM}" -j "${PARALLEL}" install
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libre2.a
+    strip_lib libre2.a
 }
 
 # hyperscan
@@ -628,13 +682,16 @@ build_hyperscan() {
     check_if_source_exist "${HYPERSCAN_SOURCE}"
     cd "${TP_SOURCE_DIR}/${HYPERSCAN_SOURCE}"
 
+    # We don't need to build tools/hsbench which depends on sqlite3 installed.
+    rm -rf "${TP_SOURCE_DIR}/${HYPERSCAN_SOURCE}/tools/hsbench"
+
     mkdir -p "${BUILD_DIR}"
     cd "${BUILD_DIR}"
 
     "${CMAKE_CMD}" -G "${GENERATOR}" -DBUILD_SHARED_LIBS=0 \
         -DBOOST_ROOT="${BOOST_SOURCE}" -DCMAKE_INSTALL_PREFIX="${TP_INSTALL_DIR}" -DBUILD_EXAMPLES=OFF ..
     "${BUILD_SYSTEM}" -j "${PARALLEL}" install
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libhs.a
+    strip_lib libhs.a
 }
 
 # boost
@@ -697,7 +754,7 @@ build_mysql() {
     # copy libmysqlclient.a
     cp libmysql/libmysqlclient.a ../../../installed/lib/
     echo "mysql client lib is installed."
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libmysqlclient.a
+    strip_lib libmysqlclient.a
 }
 
 #leveldb
@@ -713,7 +770,7 @@ build_leveldb() {
     CXXFLAGS="-fPIC" "${CMAKE_CMD}" -G "${GENERATOR}" -DCMAKE_INSTALL_PREFIX="${TP_INSTALL_DIR}" -DLEVELDB_BUILD_BENCHMARKS=OFF \
         -DLEVELDB_BUILD_TESTS=OFF ..
     "${BUILD_SYSTEM}" -j "${PARALLEL}" install
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libleveldb.a
+    strip_lib libleveldb.a
 }
 
 # brpc
@@ -738,13 +795,14 @@ build_brpc() {
     LDFLAGS="${ldflags}" \
         "${CMAKE_CMD}" -G "${GENERATOR}" -DBUILD_SHARED_LIBS=1 -DWITH_GLOG=ON -DCMAKE_INSTALL_PREFIX="${TP_INSTALL_DIR}" \
         -DCMAKE_LIBRARY_PATH="${TP_INSTALL_DIR}/lib64" -DCMAKE_INCLUDE_PATH="${TP_INSTALL_DIR}/include" \
+        -DBUILD_BRPC_TOOLS=OFF \
         -DPROTOBUF_PROTOC_EXECUTABLE="${TP_INSTALL_DIR}/bin/protoc" ..
 
     "${BUILD_SYSTEM}" -j "${PARALLEL}"
     "${BUILD_SYSTEM}" install
 
     remove_all_dylib
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libbrpc.a
+    strip_lib libbrpc.a
 }
 
 # rocksdb
@@ -756,7 +814,11 @@ build_rocksdb() {
     if [[ "${KERNEL}" != 'Darwin' ]]; then
         ldflags='-static-libstdc++ -static-libgcc'
     else
-        ldflags=''
+        if [[ "$(uname -m)" != 'x86_64' ]]; then
+            ldflags=''
+        else
+            ldflags="-L${TP_LIB_DIR} -ljemalloc_doris"
+        fi
     fi
 
     # -Wno-range-loop-construct gcc-11
@@ -767,7 +829,7 @@ build_rocksdb() {
         PORTABLE=1 make USE_RTTI=1 -j "${PARALLEL}" static_lib
     cp librocksdb.a ../../installed/lib/librocksdb.a
     cp -r include/rocksdb ../../installed/include/
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/librocksdb.a
+    strip_lib librocksdb.a
 }
 
 # cyrus_sasl
@@ -775,9 +837,11 @@ build_cyrus_sasl() {
     check_if_source_exist "${CYRUS_SASL_SOURCE}"
     cd "${TP_SOURCE_DIR}/${CYRUS_SASL_SOURCE}"
 
-    CPPFLAGS="-I${TP_INCLUDE_DIR}" \
+    CFLAGS="-fPIC -Wno-implicit-function-declaration" \
+        CPPFLAGS="-I${TP_INCLUDE_DIR}" \
         LDFLAGS="-L${TP_LIB_DIR}" \
-        ./configure --prefix="${TP_INSTALL_DIR}" --enable-static --enable-shared=no --with-openssl="${TP_INSTALL_DIR}" --with-pic
+        LIBS="-lcrypto" \
+        ./configure --prefix="${TP_INSTALL_DIR}" --enable-static --enable-shared=no --with-openssl="${TP_INSTALL_DIR}" --with-pic --enable-gssapi="${TP_INSTALL_DIR}" --with-gss_impl=mit --with-dblib=none
 
     if [[ "${KERNEL}" != 'Darwin' ]]; then
         make -j "${PARALLEL}"
@@ -794,16 +858,21 @@ build_librdkafka() {
 
     cd "${TP_SOURCE_DIR}/${LIBRDKAFKA_SOURCE}"
 
+    # NOTE(amos): librdkafka uses a weird autoconf variant (mklove) which doesn't allow extending PKG_CONFIG with spaces in cmd.
+    # As a result, we use a patch to hard code "--static" into PKG_CONFIG instead.
+    # PKG_CONFIG="pkg-config --static"
+
     CPPFLAGS="-I${TP_INCLUDE_DIR}" \
-        LDFLAGS="-L${TP_LIB_DIR}" \
+        LDFLAGS="-L${TP_LIB_DIR} -lssl -lcrypto -lzstd -lz -lsasl2 \
+        -lgssapi_krb5 -lkrb5 -lkrb5support -lk5crypto -lcom_err -lresolv" \
         ./configure --prefix="${TP_INSTALL_DIR}" --enable-static --enable-sasl --disable-c11threads
 
     make -j "${PARALLEL}"
     make install
 
     remove_all_dylib
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/librdkafka.a
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/librdkafka++.a
+    strip_lib librdkafka.a
+    strip_lib librdkafka++.a
 }
 
 # libunixodbc
@@ -813,12 +882,12 @@ build_libunixodbc() {
     cd "${TP_SOURCE_DIR}/${ODBC_SOURCE}"
 
     if [[ "${KERNEL}" != 'Darwin' ]]; then
-        cppflags="-I${TP_INCLUDE_DIR}"
+        cflags="-I${TP_INCLUDE_DIR} -Wno-int-conversion"
     else
-        cppflags="-I${TP_INCLUDE_DIR} -Wno-implicit-function-declaration"
+        cflags="-I${TP_INCLUDE_DIR} -Wno-int-conversion -Wno-implicit-function-declaration"
     fi
 
-    CPPFLAGS="${cppflags}" \
+    CFLAGS="${cflags}" \
         LDFLAGS="-L${TP_LIB_DIR}" \
         ./configure --prefix="${TP_INSTALL_DIR}" --with-included-ltdl --enable-static=yes --enable-shared=no
 
@@ -842,9 +911,11 @@ build_flatbuffers() {
         ldflags=''
     fi
 
-    CXXFLAGS="${warning_class_memaccess}" \
-        LDFLAGS="${ldflags}" \
-        "${CMAKE_CMD}" -G "${GENERATOR}" -DFLATBUFFERS_BUILD_TESTS=OFF ..
+    LDFLAGS="${ldflags}" \
+        "${CMAKE_CMD}" -G "${GENERATOR}" \
+        -DFLATBUFFERS_CXX_FLAGS="${warning_class_memaccess} ${warning_unused_but_set_variable}" \
+        -DFLATBUFFERS_BUILD_TESTS=OFF \
+        ..
 
     "${BUILD_SYSTEM}" -j "${PARALLEL}"
 
@@ -913,9 +984,9 @@ build_arrow() {
     cp -rf ./brotli_ep/src/brotli_ep-install/lib/libbrotlienc-static.a "${TP_INSTALL_DIR}/lib64/libbrotlienc.a"
     cp -rf ./brotli_ep/src/brotli_ep-install/lib/libbrotlidec-static.a "${TP_INSTALL_DIR}/lib64/libbrotlidec.a"
     cp -rf ./brotli_ep/src/brotli_ep-install/lib/libbrotlicommon-static.a "${TP_INSTALL_DIR}/lib64/libbrotlicommon.a"
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libarrow.a
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libjemalloc.a
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libparquet.a
+    strip_lib libarrow.a
+    strip_lib libjemalloc.a
+    strip_lib libparquet.a
 }
 
 # s2
@@ -948,12 +1019,18 @@ build_s2() {
 
     "${BUILD_SYSTEM}" -j "${PARALLEL}"
     "${BUILD_SYSTEM}" install
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libs2.a
+    strip_lib libs2.a
 }
 
 # bitshuffle
 build_bitshuffle() {
     check_if_source_exist "${BITSHUFFLE_SOURCE}"
+    local ld="${DORIS_BIN_UTILS}/ld"
+    local ar="${DORIS_BIN_UTILS}/ar"
+
+    if [[ ! -f "${ld}" ]]; then ld="$(command -v ld)"; fi
+    if [[ ! -f "${ar}" ]]; then ar="$(command -v ar)"; fi
+
     cd "${TP_SOURCE_DIR}/${BITSHUFFLE_SOURCE}"
     PREFIX="${TP_INSTALL_DIR}"
 
@@ -981,14 +1058,24 @@ build_bitshuffle() {
             "src/bitshuffle.c" \
             "src/iochain.c"
         # Merge the object files together to produce a combined .o file.
-        "${DORIS_BIN_UTILS}/ld" -r -o "${tmp_obj}" bitshuffle_core.o bitshuffle.o iochain.o
+        "${ld}" -r -o "${tmp_obj}" bitshuffle_core.o bitshuffle.o iochain.o
         # For the AVX2 symbols, suffix them.
         if [[ "${arch}" == "avx2" ]]; then
+            local nm="${DORIS_BIN_UTILS}/nm"
+            local objcopy="${DORIS_BIN_UTILS}/objcopy"
+
+            if [[ ! -f "${nm}" ]]; then nm="$(command -v nm)"; fi
+            if [[ ! -f "${objcopy}" ]]; then
+                if ! objcopy="$(command -v objcopy)"; then
+                    objcopy="${TP_INSTALL_DIR}/binutils/bin/objcopy"
+                fi
+            fi
+
             # Create a mapping file with '<old_sym> <suffixed_sym>' on each line.
-            "${DORIS_BIN_UTILS}/nm" --defined-only --extern-only "${tmp_obj}" | while read -r addr type sym; do
+            "${nm}" --defined-only --extern-only "${tmp_obj}" | while read -r addr type sym; do
                 echo "${sym} ${sym}_${arch}"
             done >renames.txt
-            "${DORIS_BIN_UTILS}/objcopy" --redefine-syms=renames.txt "${tmp_obj}" "${dst_obj}"
+            "${objcopy}" --redefine-syms=renames.txt "${tmp_obj}" "${dst_obj}"
         else
             mv "${tmp_obj}" "${dst_obj}"
         fi
@@ -997,7 +1084,7 @@ build_bitshuffle() {
     local links
     read -r -a links <<<"${to_link}"
     rm -f libbitshuffle.a
-    "${DORIS_BIN_UTILS}/ar" rs libbitshuffle.a "${links[@]}"
+    "${ar}" rs libbitshuffle.a "${links[@]}"
     mkdir -p "${PREFIX}/include/bitshuffle"
     cp libbitshuffle.a "${PREFIX}"/lib/
     cp "${TP_SOURCE_DIR}/${BITSHUFFLE_SOURCE}/src/bitshuffle.h" "${PREFIX}/include/bitshuffle/bitshuffle.h"
@@ -1092,11 +1179,12 @@ build_orc() {
         -DZSTD_INCLUDE_DIR="${TP_INSTALL_DIR}/include" \
         -DBUILD_LIBHDFSPP=OFF \
         -DBUILD_CPP_TESTS=OFF \
+        -DSTOP_BUILD_ON_WARNING=OFF \
         -DCMAKE_INSTALL_PREFIX="${TP_INSTALL_DIR}"
 
     "${BUILD_SYSTEM}" -j "${PARALLEL}"
     "${BUILD_SYSTEM}" install
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/liborc.a
+    strip_lib liborc.a
 }
 
 #cctz
@@ -1150,28 +1238,28 @@ build_aws_sdk() {
     "${CMAKE_CMD}" -G "${GENERATOR}" -B"${BUILD_DIR}" -DCMAKE_BUILD_TYPE=RelWithDebInfo -DCMAKE_INSTALL_PREFIX="${TP_INSTALL_DIR}" \
         -DCMAKE_PREFIX_PATH="${TP_INSTALL_DIR}" -DBUILD_SHARED_LIBS=OFF -DENABLE_TESTING=OFF \
         -DCURL_LIBRARY_RELEASE="${TP_INSTALL_DIR}/lib/libcurl.a" -DZLIB_LIBRARY_RELEASE="${TP_INSTALL_DIR}/lib/libz.a" \
-        -DBUILD_ONLY="core;s3;s3-crt;transfer" -DCMAKE_CXX_FLAGS="-Wno-nonnull" -DCPP_STANDARD=17
+        -DBUILD_ONLY="core;s3;s3-crt;transfer" -DCMAKE_CXX_FLAGS="-Wno-nonnull -Wno-deprecated-declarations" -DCPP_STANDARD=17
 
     cd "${BUILD_DIR}"
 
     "${BUILD_SYSTEM}" -j "${PARALLEL}"
     "${BUILD_SYSTEM}" install
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libaws-cpp-sdk-s3-crt.a
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libaws-cpp-sdk-s3.a
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libaws-cpp-sdk-core.a
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libs2n.a
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libaws-crt-cpp.a
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libaws-c-http.a
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libaws-c-common.a
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libaws-c-auth.a
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libaws-c-io.a
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libaws-c-mqtt.a
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libaws-c-s3.a
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libaws-c-event-stream.a
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libaws-c-cal.a
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libaws-cpp-sdk-transfer.a
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libaws-checksums.a
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libaws-c-compression.a
+    strip_lib libaws-cpp-sdk-s3-crt.a
+    strip_lib libaws-cpp-sdk-s3.a
+    strip_lib libaws-cpp-sdk-core.a
+    strip_lib libs2n.a
+    strip_lib libaws-crt-cpp.a
+    strip_lib libaws-c-http.a
+    strip_lib libaws-c-common.a
+    strip_lib libaws-c-auth.a
+    strip_lib libaws-c-io.a
+    strip_lib libaws-c-mqtt.a
+    strip_lib libaws-c-s3.a
+    strip_lib libaws-c-event-stream.a
+    strip_lib libaws-c-cal.a
+    strip_lib libaws-cpp-sdk-transfer.a
+    strip_lib libaws-checksums.a
+    strip_lib libaws-c-compression.a
 }
 
 # lzma
@@ -1195,7 +1283,7 @@ build_lzma() {
 
     make -j "${PARALLEL}"
     make install
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/liblzma.a
+    strip_lib liblzma.a
 }
 
 # xml2
@@ -1210,7 +1298,7 @@ build_xml2() {
 
     export ACLOCAL_PATH='/usr/share/aclocal'
 
-    sh autogen.sh
+    sed '/(libtoolize/,/}/d' autogen.sh | bash
     make distclean
 
     mkdir -p "${BUILD_DIR}"
@@ -1222,7 +1310,7 @@ build_xml2() {
 
     make -j "${PARALLEL}"
     make install
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libxml2.a
+    strip_lib libxml2.a
 }
 
 # idn
@@ -1253,7 +1341,9 @@ build_gsasl() {
         cflags='-Wno-implicit-function-declaration'
     fi
 
-    CFLAGS="${cflags}" ../configure --prefix="${TP_INSTALL_DIR}" --with-gssapi-impl=mit --enable-shared=no --with-pic --with-libidn-prefix="${TP_INSTALL_DIR}"
+    KRB5_CONFIG="${TP_INSTALL_DIR}/bin/krb5-config" \
+        CFLAGS="${cflags} -I${TP_INCLUDE_DIR}" \
+        ../configure --prefix="${TP_INSTALL_DIR}" --with-gssapi-impl=mit --enable-shared=no --with-pic --with-libidn-prefix="${TP_INSTALL_DIR}"
 
     make -j "${PARALLEL}"
     make install
@@ -1271,7 +1361,7 @@ build_krb5() {
         with_crypto_impl='--with-crypto-impl=openssl'
     fi
 
-    CFLAGS="-fcommon -I${TP_INSTALL_DIR}/include" LDFLAGS="-L${TP_INSTALL_DIR}/lib" \
+    CFLAGS="-fcommon -fPIC -I${TP_INSTALL_DIR}/include" LDFLAGS="-L${TP_INSTALL_DIR}/lib" \
         ../configure --prefix="${TP_INSTALL_DIR}" --disable-shared --enable-static ${with_crypto_impl:+${with_crypto_impl}}
 
     make -j "${PARALLEL}"
@@ -1287,14 +1377,41 @@ build_hdfs3() {
     cd "${BUILD_DIR}"
     rm -rf ./*
 
-    # build libhdfs3 with kerberos support
-    CPPLAGS="-I${TP_INCLUDE_DIR}" \
-        LDFLAGS="-L${TP_LIB_DIR}" \
-        ../bootstrap --dependency="${TP_INSTALL_DIR}" --prefix="${TP_INSTALL_DIR}" --disable-shared --enable-static
+    if [[ "$(uname -m)" == "x86_64" ]]; then
+        SSE_OPTION='-DENABLE_SSE=ON'
+    else
+        SSE_OPTION='-DENABLE_SSE=OFF'
+    fi
+    cmake -DCMAKE_BUILD_TYPE=RelWithDebInfo -DCMAKE_INSTALL_PREFIX="${TP_INSTALL_DIR}" \
+        -DBUILD_STATIC_LIBS=ON -DBUILD_SHARED_LIBS=OFF -DBUILD_TEST=OFF "${SSE_OPTION}" \
+        -DProtobuf_PROTOC_EXECUTABLE="${TP_INSTALL_DIR}/bin/protoc" \
+        -DProtobuf_INCLUDE_DIR="${TP_INSTALL_DIR}/include" \
+        -DProtobuf_LIBRARIES="${TP_INSTALL_DIR}/lib/libprotoc.a" \
+        -DKERBEROS_INCLUDE_DIRS="${TP_INSTALL_DIR}/include" \
+        -DKERBEROS_LIBRARIES="${TP_INSTALL_DIR}/lib/libkrb5.a" \
+        -DGSASL_INCLUDE_DIR="${TP_INSTALL_DIR}/include" \
+        -DGSASL_LIBRARIES="${TP_INSTALL_DIR}/lib/libgsasl.a" \
+        ..
 
     make CXXFLAGS="${libhdfs_cxx17}" -j "${PARALLEL}"
     make install
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libhdfs3.a
+    strip_lib libhdfs3.a
+}
+
+# jemalloc
+build_jemalloc() {
+    check_if_source_exist "${JEMALLOC_SOURCE}"
+    cd "${TP_SOURCE_DIR}/${JEMALLOC_SOURCE}"
+
+    mkdir -p "${BUILD_DIR}"
+    cd "${BUILD_DIR}"
+
+    cflags='-O3 -fno-omit-frame-pointer -fPIC -g'
+    CFLAGS="${cflags}" ../configure --prefix="${TP_INSTALL_DIR}" --with-jemalloc-prefix=je --enable-prof --disable-cxx --disable-libdl --disable-shared
+
+    make -j "${PARALLEL}"
+    make install
+    mv "${TP_INSTALL_DIR}"/lib/libjemalloc.a "${TP_INSTALL_DIR}"/lib/libjemalloc_doris.a
 }
 
 # benchmark
@@ -1366,13 +1483,13 @@ build_opentelemetry() {
 
     "${BUILD_SYSTEM}" -j "${PARALLEL}"
     "${BUILD_SYSTEM}" install
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libopentelemetry_exporter_zipkin_trace.a
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libopentelemetry_trace.a
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libopentelemetry_proto.a
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libopentelemetry_resources.a
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libopentelemetry_exporter_ostream_span.a
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libopentelemetry_http_client_curl.a
-    strip --strip-debug --strip-unneeded "${TP_LIB_DIR}"/libopentelemetry_exporter_otlp_http_client.a
+    strip_lib libopentelemetry_exporter_zipkin_trace.a
+    strip_lib libopentelemetry_trace.a
+    strip_lib libopentelemetry_proto.a
+    strip_lib libopentelemetry_resources.a
+    strip_lib libopentelemetry_exporter_ostream_span.a
+    strip_lib libopentelemetry_http_client_curl.a
+    strip_lib libopentelemetry_exporter_otlp_http_client.a
 }
 
 # sse2neon
@@ -1382,6 +1499,76 @@ build_sse2neon() {
     cp sse2neon.h "${TP_INSTALL_DIR}/include/"
 }
 
+# xxhash
+build_xxhash() {
+    check_if_source_exist "${XXHASH_SOURCE}"
+    cd "${TP_SOURCE_DIR}/${XXHASH_SOURCE}"
+
+    make -j "${PARALLEL}"
+    cp -r ./*.h "${TP_INSTALL_DIR}/include/"
+    cp libxxhash.a "${TP_INSTALL_DIR}/lib64"
+}
+
+build_binutils() {
+    check_if_source_exist "${BINUTILS_SOURCE}"
+    cd "${TP_SOURCE_DIR}/${BINUTILS_SOURCE}"
+
+    rm -rf "${BUILD_DIR}"
+    mkdir -p "${BUILD_DIR}"
+    cd "${BUILD_DIR}"
+
+    ../configure --prefix="${TP_INSTALL_DIR}/binutils" --includedir="${TP_INCLUDE_DIR}" --libdir="${TP_LIB_DIR}" \
+        --enable-install-libiberty --without-msgpack
+    make -j "${PARALLEL}"
+    make install-bfd install-libiberty install-binutils
+}
+
+build_gettext() {
+    check_if_source_exist "${GETTEXT_SOURCE}"
+    cd "${TP_SOURCE_DIR}/${GETTEXT_SOURCE}"
+
+    rm -rf "${BUILD_DIR}"
+    mkdir -p "${BUILD_DIR}"
+    cd "${BUILD_DIR}"
+
+    ../gettext-runtime/configure --prefix="${TP_INSTALL_DIR}" --disable-java
+    cd intl
+    make -j "${PARALLEL}"
+    make install
+
+    remove_all_dylib
+}
+
+# concurrentqueue
+build_concurrentqueue() {
+    check_if_source_exist "${CONCURRENTQUEUE_SOURCE}"
+    cd "${TP_SOURCE_DIR}/${CONCURRENTQUEUE_SOURCE}"
+    cp ./*.h "${TP_INSTALL_DIR}/include/"
+}
+
+# fast_float
+build_fast_float() {
+    check_if_source_exist "${FAST_FLOAT_SOURCE}"
+    cd "${TP_SOURCE_DIR}/${FAST_FLOAT_SOURCE}"
+    cp -r ./include/fast_float "${TP_INSTALL_DIR}/include/"
+}
+
+# hadoop_libs_x86
+build_hadoop_libs_x86() {
+    check_if_source_exist "${HADOOP_LIBS_X86_SOURCE}"
+    cd "${TP_SOURCE_DIR}/${HADOOP_LIBS_X86_SOURCE}"
+    mkdir -p "${TP_INSTALL_DIR}/include/hadoop_hdfs/"
+    mkdir -p "${TP_INSTALL_DIR}/lib/hadoop_hdfs/"
+    cp ./include/hdfs.h "${TP_INSTALL_DIR}/include/hadoop_hdfs/"
+    cp -r ./* "${TP_INSTALL_DIR}/lib/hadoop_hdfs/"
+}
+
+if [[ "$(uname -s)" == 'Darwin' ]]; then
+    echo 'build for Darwin'
+    build_binutils
+    build_gettext
+fi
+
 build_libunixodbc
 build_openssl
 build_libevent
@@ -1404,7 +1591,9 @@ build_hyperscan
 build_thrift
 build_leveldb
 build_brpc
+build_jemalloc
 build_rocksdb
+build_krb5 # before cyrus_sasl
 build_cyrus_sasl
 build_librdkafka
 build_flatbuffers
@@ -1426,7 +1615,6 @@ build_lzma
 build_xml2
 build_idn
 build_gsasl
-build_krb5
 build_hdfs3
 build_benchmark
 build_simdjson
@@ -1434,5 +1622,12 @@ build_nlohmann_json
 build_opentelemetry
 build_libbacktrace
 build_sse2neon
+build_xxhash
+build_concurrentqueue
+build_fast_float
+
+if [[ "$(uname -m)" == 'x86_64' ]]; then
+    build_hadoop_libs_x86
+fi
 
 echo "Finished to build all thirdparties"
